@@ -142,7 +142,7 @@ impl Agent for ClientSideConnection {
         )
     }
 
-    #[cfg(feature = "unstable")]
+    #[cfg(feature = "unstable_session_model")]
     async fn set_session_model(
         &self,
         args: SetSessionModelRequest,
@@ -155,6 +155,49 @@ impl Agent for ClientSideConnection {
             .await
     }
 
+    #[cfg(feature = "unstable_session_list")]
+    async fn list_sessions(&self, args: ListSessionsRequest) -> Result<ListSessionsResponse> {
+        self.conn
+            .request(
+                AGENT_METHOD_NAMES.session_list,
+                Some(ClientRequest::ListSessionsRequest(args)),
+            )
+            .await
+    }
+
+    #[cfg(feature = "unstable_session_fork")]
+    async fn fork_session(&self, args: ForkSessionRequest) -> Result<ForkSessionResponse> {
+        self.conn
+            .request(
+                AGENT_METHOD_NAMES.session_fork,
+                Some(ClientRequest::ForkSessionRequest(args)),
+            )
+            .await
+    }
+
+    #[cfg(feature = "unstable_session_resume")]
+    async fn resume_session(&self, args: ResumeSessionRequest) -> Result<ResumeSessionResponse> {
+        self.conn
+            .request(
+                AGENT_METHOD_NAMES.session_resume,
+                Some(ClientRequest::ResumeSessionRequest(args)),
+            )
+            .await
+    }
+
+    #[cfg(feature = "unstable_session_config_options")]
+    async fn set_session_config_option(
+        &self,
+        args: SetSessionConfigOptionRequest,
+    ) -> Result<SetSessionConfigOptionResponse> {
+        self.conn
+            .request(
+                AGENT_METHOD_NAMES.session_set_config_option,
+                Some(ClientRequest::SetSessionConfigOptionRequest(args)),
+            )
+            .await
+    }
+
     async fn ext_method(&self, args: ExtRequest) -> Result<ExtResponse> {
         self.conn
             .request(
@@ -514,10 +557,28 @@ impl Side for AgentSide {
             m if m == AGENT_METHOD_NAMES.session_set_mode => serde_json::from_str(params.get())
                 .map(ClientRequest::SetSessionModeRequest)
                 .map_err(Into::into),
-            #[cfg(feature = "unstable")]
+            #[cfg(feature = "unstable_session_model")]
             m if m == AGENT_METHOD_NAMES.session_set_model => serde_json::from_str(params.get())
                 .map(ClientRequest::SetSessionModelRequest)
                 .map_err(Into::into),
+            #[cfg(feature = "unstable_session_list")]
+            m if m == AGENT_METHOD_NAMES.session_list => serde_json::from_str(params.get())
+                .map(ClientRequest::ListSessionsRequest)
+                .map_err(Into::into),
+            #[cfg(feature = "unstable_session_fork")]
+            m if m == AGENT_METHOD_NAMES.session_fork => serde_json::from_str(params.get())
+                .map(ClientRequest::ForkSessionRequest)
+                .map_err(Into::into),
+            #[cfg(feature = "unstable_session_resume")]
+            m if m == AGENT_METHOD_NAMES.session_resume => serde_json::from_str(params.get())
+                .map(ClientRequest::ResumeSessionRequest)
+                .map_err(Into::into),
+            #[cfg(feature = "unstable_session_config_options")]
+            m if m == AGENT_METHOD_NAMES.session_set_config_option => {
+                serde_json::from_str(params.get())
+                    .map(ClientRequest::SetSessionConfigOptionRequest)
+                    .map_err(Into::into)
+            }
             m if m == AGENT_METHOD_NAMES.session_prompt => serde_json::from_str(params.get())
                 .map(ClientRequest::PromptRequest)
                 .map_err(Into::into),
@@ -582,11 +643,31 @@ impl<T: Agent> MessageHandler<AgentSide> for T {
                 let response = self.set_session_mode(args).await?;
                 Ok(AgentResponse::SetSessionModeResponse(response))
             }
-            #[cfg(feature = "unstable")]
+            #[cfg(feature = "unstable_session_model")]
             ClientRequest::SetSessionModelRequest(args) => {
                 let response = self.set_session_model(args).await?;
                 Ok(AgentResponse::SetSessionModelResponse(response))
             }
+            #[cfg(feature = "unstable_session_list")]
+            ClientRequest::ListSessionsRequest(args) => {
+                let response = self.list_sessions(args).await?;
+                Ok(AgentResponse::ListSessionsResponse(response))
+            }
+            #[cfg(feature = "unstable_session_fork")]
+            ClientRequest::ForkSessionRequest(args) => {
+                let response = self.fork_session(args).await?;
+                Ok(AgentResponse::ForkSessionResponse(response))
+            }
+            #[cfg(feature = "unstable_session_resume")]
+            ClientRequest::ResumeSessionRequest(args) => {
+                let response = self.resume_session(args).await?;
+                Ok(AgentResponse::ResumeSessionResponse(response))
+            }
+            #[cfg(feature = "unstable_session_config_options")]
+            ClientRequest::SetSessionConfigOptionRequest(args) => {
+                let response = self.set_session_config_option(args).await?;
+                Ok(AgentResponse::SetSessionConfigOptionResponse(response))
+            }
             ClientRequest::ExtMethodRequest(args) => {
                 let response = self.ext_method(args).await?;
                 Ok(AgentResponse::ExtMethodResponse(response))
