@@ -2,6 +2,8 @@ use agent_client_protocol_schema::{
     AGENT_METHOD_NAMES, AgentSide, CLIENT_METHOD_NAMES, ClientSide, JsonRpcMessage,
     OutgoingMessage, ProtocolVersion,
 };
+#[cfg(feature = "unstable_cancel_request")]
+use agent_client_protocol_schema::{PROTOCOL_LEVEL_METHOD_NAMES, ProtocolLevelNotification};
 use schemars::{
     JsonSchema,
     generate::SchemaSettings,
@@ -24,9 +26,12 @@ struct ClientOutgoingMessage(JsonRpcMessage<OutgoingMessage<ClientSide, AgentSid
 #[expect(dead_code)]
 #[derive(JsonSchema)]
 #[serde(untagged)]
+#[schemars(title = "Agent Client Protocol")]
 enum AcpTypes {
-    AgentOutgoingMessage(AgentOutgoingMessage),
-    ClientOutgoingMessage(ClientOutgoingMessage),
+    Agent(AgentOutgoingMessage),
+    Client(ClientOutgoingMessage),
+    #[cfg(feature = "unstable_cancel_request")]
+    ProtocolLevel(ProtocolLevelNotification),
 }
 
 fn main() {
@@ -38,11 +43,7 @@ fn main() {
         .with_transform(bool_schemas);
 
     let generator = settings.into_generator();
-    let mut schema = generator.into_root_schema_for::<AcpTypes>();
-    {
-        let schema = schema.as_object_mut().unwrap();
-        schema.remove("title");
-    }
+    let schema = generator.into_root_schema_for::<AcpTypes>();
 
     // Convert to serde_json::Value for post-processing
     let schema_value = serde_json::to_value(&schema).unwrap();
@@ -66,11 +67,19 @@ fn main() {
     .expect("Failed to write {schema_file}");
 
     // Create a combined metadata object
+    #[cfg(not(feature = "unstable_cancel_request"))]
     let metadata = serde_json::json!({
         "version": ProtocolVersion::LATEST,
         "agentMethods": AGENT_METHOD_NAMES,
         "clientMethods": CLIENT_METHOD_NAMES,
     });
+    #[cfg(feature = "unstable_cancel_request")]
+    let metadata = serde_json::json!({
+        "version": ProtocolVersion::LATEST,
+        "agentMethods": AGENT_METHOD_NAMES,
+        "clientMethods": CLIENT_METHOD_NAMES,
+        "protocolMethods": PROTOCOL_LEVEL_METHOD_NAMES,
+    });
 
     let meta_file = if cfg!(feature = "unstable") {
         "meta.unstable.json"
@@ -88,7 +97,7 @@ fn main() {
     let markdown_doc = markdown_gen.generate(&schema_value);
 
     let doc_file = if cfg!(feature = "unstable") {
-        "schema.unstable.mdx"
+        "draft/schema.mdx"
     } else {
         "schema.mdx"
     };
@@ -139,6 +148,7 @@ mod markdown_generator {
 
             let mut agent_types: BTreeMap<String, Vec<(String, Value)>> = BTreeMap::new();
             let mut client_types: BTreeMap<String, Vec<(String, Value)>> = BTreeMap::new();
+            let mut protocol_types: BTreeMap<String, Vec<(String, Value)>> = BTreeMap::new();
             let mut referenced_types: Vec<(String, Value)> = Vec::new();
 
             for (name, def) in &self.definitions {
@@ -153,17 +163,17 @@ mod markdown_generator {
                 if let Some(side) = def.get("x-side").and_then(|v| v.as_str()) {
                     let method = def.get("x-method").unwrap().as_str().unwrap();
 
-                    if side == "agent" {
-                        agent_types
-                            .entry(method.to_string())
-                            .or_default()
-                            .push((name.clone(), def.clone()));
-                    } else {
-                        client_types
-                            .entry(method.to_string())
-                            .or_default()
-                            .push((name.clone(), def.clone()));
-                    }
+                    let types = match side {
+                        "agent" => &mut agent_types,
+                        "client" => &mut client_types,
+                        "protocol" => &mut protocol_types,
+                        _ => unimplemented!("Unexpected side {side}"),
+                    };
+
+                    types
+                        .entry(method.to_string())
+                        .or_default()
+                        .push((name.clone(), def.clone()));
                 } else {
                     referenced_types.push((name.clone(), def.clone()));
                 }
@@ -202,6 +212,27 @@ and control access to resources."
             for (method, types) in client_types {
                 self.generate_method(&method, side_docs.client_method_doc(&method), types);
             }
+            #[cfg(feature = "unstable_cancel_request")]
+            {
+                writeln!(&mut self.output, "## Protocol Level").unwrap();
+                writeln!(&mut self.output).unwrap();
+                writeln!(
+            &mut self.output,
+            "Defines the interface that ACP-compliant agents and clients must both implement.
+
+Notifications whose methods start with '$/' are messages which are protocol
+implementation dependent and might not be implementable in all clients or
+agents. For example if the implementation uses a single threaded synchronous
+programming language then there is little it can do to react to a
+`$/cancel_request` notification. If an agent or client receives notifications
+starting with '$/' it is free to ignore the notification."
+        )
+                .unwrap();
+
+                for (method, types) in protocol_types {
+                    self.generate_method(&method, side_docs.protocol_method_doc(&method), types);
+                }
+            }
 
             referenced_types.sort_by_key(|(name, _)| name.clone());
             for (name, def) in referenced_types {
@@ -653,6 +684,41 @@ and control access to resources."
             }
         }
 
+        fn get_ref_type_name(schema: &Value) -> Option<&str> {
+            if let Some(ref_val) = schema.get("$ref").and_then(|v| v.as_str()) {
+                return Some(ref_val.strip_prefix("#/$defs/").unwrap_or(ref_val));
+            }
+
+            // Check for single-item allOf/anyOf/oneOf wrappers (often used for $ref with sibling properties)
+            for key in ["allOf", "anyOf", "oneOf"] {
+                if let Some(arr) = schema.get(key).and_then(|v| v.as_array())
+                    && arr.len() == 1
+                    && let Some(type_name) = Self::get_ref_type_name(&arr[0])
+                {
+                    return Some(type_name);
+                }
+            }
+
+            None
+        }
+
+        fn get_array_type_string(schema: &Value) -> String {
+            if let Some(items) = schema.get("items") {
+                if let Some(type_name) = Self::get_ref_type_name(items) {
+                    return format!(
+                        "<a href=\"#{}\">{}[]</a>",
+                        MarkdownGenerator::anchor_text(type_name),
+                        type_name
+                    );
+                }
+
+                let item_type = MarkdownGenerator::get_type_string(items);
+                format!("<><span>{item_type}</span><span>[]</span></>")
+            } else {
+                "\"array\"".to_string()
+            }
+        }
+
         fn get_type_string(schema: &Value) -> String {
             // Check for $ref
             if let Some(ref_val) = schema.get("$ref").and_then(|v| v.as_str()) {
@@ -677,14 +743,7 @@ and control access to resources."
             if let Some(type_val) = schema.get("type") {
                 if let Some(type_str) = type_val.as_str() {
                     return match type_str {
-                        "array" => {
-                            if let Some(items) = schema.get("items") {
-                                let item_type = Self::get_type_string(items);
-                                format!("<><span>{item_type}</span><span>[]</span></>")
-                            } else {
-                                "\"array\"".to_string()
-                            }
-                        }
+                        "array" => Self::get_array_type_string(schema),
                         "integer" => {
                             let type_str = if let Some(format) =
                                 schema.get("format").and_then(|v| v.as_str())
@@ -701,13 +760,26 @@ and control access to resources."
 
                 // Handle multiple types (nullable)
                 if let Some(arr) = type_val.as_array() {
-                    let types: Vec<String> = arr
-                        .iter()
-                        .filter_map(|v| v.as_str().map(ToString::to_string))
-                        .collect();
-                    if !types.is_empty() {
-                        return format!("\"{}\"", types.join(" | "));
+                    let types: Vec<&str> = arr.iter().filter_map(|v| v.as_str()).collect();
+                    if types.is_empty() {
+                        return "\"object\"".to_string();
                     }
+
+                    // Special-case nullable arrays so we can still show the item type (and link to it).
+                    if types.contains(&"array") && schema.get("items").is_some() {
+                        let array_type = Self::get_array_type_string(schema);
+                        let rest: Vec<&str> =
+                            types.iter().copied().filter(|t| *t != "array").collect();
+                        if rest.is_empty() {
+                            return array_type;
+                        }
+                        let rest_text = rest.join(" | ");
+                        return format!(
+                            "<><span>{array_type}</span><span> | {rest_text}</span></>"
+                        );
+                    }
+
+                    return format!("\"{}\"", types.join(" | "));
                 }
             }
 
@@ -728,11 +800,8 @@ and control access to resources."
                             other_type = Some(t);
                         }
                     }
-                    if has_null && other_type.is_some() {
-                        return format!(
-                            "<><span>{}</span><span> | null</span></>",
-                            other_type.unwrap()
-                        );
+                    if has_null && let Some(other_type) = other_type {
+                        return format!("<><span>{other_type}</span><span> | null</span></>");
                     }
                 }
                 return "union".to_string();
@@ -747,9 +816,13 @@ and control access to resources."
         }
 
         fn get_inline_variant_type(variant: &Value) -> Option<String> {
+            if variant.get("oneOf").is_some() || variant.get("anyOf").is_some() {
+                return None;
+            }
+
             // Check for simple type
-            if let Some(type_str) = variant.get("type").and_then(|v| v.as_str()) {
-                return Some(format!("\"{type_str}\""));
+            if variant.get("type").and_then(|v| v.as_str()).is_some() {
+                return Some(Self::get_type_string(variant));
             }
             // Check for $ref
             if let Some(ref_val) = variant.get("$ref").and_then(|v| v.as_str()) {
@@ -803,24 +876,30 @@ and control access to resources."
         }
     }
 
+    #[derive(Default)]
     struct SideDocs {
-        agent_methods: HashMap<String, String>,
-        client_methods: HashMap<String, String>,
+        agent: HashMap<String, String>,
+        client: HashMap<String, String>,
+        protocol: HashMap<String, String>,
     }
 
     impl SideDocs {
         fn agent_method_doc(&self, method_name: &str) -> &String {
             match method_name {
-                "initialize" => self.agent_methods.get("InitializeRequest").unwrap(),
-                "authenticate" => self.agent_methods.get("AuthenticateRequest").unwrap(),
-                "session/new" => self.agent_methods.get("NewSessionRequest").unwrap(),
-                "session/load" => self.agent_methods.get("LoadSessionRequest").unwrap(),
-                "session/list" => self.agent_methods.get("ListSessionsRequest").unwrap(),
-                "session/fork" => self.agent_methods.get("ForkSessionRequest").unwrap(),
-                "session/set_mode" => self.agent_methods.get("SetSessionModeRequest").unwrap(),
-                "session/prompt" => self.agent_methods.get("PromptRequest").unwrap(),
-                "session/cancel" => self.agent_methods.get("CancelNotification").unwrap(),
-                "session/set_model" => self.agent_methods.get("SetSessionModelRequest").unwrap(),
+                "initialize" => self.agent.get("InitializeRequest").unwrap(),
+                "authenticate" => self.agent.get("AuthenticateRequest").unwrap(),
+                "session/new" => self.agent.get("NewSessionRequest").unwrap(),
+                "session/load" => self.agent.get("LoadSessionRequest").unwrap(),
+                "session/list" => self.agent.get("ListSessionsRequest").unwrap(),
+                "session/fork" => self.agent.get("ForkSessionRequest").unwrap(),
+                "session/resume" => self.agent.get("ResumeSessionRequest").unwrap(),
+                "session/set_mode" => self.agent.get("SetSessionModeRequest").unwrap(),
+                "session/set_config_option" => {
+                    self.agent.get("SetSessionConfigOptionRequest").unwrap()
+                }
+                "session/prompt" => self.agent.get("PromptRequest").unwrap(),
+                "session/cancel" => self.agent.get("CancelNotification").unwrap(),
+                "session/set_model" => self.agent.get("SetSessionModelRequest").unwrap(),
                 _ => panic!("Introduced a method? Add it here :)"),
             }
         }
@@ -828,22 +907,24 @@ and control access to resources."
         fn client_method_doc(&self, method_name: &str) -> &String {
             match method_name {
                 "session/request_permission" => {
-                    self.client_methods.get("RequestPermissionRequest").unwrap()
+                    self.client.get("RequestPermissionRequest").unwrap()
                 }
-                "fs/write_text_file" => self.client_methods.get("WriteTextFileRequest").unwrap(),
-                "fs/read_text_file" => self.client_methods.get("ReadTextFileRequest").unwrap(),
-                "session/update" => self.client_methods.get("SessionNotification").unwrap(),
-                "terminal/create" => self.client_methods.get("CreateTerminalRequest").unwrap(),
-                "terminal/output" => self.client_methods.get("TerminalOutputRequest").unwrap(),
-                "terminal/release" => self.client_methods.get("ReleaseTerminalRequest").unwrap(),
-                "terminal/wait_for_exit" => self
-                    .client_methods
-                    .get("WaitForTerminalExitRequest")
-                    .unwrap(),
-                "terminal/kill" => self
-                    .client_methods
-                    .get("KillTerminalCommandRequest")
-                    .unwrap(),
+                "fs/write_text_file" => self.client.get("WriteTextFileRequest").unwrap(),
+                "fs/read_text_file" => self.client.get("ReadTextFileRequest").unwrap(),
+                "session/update" => self.client.get("SessionNotification").unwrap(),
+                "terminal/create" => self.client.get("CreateTerminalRequest").unwrap(),
+                "terminal/output" => self.client.get("TerminalOutputRequest").unwrap(),
+                "terminal/release" => self.client.get("ReleaseTerminalRequest").unwrap(),
+                "terminal/wait_for_exit" => self.client.get("WaitForTerminalExitRequest").unwrap(),
+                "terminal/kill" => self.client.get("KillTerminalCommandRequest").unwrap(),
+                _ => panic!("Introduced a method? Add it here :)"),
+            }
+        }
+
+        #[cfg(feature = "unstable_cancel_request")]
+        fn protocol_method_doc(&self, method_name: &str) -> &String {
+            match method_name {
+                "$/cancel_request" => self.protocol.get("CancelRequestNotification").unwrap(),
                 _ => panic!("Introduced a method? Add it here :)"),
             }
         }
@@ -877,10 +958,7 @@ and control access to resources."
         let json_content = fs::read_to_string(json_path).unwrap();
         let doc: Value = serde_json::from_str(&json_content).unwrap();
 
-        let mut side_docs = SideDocs {
-            agent_methods: HashMap::new(),
-            client_methods: HashMap::new(),
-        };
+        let mut side_docs = SideDocs::default();
 
         if let Some(index) = doc["index"].as_object() {
             for (_, item) in index {
@@ -891,7 +969,7 @@ and control access to resources."
                         if let Some(variant) = doc["index"][variant_id.to_string()].as_object()
                             && let Some(name) = variant["name"].as_str()
                         {
-                            side_docs.agent_methods.insert(
+                            side_docs.agent.insert(
                                 name.to_string(),
                                 variant["docs"].as_str().unwrap_or_default().to_string(),
                             );
@@ -906,7 +984,7 @@ and control access to resources."
                         if let Some(variant) = doc["index"][variant_id.to_string()].as_object()
                             && let Some(name) = variant["name"].as_str()
                         {
-                            side_docs.agent_methods.insert(
+                            side_docs.agent.insert(
                                 name.to_string(),
                                 variant["docs"].as_str().unwrap_or_default().to_string(),
                             );
@@ -921,7 +999,7 @@ and control access to resources."
                         if let Some(variant) = doc["index"][variant_id.to_string()].as_object()
                             && let Some(name) = variant["name"].as_str()
                         {
-                            side_docs.client_methods.insert(
+                            side_docs.client.insert(
                                 name.to_string(),
                                 variant["docs"].as_str().unwrap_or_default().to_string(),
                             );
@@ -936,7 +1014,22 @@ and control access to resources."
                         if let Some(variant) = doc["index"][variant_id.to_string()].as_object()
                             && let Some(name) = variant["name"].as_str()
                         {
-                            side_docs.client_methods.insert(
+                            side_docs.client.insert(
+                                name.to_string(),
+                                variant["docs"].as_str().unwrap_or_default().to_string(),
+                            );
+                        }
+                    }
+                }
+
+                if item["name"].as_str() == Some("ProtocolLevelNotification")
+                    && let Some(variants) = item["inner"]["enum"]["variants"].as_array()
+                {
+                    for variant_id in variants {
+                        if let Some(variant) = doc["index"][variant_id.to_string()].as_object()
+                            && let Some(name) = variant["name"].as_str()
+                        {
+                            side_docs.protocol.insert(
                                 name.to_string(),
                                 variant["docs"].as_str().unwrap_or_default().to_string(),
                             );
