@@ -92,6 +92,20 @@ impl Error {
         ErrorCode::InternalError.into()
     }
 
+    /// **UNSTABLE**
+    ///
+    /// This capability is not part of the spec yet, and may be removed or changed at any point.
+    ///
+    /// Request was cancelled.
+    ///
+    /// Execution of the method was aborted either due to a cancellation request from the caller
+    /// or because of resource constraints or shutdown.
+    #[cfg(feature = "unstable_cancel_request")]
+    #[must_use]
+    pub fn request_cancelled() -> Self {
+        ErrorCode::RequestCancelled.into()
+    }
+
     /// Authentication required.
     #[must_use]
     pub fn auth_required() -> Self {
@@ -241,6 +255,7 @@ fn error_code_transform(schema: &mut Schema) {
         .as_str()
         .expect("Unexpected type for description")
         .to_owned();
+    schema.insert("title".into(), code.to_string().into());
     description.insert_str(0, &format!("**{code}**: "));
     schema.insert("description".into(), description.into());
     schema.insert("const".into(), i32::from(code).into());
