@@ -189,7 +189,6 @@
 #![cfg_attr(not(feature = "std"), no_std)]
 #![cfg_attr(feature = "unstable", feature(maybe_uninit_slice))]
 #![cfg_attr(feature = "unstable", feature(maybe_uninit_write_slice))]
-#![cfg_attr(feature = "unstable", feature(one_sided_range))]
 #![warn(missing_debug_implementations)]
 #![warn(missing_docs)]
 #![warn(unreachable_pub)]
@@ -259,11 +258,11 @@ const fn sub_mod(x: usize, y: usize, m: usize) -> usize {
 #[inline]
 const unsafe fn slice_assume_init_ref<T>(slice: &[MaybeUninit<T>]) -> &[T] {
     #[cfg(feature = "unstable")]
-    {
+    unsafe {
         slice.assume_init_ref()
     }
     #[cfg(not(feature = "unstable"))]
-    {
+    unsafe {
         &*(slice as *const [MaybeUninit<T>] as *const [T])
     }
 }
@@ -271,11 +270,11 @@ const unsafe fn slice_assume_init_ref<T>(slice: &[MaybeUninit<T>]) -> &[T] {
 #[inline]
 unsafe fn slice_assume_init_mut<T>(slice: &mut [MaybeUninit<T>]) -> &mut [T] {
     #[cfg(feature = "unstable")]
-    {
+    unsafe {
         slice.assume_init_mut()
     }
     #[cfg(not(feature = "unstable"))]
-    {
+    unsafe {
         &mut *(slice as *mut [MaybeUninit<T>] as *mut [T])
     }
 }
