@@ -5,108 +5,17 @@ use crate::diagnostic::{LabelStyle, Severity};
 use crate::files::{Error, Location};
 use crate::term::{Chars, Config};
 
-#[cfg(feature = "std")]
-pub use std::io::Write as GeneralWrite;
+#[cfg(feature = "termcolor")]
+use {
+    crate::term::Styles,
+    termcolor::{ColorSpec, WriteColor},
+};
 
 #[cfg(feature = "std")]
-pub type GeneralWriteResult = std::io::Result<()>;
+use std::io::{self, Write};
 
 #[cfg(not(feature = "std"))]
-pub use core::fmt::Write as GeneralWrite;
-
-#[cfg(not(feature = "std"))]
-pub use core::fmt::Result as GeneralWriteResult;
-
-/// A writer that can apply styling for different parts of a diagnostic renderer.
-///
-/// # Implementations
-///
-/// - [`PlainWriter<W>`](PlainWriter) - no-op styling, plain text output
-/// - [`StylesWriter<W>`](crate::term::StylesWriter) - custom styles (requires `termcolor` feature)
-/// - `T: WriteColor` - blanket impl using default styles (requires `termcolor` feature)
-pub trait WriteStyle: GeneralWrite {
-    fn set_header(&mut self, severity: Severity) -> GeneralWriteResult;
-
-    fn set_header_message(&mut self) -> GeneralWriteResult;
-
-    fn set_line_number(&mut self) -> GeneralWriteResult;
-
-    fn set_note_bullet(&mut self) -> GeneralWriteResult;
-
-    fn set_source_border(&mut self) -> GeneralWriteResult;
-
-    fn set_label(&mut self, severity: Severity, label_style: LabelStyle) -> GeneralWriteResult;
-
-    fn reset(&mut self) -> GeneralWriteResult;
-}
-
-/// A [`WriteStyle`] implementation that ignores all styling calls. useful for non color output.
-/// Available on all targets
-pub struct PlainWriter<W: GeneralWrite> {
-    w: W,
-}
-
-impl<W: GeneralWrite> PlainWriter<W> {
-    pub fn new(writer: W) -> Self {
-        Self { w: writer }
-    }
-}
-
-#[cfg(feature = "std")]
-impl<W: std::io::Write> std::io::Write for PlainWriter<W> {
-    fn write(&mut self, buf: &[u8]) -> std::io::Result<usize> {
-        self.w.write(buf)
-    }
-
-    fn flush(&mut self) -> std::io::Result<()> {
-        self.w.flush()
-    }
-}
-
-#[cfg(not(feature = "std"))]
-impl<W: core::fmt::Write> core::fmt::Write for PlainWriter<W> {
-    fn write_str(&mut self, s: &str) -> core::fmt::Result {
-        self.w.write_str(s)
-    }
-
-    fn write_char(&mut self, c: char) -> core::fmt::Result {
-        self.w.write_char(c)
-    }
-
-    fn write_fmt(&mut self, args: core::fmt::Arguments<'_>) -> core::fmt::Result {
-        self.w.write_fmt(args)
-    }
-}
-
-impl<W: GeneralWrite> WriteStyle for PlainWriter<W> {
-    fn set_header(&mut self, _severity: Severity) -> GeneralWriteResult {
-        Ok(())
-    }
-
-    fn set_header_message(&mut self) -> GeneralWriteResult {
-        Ok(())
-    }
-
-    fn set_line_number(&mut self) -> GeneralWriteResult {
-        Ok(())
-    }
-
-    fn set_note_bullet(&mut self) -> GeneralWriteResult {
-        Ok(())
-    }
-
-    fn set_source_border(&mut self) -> GeneralWriteResult {
-        Ok(())
-    }
-
-    fn set_label(&mut self, _severity: Severity, _label_style: LabelStyle) -> GeneralWriteResult {
-        Ok(())
-    }
-
-    fn reset(&mut self) -> GeneralWriteResult {
-        Ok(())
-    }
-}
+use core::fmt::{Arguments, Write};
 
 /// The 'location focus' of a source code snippet.
 pub struct Locus {
@@ -137,7 +46,7 @@ pub enum MultiLabel<'diagnostic> {
     /// Can also be rendered at the beginning of the line
     /// if there is only whitespace before the label starts.
     ///
-    /// ```text
+    /// /// ```text
     /// ╭
     /// ```
     Top(usize),
@@ -209,16 +118,20 @@ type Underline = (LabelStyle, VerticalBound);
 ///         empty ── │
 /// ```
 ///
-/// > Filler text from <http://www.cupcakeipsum.com>
+/// Filler text from http://www.cupcakeipsum.com
 pub struct Renderer<'writer, 'config> {
-    writer: &'writer mut dyn WriteStyle,
+    #[cfg(feature = "termcolor")]
+    writer: &'writer mut dyn WriteColor,
+    #[cfg(not(feature = "termcolor"))]
+    writer: &'writer mut dyn Write,
     config: &'config Config,
 }
 
 impl<'writer, 'config> Renderer<'writer, 'config> {
     /// Construct a renderer from the given writer and config.
     pub fn new(
-        writer: &'writer mut dyn WriteStyle,
+        #[cfg(feature = "termcolor")] writer: &'writer mut dyn WriteColor,
+        #[cfg(not(feature = "termcolor"))] writer: &'writer mut dyn Write,
         config: &'config Config,
     ) -> Renderer<'writer, 'config> {
         Renderer { writer, config }
@@ -228,6 +141,11 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
         &self.config.chars
     }
 
+    #[cfg(feature = "termcolor")]
+    fn styles(&self) -> &'config Styles {
+        &self.config.styles
+    }
+
     /// Diagnostic header, with severity, code, and message.
     ///
     /// ```text
@@ -255,7 +173,8 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
         // ```text
         // error
         // ```
-        self.set_header(severity)?;
+        #[cfg(feature = "termcolor")]
+        self.set_color(self.styles().header(severity))?;
         match severity {
             Severity::Bug => write!(self, "bug")?,
             Severity::Error => write!(self, "error")?,
@@ -270,7 +189,7 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
         // [E0001]
         // ```
         if let Some(code) = &code.filter(|code| !code.is_empty()) {
-            write!(self, "[{code}]")?;
+            write!(self, "[{}]", code)?;
         }
 
         // Write diagnostic message
@@ -278,8 +197,10 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
         // ```text
         // : unexpected type in `+` application
         // ```
-        self.set_header_message()?;
-        write!(self, ": {message}")?;
+        #[cfg(feature = "termcolor")]
+        self.set_color(&self.styles().header_message)?;
+        write!(self, ": {}", message)?;
+        #[cfg(feature = "termcolor")]
         self.reset()?;
 
         writeln!(self)?;
@@ -305,8 +226,10 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
     ) -> Result<(), Error> {
         self.outer_gutter(outer_padding)?;
 
-        self.set_source_border()?;
+        #[cfg(feature = "termcolor")]
+        self.set_color(&self.styles().source_border)?;
         write!(self, "{}", self.chars().snippet_start)?;
+        #[cfg(feature = "termcolor")]
         self.reset()?;
 
         write!(self, " ")?;
@@ -390,19 +313,22 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
 
                 // Set the source color if we are in a primary label
                 if is_primary && !in_primary {
-                    self.set_label(severity, LabelStyle::Primary)?;
+                    #[cfg(feature = "termcolor")]
+                    self.set_color(self.styles().label(severity, LabelStyle::Primary))?;
                     in_primary = true;
                 } else if !is_primary && in_primary {
+                    #[cfg(feature = "termcolor")]
                     self.reset()?;
                     in_primary = false;
                 }
 
                 match ch {
                     '\t' => (0..metrics.unicode_width).try_for_each(|_| write!(self, " "))?,
-                    _ => write!(self, "{ch}")?,
+                    _ => write!(self, "{}", ch)?,
                 }
             }
             if in_primary {
+                #[cfg(feature = "termcolor")]
                 self.reset()?;
             }
             writeln!(self)?;
@@ -524,17 +450,20 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
                 let current_label_style = single_labels
                     .iter()
                     .filter(|(_, range, _)| is_overlapping(range, &column_range))
-                    .map(|(label_style, _, _)| label_style.clone())
+                    .map(|(label_style, _, _)| *label_style)
                     .max_by_key(label_priority_key);
 
                 // Update writer style if necessary
                 if previous_label_style != current_label_style {
                     match current_label_style {
                         None => {
+                            #[cfg(feature = "termcolor")]
                             self.reset()?;
                         }
+                        #[cfg_attr(not(feature = "termcolor"), allow(unused))]
                         Some(label_style) => {
-                            self.set_label(severity, label_style)?;
+                            #[cfg(feature = "termcolor")]
+                            self.set_color(self.styles().label(severity, label_style))?;
                         }
                     }
                 }
@@ -548,20 +477,24 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
                 };
                 if let Some(caret_ch) = caret_ch {
                     // FIXME: improve rendering of carets between character boundaries
-                    (0..metrics.unicode_width).try_for_each(|_| write!(self, "{caret_ch}",))?;
+                    (0..metrics.unicode_width).try_for_each(|_| write!(self, "{}", caret_ch))?;
                 }
 
                 previous_label_style = current_label_style;
             }
             // Reset style if it was previously set
             if previous_label_style.is_some() {
+                #[cfg(feature = "termcolor")]
                 self.reset()?;
             }
             // Write first trailing label message
+            #[cfg_attr(not(feature = "termcolor"), allow(unused))]
             if let Some((_, (label_style, _, message))) = trailing_label {
                 write!(self, " ")?;
-                self.set_label(severity, *label_style)?;
-                write!(self, "{message}",)?;
+                #[cfg(feature = "termcolor")]
+                self.set_color(self.styles().label(severity, *label_style))?;
+                write!(self, "{}", message)?;
+                #[cfg(feature = "termcolor")]
                 self.reset()?;
             }
             writeln!(self)?;
@@ -600,6 +533,7 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
                 //   │     first borrow later used by call
                 //   │     help: some help here
                 // ```
+                #[cfg_attr(not(feature = "termcolor"), allow(unused))]
                 for (label_style, range, message) in
                     hanging_labels(single_labels, trailing_label).rev()
                 {
@@ -616,8 +550,10 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
                             .char_indices()
                             .take_while(|(byte_index, _)| *byte_index < range.start),
                     )?;
-                    self.set_label(severity, *label_style)?;
-                    write!(self, "{message}",)?;
+                    #[cfg(feature = "termcolor")]
+                    self.set_color(self.styles().label(severity, *label_style))?;
+                    write!(self, "{}", message)?;
+                    #[cfg(feature = "termcolor")]
                     self.reset()?;
                     writeln!(self)?;
                 }
@@ -666,7 +602,7 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
                             }
                             MultiLabel::Top(..) if multi_label_index == *i => {
                                 underline = Some((*ls, VerticalBound::Top));
-                                self.label_multi_top_left(severity, label_style)?;
+                                self.label_multi_top_left(severity, label_style)?
                             }
                             MultiLabel::Bottom(..) if multi_label_index == *i => {
                                 underline = Some((*ls, VerticalBound::Bottom));
@@ -686,7 +622,7 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
             match bottom_message {
                 None => self.label_multi_top_caret(severity, label_style, source, *range)?,
                 Some(message) => {
-                    self.label_multi_bottom_caret(severity, label_style, source, *range, message)?;
+                    self.label_multi_bottom_caret(severity, label_style, source, *range, message)?
                 }
             }
         }
@@ -747,14 +683,16 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
             self.outer_gutter(outer_padding)?;
             match note_line_index {
                 0 => {
-                    self.set_note_bullet()?;
+                    #[cfg(feature = "termcolor")]
+                    self.set_color(&self.styles().note_bullet)?;
                     write!(self, "{}", self.chars().note_bullet)?;
+                    #[cfg(feature = "termcolor")]
                     self.reset()?;
                 }
                 _ => write!(self, " ")?,
             }
             // Write line of message
-            writeln!(self, " {line}",)?;
+            writeln!(self, " {}", line)?;
         }
 
         Ok(())
@@ -811,8 +749,15 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
         line_number: usize,
         outer_padding: usize,
     ) -> Result<(), Error> {
-        self.set_line_number()?;
-        write!(self, "{line_number: >outer_padding$}",)?;
+        #[cfg(feature = "termcolor")]
+        self.set_color(&self.styles().line_number)?;
+        write!(
+            self,
+            "{line_number: >width$}",
+            line_number = line_number,
+            width = outer_padding,
+        )?;
+        #[cfg(feature = "termcolor")]
         self.reset()?;
         write!(self, " ")?;
         Ok(())
@@ -820,16 +765,20 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
 
     /// The left-hand border of a source line.
     fn border_left(&mut self) -> Result<(), Error> {
-        self.set_source_border()?;
+        #[cfg(feature = "termcolor")]
+        self.set_color(&self.styles().source_border)?;
         write!(self, "{}", self.chars().source_border_left)?;
+        #[cfg(feature = "termcolor")]
         self.reset()?;
         Ok(())
     }
 
     /// The broken left-hand border of a source line.
     fn border_left_break(&mut self) -> Result<(), Error> {
-        self.set_source_border()?;
+        #[cfg(feature = "termcolor")]
+        self.set_color(&self.styles().source_border)?;
         write!(self, "{}", self.chars().source_border_left_break)?;
+        #[cfg(feature = "termcolor")]
         self.reset()?;
         Ok(())
     }
@@ -837,7 +786,7 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
     /// Write vertical lines pointing to carets.
     fn caret_pointers(
         &mut self,
-        severity: Severity,
+        #[cfg_attr(not(feature = "termcolor"), allow(unused))] severity: Severity,
         max_label_start: usize,
         single_labels: &[SingleLabel<'_>],
         trailing_label: Option<(usize, &SingleLabel<'_>)>,
@@ -847,14 +796,17 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
             let column_range = metrics.byte_index..(metrics.byte_index + ch.len_utf8());
             let label_style = hanging_labels(single_labels, trailing_label)
                 .filter(|(_, range, _)| column_range.contains(&range.start))
-                .map(|(label_style, _, _)| label_style.clone())
+                .map(|(label_style, _, _)| *label_style)
                 .max_by_key(label_priority_key);
 
             let mut spaces = match label_style {
                 None => 0..metrics.unicode_width,
+                #[cfg_attr(not(feature = "termcolor"), allow(unused))]
                 Some(label_style) => {
-                    self.set_label(severity, label_style)?;
+                    #[cfg(feature = "termcolor")]
+                    self.set_color(self.styles().label(severity, label_style))?;
                     write!(self, "{}", self.chars().pointer_left)?;
+                    #[cfg(feature = "termcolor")]
                     self.reset()?;
                     1..metrics.unicode_width
                 }
@@ -875,21 +827,26 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
     /// ```
     fn label_multi_left(
         &mut self,
-        severity: Severity,
-        label_style: LabelStyle,
+        #[cfg_attr(not(feature = "termcolor"), allow(unused))] severity: Severity,
+        #[cfg_attr(not(feature = "termcolor"), allow(unused))] label_style: LabelStyle,
         underline: Option<LabelStyle>,
     ) -> Result<(), Error> {
         match underline {
             None => write!(self, " ")?,
             // Continue an underline horizontally
+            #[cfg_attr(not(feature = "termcolor"), allow(unused))]
             Some(label_style) => {
-                self.set_label(severity, label_style)?;
+                #[cfg(feature = "termcolor")]
+                self.set_color(self.styles().label(severity, label_style))?;
                 write!(self, "{}", self.chars().multi_top)?;
+                #[cfg(feature = "termcolor")]
                 self.reset()?;
             }
         }
-        self.set_label(severity, label_style)?;
+        #[cfg(feature = "termcolor")]
+        self.set_color(self.styles().label(severity, label_style))?;
         write!(self, "{}", self.chars().multi_left)?;
+        #[cfg(feature = "termcolor")]
         self.reset()?;
         Ok(())
     }
@@ -901,12 +858,14 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
     /// ```
     fn label_multi_top_left(
         &mut self,
-        severity: Severity,
-        label_style: LabelStyle,
+        #[cfg_attr(not(feature = "termcolor"), allow(unused))] severity: Severity,
+        #[cfg_attr(not(feature = "termcolor"), allow(unused))] label_style: LabelStyle,
     ) -> Result<(), Error> {
         write!(self, " ")?;
-        self.set_label(severity, label_style)?;
+        #[cfg(feature = "termcolor")]
+        self.set_color(self.styles().label(severity, label_style))?;
         write!(self, "{}", self.chars().multi_top_left)?;
+        #[cfg(feature = "termcolor")]
         self.reset()?;
         Ok(())
     }
@@ -918,12 +877,14 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
     /// ```
     fn label_multi_bottom_left(
         &mut self,
-        severity: Severity,
-        label_style: LabelStyle,
+        #[cfg_attr(not(feature = "termcolor"), allow(unused))] severity: Severity,
+        #[cfg_attr(not(feature = "termcolor"), allow(unused))] label_style: LabelStyle,
     ) -> Result<(), Error> {
         write!(self, " ")?;
-        self.set_label(severity, label_style)?;
+        #[cfg(feature = "termcolor")]
+        self.set_color(self.styles().label(severity, label_style))?;
         write!(self, "{}", self.chars().multi_bottom_left)?;
+        #[cfg(feature = "termcolor")]
         self.reset()?;
         Ok(())
     }
@@ -935,12 +896,13 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
     /// ```
     fn label_multi_top_caret(
         &mut self,
-        severity: Severity,
+        #[cfg_attr(not(feature = "termcolor"), allow(unused))] severity: Severity,
         label_style: LabelStyle,
         source: &str,
         start: usize,
     ) -> Result<(), Error> {
-        self.set_label(severity, label_style)?;
+        #[cfg(feature = "termcolor")]
+        self.set_color(self.styles().label(severity, label_style))?;
 
         for (metrics, _) in self
             .char_metrics(source.char_indices())
@@ -955,7 +917,8 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
             LabelStyle::Primary => self.config.chars.multi_primary_caret_start,
             LabelStyle::Secondary => self.config.chars.multi_secondary_caret_start,
         };
-        write!(self, "{caret_start}",)?;
+        write!(self, "{}", caret_start)?;
+        #[cfg(feature = "termcolor")]
         self.reset()?;
         writeln!(self)?;
         Ok(())
@@ -968,13 +931,14 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
     /// ```
     fn label_multi_bottom_caret(
         &mut self,
-        severity: Severity,
+        #[cfg_attr(not(feature = "termcolor"), allow(unused))] severity: Severity,
         label_style: LabelStyle,
         source: &str,
         start: usize,
         message: &str,
     ) -> Result<(), Error> {
-        self.set_label(severity, label_style)?;
+        #[cfg(feature = "termcolor")]
+        self.set_color(self.styles().label(severity, label_style))?;
 
         for (metrics, _) in self
             .char_metrics(source.char_indices())
@@ -989,10 +953,11 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
             LabelStyle::Primary => self.config.chars.multi_primary_caret_start,
             LabelStyle::Secondary => self.config.chars.multi_secondary_caret_start,
         };
-        write!(self, "{caret_end}")?;
+        write!(self, "{}", caret_end)?;
         if !message.is_empty() {
-            write!(self, " {message}")?;
+            write!(self, " {}", message)?;
         }
+        #[cfg(feature = "termcolor")]
         self.reset()?;
         writeln!(self)?;
         Ok(())
@@ -1001,18 +966,21 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
     /// Writes an empty gutter space, or continues an underline horizontally.
     fn inner_gutter_column(
         &mut self,
-        severity: Severity,
+        #[cfg_attr(not(feature = "termcolor"), allow(unused))] severity: Severity,
         underline: Option<Underline>,
     ) -> Result<(), Error> {
         match underline {
             None => self.inner_gutter_space(),
+            #[cfg_attr(not(feature = "termcolor"), allow(unused))]
             Some((label_style, vertical_bound)) => {
-                self.set_label(severity, label_style)?;
+                #[cfg(feature = "termcolor")]
+                self.set_color(self.styles().label(severity, label_style))?;
                 let ch = match vertical_bound {
                     VerticalBound::Top => self.config.chars.multi_top,
                     VerticalBound::Bottom => self.config.chars.multi_bottom,
                 };
-                write!(self, "{ch}{ch}")?;
+                write!(self, "{0}{0}", ch)?;
+                #[cfg(feature = "termcolor")]
                 self.reset()?;
                 Ok(())
             }
@@ -1053,19 +1021,8 @@ impl<'writer, 'config> Renderer<'writer, 'config> {
     }
 }
 
-#[cfg(feature = "std")]
-impl std::io::Write for Renderer<'_, '_> {
-    fn write(&mut self, buf: &[u8]) -> std::io::Result<usize> {
-        self.writer.write(buf)
-    }
-
-    fn flush(&mut self) -> std::io::Result<()> {
-        self.writer.flush()
-    }
-}
-
 #[cfg(not(feature = "std"))]
-impl core::fmt::Write for Renderer<'_, '_> {
+impl Write for Renderer<'_, '_> {
     fn write_str(&mut self, s: &str) -> core::fmt::Result {
         self.writer.write_str(s)
     }
@@ -1074,38 +1031,39 @@ impl core::fmt::Write for Renderer<'_, '_> {
         self.writer.write_char(c)
     }
 
-    fn write_fmt(&mut self, args: core::fmt::Arguments<'_>) -> core::fmt::Result {
+    fn write_fmt(&mut self, args: Arguments<'_>) -> core::fmt::Result {
         self.writer.write_fmt(args)
     }
 }
 
-impl WriteStyle for Renderer<'_, '_> {
-    fn set_header(&mut self, severity: Severity) -> GeneralWriteResult {
-        self.writer.set_header(severity)
+#[cfg(feature = "std")]
+impl Write for Renderer<'_, '_> {
+    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
+        self.writer.write(buf)
     }
 
-    fn set_header_message(&mut self) -> GeneralWriteResult {
-        self.writer.set_header_message()
+    fn flush(&mut self) -> io::Result<()> {
+        self.writer.flush()
+    }
+}
+
+#[cfg(feature = "termcolor")]
+impl WriteColor for Renderer<'_, '_> {
+    fn supports_color(&self) -> bool {
+        self.writer.supports_color()
     }
 
-    fn set_line_number(&mut self) -> GeneralWriteResult {
-        self.writer.set_line_number()
+    fn set_color(&mut self, spec: &ColorSpec) -> io::Result<()> {
+        self.writer.set_color(spec)
     }
 
-    fn set_note_bullet(&mut self) -> GeneralWriteResult {
-        self.writer.set_note_bullet()
-    }
-
-    fn set_source_border(&mut self) -> GeneralWriteResult {
-        self.writer.set_source_border()
-    }
-
-    fn set_label(&mut self, severity: Severity, label_style: LabelStyle) -> GeneralWriteResult {
-        self.writer.set_label(severity, label_style)
-    }
-    fn reset(&mut self) -> GeneralWriteResult {
+    fn reset(&mut self) -> io::Result<()> {
         self.writer.reset()
     }
+
+    fn is_synchronous(&self) -> bool {
+        self.writer.is_synchronous()
+    }
 }
 
 struct Metrics {
