@@ -11,7 +11,7 @@ use crate::ir::{
     StackSlot, StackSlotData, StackSlots, Type,
 };
 use crate::isa::CallConv;
-use crate::write::write_function;
+use crate::write::{write_function, write_function_spec};
 use crate::HashMap;
 #[cfg(feature = "enable-serde")]
 use alloc::string::String;
@@ -282,7 +282,9 @@ impl FunctionStencil {
     /// Rewrite the branch destination to `new_dest` if the destination matches `old_dest`.
     /// Does nothing if called with a non-jump or non-branch instruction.
     pub fn rewrite_branch_destination(&mut self, inst: Inst, old_dest: Block, new_dest: Block) {
-        for dest in self.dfg.insts[inst].branch_destination_mut(&mut self.dfg.jump_tables) {
+        for dest in self.dfg.insts[inst]
+            .branch_destination_mut(&mut self.dfg.jump_tables, &mut self.dfg.exception_tables)
+        {
             if dest.block(&self.dfg.value_lists) == old_dest {
                 dest.set_block(new_dest, &mut self.dfg.value_lists)
             }
@@ -312,7 +314,7 @@ impl FunctionStencil {
     pub fn block_successors(&self, block: Block) -> impl DoubleEndedIterator<Item = Block> + '_ {
         self.layout.last_inst(block).into_iter().flat_map(|inst| {
             self.dfg.insts[inst]
-                .branch_destination(&self.dfg.jump_tables)
+                .branch_destination(&self.dfg.jump_tables, &self.dfg.exception_tables)
                 .iter()
                 .map(|block| block.block(&self.dfg.value_lists))
         })
@@ -445,6 +447,11 @@ impl Function {
         DisplayFunction(self)
     }
 
+    /// Return an object that can display this function's name and signature.
+    pub fn display_spec(&self) -> DisplayFunctionSpec<'_> {
+        DisplayFunctionSpec(self)
+    }
+
     /// Sets an absolute source location for the given instruction.
     ///
     /// If no base source location has been set yet, records it at the same time.
@@ -493,3 +500,18 @@ impl fmt::Debug for Function {
         write_function(fmt, self)
     }
 }
+
+/// Wrapper type capable of displaying a 'Function's name and signature.
+pub struct DisplayFunctionSpec<'a>(&'a Function);
+
+impl<'a> fmt::Display for DisplayFunctionSpec<'a> {
+    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
+        write_function_spec(fmt, self.0)
+    }
+}
+
+impl<'a> fmt::Debug for DisplayFunctionSpec<'a> {
+    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
+        write_function_spec(fmt, self.0)
+    }
+}
