@@ -62,8 +62,14 @@ where
     ) -> CodegenResult<(u32, Option<usize>)> {
         // NB: make sure this method stays in sync with
         // `cranelift_pulley::interp::Vm::call`.
+        //
+        // In general we use the first half of all register banks as argument
+        // passing registers because, well, why not for now. Currently the only
+        // exception is x15 which is reserved as a single caller-saved register
+        // not used for arguments. This is used in `ReturnCallIndirect` to hold
+        // the location of where we're jumping to.
 
-        let x_end = 15;
+        let x_end = 14;
         let f_end = 15;
         let v_end = 15;
 
@@ -319,14 +325,19 @@ where
 
         match &style {
             FrameStyle::None => {}
-            FrameStyle::PulleySetupNoClobbers => insts.push(RawInst::PushFrame.into()),
+            FrameStyle::PulleyBasicSetup { frame_size } => {
+                insts.push(RawInst::PushFrame.into());
+                insts.extend(Self::gen_sp_reg_adjust(
+                    -i32::try_from(*frame_size).unwrap(),
+                ));
+            }
             FrameStyle::PulleySetupAndSaveClobbers {
                 frame_size,
                 saved_by_pulley,
             } => insts.push(
                 RawInst::PushFrameSave {
                     amt: *frame_size,
-                    regs: pulley_interpreter::RegSet::from_bitset(*saved_by_pulley),
+                    regs: pulley_interpreter::UpperRegSet::from_bitset(*saved_by_pulley),
                 }
                 .into(),
             ),
@@ -349,16 +360,6 @@ where
         _call_conv: isa::CallConv,
         _flags: &settings::Flags,
         _isa_flags: &PulleyFlags,
-        _frame_layout: &FrameLayout,
-    ) -> SmallInstVec<Self::I> {
-        // Note that this is intentionally empty as `gen_return` does
-        // everything.
-        SmallVec::new()
-    }
-
-    fn gen_return(
-        _call_conv: isa::CallConv,
-        _isa_flags: &PulleyFlags,
         frame_layout: &FrameLayout,
     ) -> SmallInstVec<Self::I> {
         let mut insts = SmallVec::new();
@@ -381,14 +382,17 @@ where
         // Perform the inverse of `gen_prologue_frame_setup`.
         match &style {
             FrameStyle::None => {}
-            FrameStyle::PulleySetupNoClobbers => insts.push(RawInst::PopFrame.into()),
+            FrameStyle::PulleyBasicSetup { frame_size } => {
+                insts.extend(Self::gen_sp_reg_adjust(i32::try_from(*frame_size).unwrap()));
+                insts.push(RawInst::PopFrame.into());
+            }
             FrameStyle::PulleySetupAndSaveClobbers {
                 frame_size,
                 saved_by_pulley,
             } => insts.push(
                 RawInst::PopFrameRestore {
                     amt: *frame_size,
-                    regs: pulley_interpreter::RegSet::from_bitset(*saved_by_pulley),
+                    regs: pulley_interpreter::UpperRegSet::from_bitset(*saved_by_pulley),
                 }
                 .into(),
             ),
@@ -397,20 +401,24 @@ where
             }
         }
 
+        insts
+    }
+
+    fn gen_return(
+        call_conv: isa::CallConv,
+        _isa_flags: &PulleyFlags,
+        frame_layout: &FrameLayout,
+    ) -> SmallInstVec<Self::I> {
+        let mut insts = SmallVec::new();
+
         // Handle final stack adjustments for the tail-call ABI.
-        if frame_layout.tail_args_size > 0 {
+        if call_conv == isa::CallConv::Tail && frame_layout.tail_args_size > 0 {
             insts.extend(Self::gen_sp_reg_adjust(
                 frame_layout.tail_args_size.try_into().unwrap(),
             ));
         }
-
-        // And finally, return.
-        //
-        // FIXME: if `frame_layout.tail_args_size` is zero this instruction
-        // should get folded into the macro-instructions above. No need to have
-        // all functions do `pop_frame; ret`, that could be `pop_frame_and_ret`.
-        // Should benchmark whether this is worth it though.
         insts.push(RawInst::Ret {}.into());
+
         insts
     }
 
@@ -454,16 +462,7 @@ where
                 // `info.dest.args` to be handled differently during register
                 // allocation.
                 let mut args = SmallVec::new();
-                if cfg!(debug_assertions) {
-                    let xargs = info
-                        .uses
-                        .iter()
-                        .filter_map(|a| XReg::new(a.preg))
-                        .collect::<Vec<_>>();
-                    for window in xargs.windows(2) {
-                        assert!(window[0] < window[1]);
-                    }
-                }
+                info.uses.sort_by_key(|arg| arg.preg);
                 info.uses.retain(|arg| {
                     if arg.preg != x0() && arg.preg != x1() && arg.preg != x2() && arg.preg != x3()
                     {
@@ -482,15 +481,19 @@ where
             }
             // "far" calls are pulley->host calls so they use a different opcode
             // which is lowered with a special relocation in the backend.
-            CallDest::ExtName(name, RelocDistance::Far) => smallvec![Inst::IndirectCallHost {
-                info: Box::new(info.map(|()| name.clone()))
+            CallDest::ExtName(name, RelocDistance::Far) => {
+                smallvec![Inst::IndirectCallHost {
+                    info: Box::new(info.map(|()| name.clone()))
+                }
+                .into()]
             }
-            .into()],
             // Indirect calls are all assumed to be pulley->pulley calls
-            CallDest::Reg(reg) => smallvec![Inst::IndirectCall {
-                info: Box::new(info.map(|()| XReg::new(*reg).unwrap()))
+            CallDest::Reg(reg) => {
+                smallvec![Inst::IndirectCall {
+                    info: Box::new(info.map(|()| XReg::new(*reg).unwrap()))
+                }
+                .into()]
             }
-            .into()],
         }
     }
 
@@ -529,8 +532,15 @@ where
         MACHINE_ENV.get_or_init(create_reg_environment)
     }
 
-    fn get_regs_clobbered_by_call(_call_conv_of_callee: isa::CallConv) -> PRegSet {
-        DEFAULT_CLOBBERS
+    fn get_regs_clobbered_by_call(
+        _call_conv_of_callee: isa::CallConv,
+        is_exception: bool,
+    ) -> PRegSet {
+        if is_exception {
+            ALL_CLOBBERS
+        } else {
+            DEFAULT_CLOBBERS
+        }
     }
 
     fn compute_frame_layout(
@@ -541,6 +551,7 @@ where
         is_leaf: bool,
         incoming_args_size: u32,
         tail_args_size: u32,
+        stackslots_size: u32,
         fixed_frame_storage_size: u32,
         outgoing_args_size: u32,
     ) -> FrameLayout {
@@ -575,6 +586,7 @@ where
             setup_area_size: setup_area_size.into(),
             clobber_size,
             fixed_frame_storage_size,
+            stackslots_size,
             outgoing_args_size,
             clobbered_callee_saves: regs,
         }
@@ -589,6 +601,20 @@ where
         // Pulley doesn't need inline probestacks because it always checks stack
         // decrements.
     }
+
+    fn retval_temp_reg(_call_conv_of_callee: isa::CallConv) -> Writable<Reg> {
+        // Use x15 as a temp if needed: clobbered, not a
+        // retval.
+        Writable::from_reg(regs::x_reg(15))
+    }
+
+    fn exception_payload_regs(_call_conv: isa::CallConv) -> &'static [Reg] {
+        const PAYLOAD_REGS: &'static [Reg] = &[
+            Reg::from_real_reg(regs::px_reg(0)),
+            Reg::from_real_reg(regs::px_reg(1)),
+        ];
+        PAYLOAD_REGS
+    }
 }
 
 /// Different styles of management of fp/lr and clobbered registers.
@@ -600,9 +626,9 @@ enum FrameStyle {
     /// No stack is being allocated either.
     None,
 
-    /// No stack is being allocated and nothing is clobbered, but Pulley should
-    /// save the fp/lr combo.
-    PulleySetupNoClobbers,
+    /// Pulley saves the fp/lr combo and then stack adjustments/clobbers are
+    /// handled manually.
+    PulleyBasicSetup { frame_size: u32 },
 
     /// Pulley is managing the fp/lr combo, the stack size, and clobbered
     /// X-class registers.
@@ -612,9 +638,9 @@ enum FrameStyle {
     /// instruction.
     PulleySetupAndSaveClobbers {
         /// The size of the frame, including clobbers, that's being allocated.
-        frame_size: u32,
+        frame_size: u16,
         /// Registers that pulley is saving/restoring.
-        saved_by_pulley: ScalarBitSet<u32>,
+        saved_by_pulley: ScalarBitSet<u16>,
     },
 
     /// Cranelift is manually managing everything, both clobbers and stack
@@ -655,14 +681,20 @@ impl FrameLayout {
 
             // No stack allocated, saving fp/lr, no clobbers, so this is
             // pulley-managed via push/pop_frame.
-            (0, true, true) => FrameStyle::PulleySetupNoClobbers,
+            (0, true, true) => FrameStyle::PulleyBasicSetup { frame_size: 0 },
 
             // Some stack is being allocated and pulley is managing fp/lr. Let
             // pulley manage clobbered registers as well, regardless if they're
             // present or not.
-            (frame_size, true, _) => FrameStyle::PulleySetupAndSaveClobbers {
-                frame_size,
-                saved_by_pulley,
+            //
+            // If the stack is too large then `PulleyBasicSetup` is used
+            // otherwise we'll be pushing `PushFrameSave` and `PopFrameRestore`.
+            (frame_size, true, _) => match frame_size.try_into() {
+                Ok(frame_size) => FrameStyle::PulleySetupAndSaveClobbers {
+                    frame_size,
+                    saved_by_pulley,
+                },
+                Err(_) => FrameStyle::PulleyBasicSetup { frame_size },
             },
 
             // Some stack is being allocated, but pulley isn't managing fp/lr,
@@ -677,8 +709,8 @@ impl FrameLayout {
 
     /// Returns the set of clobbered registers that Pulley is managing via its
     /// macro instructions rather than the generated code.
-    fn clobbered_xregs_saved_by_pulley(&self) -> ScalarBitSet<u32> {
-        let mut clobbered: ScalarBitSet<u32> = ScalarBitSet::new();
+    fn clobbered_xregs_saved_by_pulley(&self) -> ScalarBitSet<u16> {
+        let mut clobbered: ScalarBitSet<u16> = ScalarBitSet::new();
         // Pulley only manages clobbers if it's also managing fp/lr.
         if !self.setup_frame() {
             return clobbered;
@@ -694,7 +726,9 @@ impl FrameLayout {
             // incorrect.
             if r_reg.class() == RegClass::Int {
                 assert!(!found_manual_clobber);
-                clobbered.insert(r_reg.hw_enc());
+                if let Some(offset) = r_reg.hw_enc().checked_sub(16) {
+                    clobbered.insert(offset);
+                }
             } else {
                 found_manual_clobber = true;
             }
@@ -728,8 +762,10 @@ impl FrameLayout {
                         saved_by_pulley, ..
                     } = style
                     {
-                        if saved_by_pulley.contains(r_reg.hw_enc()) {
-                            return None;
+                        if let Some(reg) = r_reg.hw_enc().checked_sub(16) {
+                            if saved_by_pulley.contains(reg) {
+                                return None;
+                            }
                         }
                     }
                     I64
@@ -913,6 +949,104 @@ const DEFAULT_CLOBBERS: PRegSet = PRegSet::empty()
     .with(pv_reg(30))
     .with(pv_reg(31));
 
+const ALL_CLOBBERS: PRegSet = PRegSet::empty()
+    .with(px_reg(0))
+    .with(px_reg(1))
+    .with(px_reg(2))
+    .with(px_reg(3))
+    .with(px_reg(4))
+    .with(px_reg(5))
+    .with(px_reg(6))
+    .with(px_reg(7))
+    .with(px_reg(8))
+    .with(px_reg(9))
+    .with(px_reg(10))
+    .with(px_reg(11))
+    .with(px_reg(12))
+    .with(px_reg(13))
+    .with(px_reg(14))
+    .with(px_reg(15))
+    .with(px_reg(16))
+    .with(px_reg(17))
+    .with(px_reg(18))
+    .with(px_reg(19))
+    .with(px_reg(20))
+    .with(px_reg(21))
+    .with(px_reg(22))
+    .with(px_reg(23))
+    .with(px_reg(24))
+    .with(px_reg(25))
+    .with(px_reg(26))
+    .with(px_reg(27))
+    .with(px_reg(28))
+    .with(px_reg(29))
+    .with(px_reg(30))
+    .with(px_reg(31))
+    .with(pf_reg(0))
+    .with(pf_reg(1))
+    .with(pf_reg(2))
+    .with(pf_reg(3))
+    .with(pf_reg(4))
+    .with(pf_reg(5))
+    .with(pf_reg(6))
+    .with(pf_reg(7))
+    .with(pf_reg(8))
+    .with(pf_reg(9))
+    .with(pf_reg(10))
+    .with(pf_reg(11))
+    .with(pf_reg(12))
+    .with(pf_reg(13))
+    .with(pf_reg(14))
+    .with(pf_reg(15))
+    .with(pf_reg(16))
+    .with(pf_reg(17))
+    .with(pf_reg(18))
+    .with(pf_reg(19))
+    .with(pf_reg(20))
+    .with(pf_reg(21))
+    .with(pf_reg(22))
+    .with(pf_reg(23))
+    .with(pf_reg(24))
+    .with(pf_reg(25))
+    .with(pf_reg(26))
+    .with(pf_reg(27))
+    .with(pf_reg(28))
+    .with(pf_reg(29))
+    .with(pf_reg(30))
+    .with(pf_reg(31))
+    .with(pv_reg(0))
+    .with(pv_reg(1))
+    .with(pv_reg(2))
+    .with(pv_reg(3))
+    .with(pv_reg(4))
+    .with(pv_reg(5))
+    .with(pv_reg(6))
+    .with(pv_reg(7))
+    .with(pv_reg(8))
+    .with(pv_reg(9))
+    .with(pv_reg(10))
+    .with(pv_reg(11))
+    .with(pv_reg(12))
+    .with(pv_reg(13))
+    .with(pv_reg(14))
+    .with(pv_reg(15))
+    .with(pv_reg(16))
+    .with(pv_reg(17))
+    .with(pv_reg(18))
+    .with(pv_reg(19))
+    .with(pv_reg(20))
+    .with(pv_reg(21))
+    .with(pv_reg(22))
+    .with(pv_reg(23))
+    .with(pv_reg(24))
+    .with(pv_reg(25))
+    .with(pv_reg(26))
+    .with(pv_reg(27))
+    .with(pv_reg(28))
+    .with(pv_reg(29))
+    .with(pv_reg(30))
+    .with(pv_reg(31));
+
 fn create_reg_environment() -> MachineEnv {
     // Prefer caller-saved registers over callee-saved registers, because that
     // way we don't need to emit code to save and restore them if we don't
