@@ -1,17 +1,20 @@
 use crate::gen::block::Block;
+use crate::gen::guard::Guard;
 use crate::gen::nested::NamespaceEntries;
 use crate::gen::out::OutFile;
-use crate::gen::{builtin, include, Opt};
+use crate::gen::{builtin, include, pragma, Opt};
 use crate::syntax::atom::Atom::{self, *};
+use crate::syntax::discriminant::{Discriminant, Limits};
 use crate::syntax::instantiate::{ImplKey, NamedImplKey};
 use crate::syntax::map::UnorderedMap as Map;
+use crate::syntax::namespace::Namespace;
 use crate::syntax::primitive::{self, PrimitiveKind};
 use crate::syntax::set::UnorderedSet;
 use crate::syntax::symbol::{self, Symbol};
 use crate::syntax::trivial::{self, TrivialReason};
 use crate::syntax::{
-    derive, mangle, Api, Doc, Enum, EnumRepr, ExternFn, ExternType, Pair, Signature, Struct, Trait,
-    Type, TypeAlias, Types, Var,
+    derive, mangle, Api, Doc, Enum, ExternFn, ExternType, FnKind, Lang, Pair, Signature, Struct,
+    Trait, Type, TypeAlias, Types, Var,
 };
 use proc_macro2::Ident;
 
@@ -29,6 +32,7 @@ pub(super) fn gen(apis: &[Api], types: &Types, opt: &Opt, header: bool) -> Vec<u
     write_generic_instantiations(out);
 
     builtin::write(out);
+    pragma::write(out);
     include::write(out);
 
     out_file.content()
@@ -95,9 +99,9 @@ fn write_data_structures<'a>(out: &mut OutFile<'a>, apis: &'a [Api]) {
     let mut methods_for_type = Map::new();
     for api in apis {
         if let Api::CxxFunction(efn) | Api::RustFunction(efn) = api {
-            if let Some(receiver) = &efn.sig.receiver {
+            if let Some(self_type) = efn.self_type() {
                 methods_for_type
-                    .entry(&receiver.ty.rust)
+                    .entry(self_type)
                     .or_insert_with(Vec::new)
                     .push(efn);
             }
@@ -126,10 +130,10 @@ fn write_data_structures<'a>(out: &mut OutFile<'a>, apis: &'a [Api]) {
             }
             Api::Enum(enm) => {
                 out.next_section();
-                if !out.types.cxx.contains(&enm.name.rust) {
-                    write_enum(out, enm);
-                } else if !enm.variants_from_header {
+                if out.types.cxx.contains(&enm.name.rust) {
                     check_enum(out, enm);
+                } else {
+                    write_enum(out, enm);
                 }
             }
             Api::RustType(ety) => {
@@ -198,6 +202,7 @@ fn write_std_specializations(out: &mut OutFile, apis: &[Api]) {
                 out.next_section();
                 out.include.cstddef = true;
                 out.include.functional = true;
+                out.pragma.dollar_in_identifier = true;
                 let qualified = strct.name.to_fully_qualified();
                 writeln!(out, "template <> struct hash<{}> {{", qualified);
                 writeln!(
@@ -275,11 +280,27 @@ fn write_struct<'a>(out: &mut OutFile<'a>, strct: &'a Struct, methods: &[&Extern
     let operator_ord = derive::contains(&strct.derives, Trait::PartialOrd);
 
     out.set_namespace(&strct.name.namespace);
-    let guard = format!("CXXBRIDGE1_STRUCT_{}", strct.name.to_symbol());
+    let guard = Guard::new(out, "CXXBRIDGE1_STRUCT", &strct.name);
     writeln!(out, "#ifndef {}", guard);
     writeln!(out, "#define {}", guard);
     write_doc(out, "", &strct.doc);
-    writeln!(out, "struct {} final {{", strct.name.cxx);
+    write!(out, "struct");
+    if let Some(align) = &strct.align {
+        out.builtin.alignmax = true;
+        writeln!(out, " alignas(::rust::repr::alignmax<");
+        writeln!(out, "  {},", align.base10_parse::<u32>().unwrap());
+        for (i, field) in strct.fields.iter().enumerate() {
+            write!(out, "  alignof(");
+            write_type(out, &field.ty);
+            write!(out, ")");
+            if i + 1 != strct.fields.len() {
+                write!(out, ",");
+            }
+            writeln!(out);
+        }
+        write!(out, ">)");
+    }
+    writeln!(out, " {} final {{", strct.name.cxx);
 
     for field in &strct.fields {
         write_doc(out, "  ", &field.doc);
@@ -305,10 +326,12 @@ fn write_struct<'a>(out: &mut OutFile<'a>, strct: &'a Struct, methods: &[&Extern
         }
         write_doc(out, "  ", &method.doc);
         write!(out, "  ");
-        let sig = &method.sig;
         let local_name = method.name.cxx.to_string();
+        let sig = &method.sig;
+        let in_class = true;
         let indirect_call = false;
-        write_rust_function_shim_decl(out, &local_name, sig, indirect_call);
+        let main = false;
+        write_rust_function_shim_decl(out, &local_name, sig, in_class, indirect_call, main);
         writeln!(out, ";");
         if !method.doc.is_empty() {
             out.next_section();
@@ -363,13 +386,8 @@ fn write_struct_decl(out: &mut OutFile, ident: &Pair) {
 }
 
 fn write_enum_decl(out: &mut OutFile, enm: &Enum) {
-    let repr = match &enm.repr {
-        #[cfg(feature = "experimental-enum-variants-from-header")]
-        EnumRepr::Foreign { .. } => return,
-        EnumRepr::Native { atom, .. } => *atom,
-    };
     write!(out, "enum class {} : ", enm.name.cxx);
-    write_atom(out, repr);
+    write_atom(out, enm.repr.atom);
     writeln!(out, ";");
 }
 
@@ -379,7 +397,7 @@ fn write_struct_using(out: &mut OutFile, ident: &Pair) {
 
 fn write_opaque_type<'a>(out: &mut OutFile<'a>, ety: &'a ExternType, methods: &[&ExternFn]) {
     out.set_namespace(&ety.name.namespace);
-    let guard = format!("CXXBRIDGE1_STRUCT_{}", ety.name.to_symbol());
+    let guard = Guard::new(out, "CXXBRIDGE1_STRUCT", &ety.name);
     writeln!(out, "#ifndef {}", guard);
     writeln!(out, "#define {}", guard);
     write_doc(out, "", &ety.doc);
@@ -397,10 +415,12 @@ fn write_opaque_type<'a>(out: &mut OutFile<'a>, ety: &'a ExternType, methods: &[
         }
         write_doc(out, "  ", &method.doc);
         write!(out, "  ");
-        let sig = &method.sig;
         let local_name = method.name.cxx.to_string();
+        let sig = &method.sig;
+        let in_class = true;
         let indirect_call = false;
-        write_rust_function_shim_decl(out, &local_name, sig, indirect_call);
+        let main = false;
+        write_rust_function_shim_decl(out, &local_name, sig, in_class, indirect_call, main);
         writeln!(out, ";");
         if !method.doc.is_empty() {
             out.next_section();
@@ -423,33 +443,31 @@ fn write_opaque_type<'a>(out: &mut OutFile<'a>, ety: &'a ExternType, methods: &[
 }
 
 fn write_enum<'a>(out: &mut OutFile<'a>, enm: &'a Enum) {
-    let repr = match &enm.repr {
-        #[cfg(feature = "experimental-enum-variants-from-header")]
-        EnumRepr::Foreign { .. } => return,
-        EnumRepr::Native { atom, .. } => *atom,
-    };
     out.set_namespace(&enm.name.namespace);
-    let guard = format!("CXXBRIDGE1_ENUM_{}", enm.name.to_symbol());
+    let guard = Guard::new(out, "CXXBRIDGE1_ENUM", &enm.name);
     writeln!(out, "#ifndef {}", guard);
     writeln!(out, "#define {}", guard);
+
     write_doc(out, "", &enm.doc);
     write!(out, "enum class {} : ", enm.name.cxx);
-    write_atom(out, repr);
+    write_atom(out, enm.repr.atom);
     writeln!(out, " {{");
     for variant in &enm.variants {
         write_doc(out, "  ", &variant.doc);
-        writeln!(out, "  {} = {},", variant.name.cxx, variant.discriminant);
+        write!(out, "  {} = ", variant.name.cxx);
+        write_discriminant(out, enm.repr.atom, variant.discriminant);
+        writeln!(out, ",");
     }
     writeln!(out, "}};");
+
+    if out.header {
+        write_enum_operators(out, enm);
+    }
+
     writeln!(out, "#endif // {}", guard);
 }
 
 fn check_enum<'a>(out: &mut OutFile<'a>, enm: &'a Enum) {
-    let repr = match &enm.repr {
-        #[cfg(feature = "experimental-enum-variants-from-header")]
-        EnumRepr::Foreign { .. } => return,
-        EnumRepr::Native { atom, .. } => *atom,
-    };
     out.set_namespace(&enm.name.namespace);
     out.include.type_traits = true;
     writeln!(
@@ -458,16 +476,71 @@ fn check_enum<'a>(out: &mut OutFile<'a>, enm: &'a Enum) {
         enm.name.cxx,
     );
     write!(out, "static_assert(sizeof({}) == sizeof(", enm.name.cxx);
-    write_atom(out, repr);
+    write_atom(out, enm.repr.atom);
     writeln!(out, "), \"incorrect size\");");
     for variant in &enm.variants {
         write!(out, "static_assert(static_cast<");
+        write_atom(out, enm.repr.atom);
+        writeln!(out, ">({}::{}) == ", enm.name.cxx, variant.name.cxx);
+        write_discriminant(out, enm.repr.atom, variant.discriminant);
+        writeln!(out, ", \"disagrees with the value in #[cxx::bridge]\");");
+    }
+
+    if out.header
+        && (derive::contains(&enm.derives, Trait::BitAnd)
+            || derive::contains(&enm.derives, Trait::BitOr)
+            || derive::contains(&enm.derives, Trait::BitXor))
+    {
+        out.next_section();
+        let guard = Guard::new(out, "CXXBRIDGE1_ENUM", &enm.name);
+        writeln!(out, "#ifndef {}", guard);
+        writeln!(out, "#define {}", guard);
+        out.suppress_next_section();
+        write_enum_operators(out, enm);
+        writeln!(out, "#endif // {}", guard);
+    }
+}
+
+fn write_discriminant(out: &mut OutFile, repr: Atom, discriminant: Discriminant) {
+    let limits = Limits::of(repr).unwrap();
+    if discriminant == limits.min && limits.min < Discriminant::zero() {
+        out.include.limits = true;
+        write!(out, "::std::numeric_limits<");
         write_atom(out, repr);
-        writeln!(
-            out,
-            ">({}::{}) == {}, \"disagrees with the value in #[cxx::bridge]\");",
-            enm.name.cxx, variant.name.cxx, variant.discriminant,
-        );
+        write!(out, ">::min()");
+    } else {
+        write!(out, "{}", discriminant);
+    }
+}
+
+fn write_binary_bitwise_op(out: &mut OutFile, op: &str, enm: &Enum) {
+    let enum_name = &enm.name.cxx;
+    writeln!(
+        out,
+        "inline {enum_name} operator{op}({enum_name} lhs, {enum_name} rhs) {{",
+    );
+    write!(out, "  return static_cast<{enum_name}>(static_cast<");
+    write_atom(out, enm.repr.atom);
+    write!(out, ">(lhs) {op} static_cast<");
+    write_atom(out, enm.repr.atom);
+    writeln!(out, ">(rhs));");
+    writeln!(out, "}}");
+}
+
+fn write_enum_operators(out: &mut OutFile, enm: &Enum) {
+    if derive::contains(&enm.derives, Trait::BitAnd) {
+        out.next_section();
+        write_binary_bitwise_op(out, "&", enm);
+    }
+
+    if derive::contains(&enm.derives, Trait::BitOr) {
+        out.next_section();
+        write_binary_bitwise_op(out, "|", enm);
+    }
+
+    if derive::contains(&enm.derives, Trait::BitXor) {
+        out.next_section();
+        write_binary_bitwise_op(out, "^", enm);
     }
 }
 
@@ -507,11 +580,20 @@ fn check_trivial_extern_type(out: &mut OutFile, alias: &TypeAlias, reasons: &[Tr
 
     let id = alias.name.to_fully_qualified();
     out.builtin.relocatable = true;
-    writeln!(out, "static_assert(");
-    if reasons
-        .iter()
-        .all(|r| matches!(r, TrivialReason::StructField(_) | TrivialReason::VecElement))
-    {
+
+    let mut rust_type_ok = true;
+    let mut array_ok = true;
+    for reason in reasons {
+        // Allow extern type that inherits from ::rust::Opaque in positions
+        // where an opaque Rust type would be allowed.
+        rust_type_ok &= match reason {
+            TrivialReason::BoxTarget { .. }
+            | TrivialReason::VecElement { .. }
+            | TrivialReason::SliceElement { .. } => true,
+            TrivialReason::StructField(_)
+            | TrivialReason::FunctionArgument(_)
+            | TrivialReason::FunctionReturn(_) => false,
+        };
         // If the type is only used as a struct field or Vec element, not as
         // by-value function argument or return value, then C array of trivially
         // relocatable type is also permissible.
@@ -522,10 +604,27 @@ fn check_trivial_extern_type(out: &mut OutFile, alias: &TypeAlias, reasons: &[Tr
         //     --- means something totally different:
         //     void f(char buf[N]);
         //
+        array_ok &= match reason {
+            TrivialReason::StructField(_) | TrivialReason::VecElement { .. } => true,
+            TrivialReason::FunctionArgument(_)
+            | TrivialReason::FunctionReturn(_)
+            | TrivialReason::BoxTarget { .. }
+            | TrivialReason::SliceElement { .. } => false,
+        };
+    }
+
+    writeln!(out, "static_assert(");
+    write!(out, "    ");
+    if rust_type_ok {
+        out.include.type_traits = true;
+        out.builtin.opaque = true;
+        write!(out, "::std::is_base_of<::rust::Opaque, {}>::value || ", id);
+    }
+    if array_ok {
         out.builtin.relocatable_or_array = true;
-        writeln!(out, "    ::rust::IsRelocatableOrArray<{}>::value,", id);
+        writeln!(out, "::rust::IsRelocatableOrArray<{}>::value,", id);
     } else {
-        writeln!(out, "    ::rust::IsRelocatable<{}>::value,", id);
+        writeln!(out, "::rust::IsRelocatable<{}>::value,", id);
     }
     writeln!(
         out,
@@ -540,6 +639,8 @@ fn write_struct_operator_decls<'a>(out: &mut OutFile<'a>, strct: &'a Struct) {
     out.begin_block(Block::ExternC);
 
     if derive::contains(&strct.derives, Trait::PartialEq) {
+        out.pragma.dollar_in_identifier = true;
+        out.pragma.missing_declarations = true;
         let link_name = mangle::operator(&strct.name, "eq");
         writeln!(
             out,
@@ -558,6 +659,8 @@ fn write_struct_operator_decls<'a>(out: &mut OutFile<'a>, strct: &'a Struct) {
     }
 
     if derive::contains(&strct.derives, Trait::PartialOrd) {
+        out.pragma.dollar_in_identifier = true;
+        out.pragma.missing_declarations = true;
         let link_name = mangle::operator(&strct.name, "lt");
         writeln!(
             out,
@@ -591,6 +694,8 @@ fn write_struct_operator_decls<'a>(out: &mut OutFile<'a>, strct: &'a Struct) {
 
     if derive::contains(&strct.derives, Trait::Hash) {
         out.include.cstddef = true;
+        out.pragma.dollar_in_identifier = true;
+        out.pragma.missing_declarations = true;
         let link_name = mangle::operator(&strct.name, "hash");
         writeln!(
             out,
@@ -610,6 +715,8 @@ fn write_struct_operators<'a>(out: &mut OutFile<'a>, strct: &'a Struct) {
     out.set_namespace(&strct.name.namespace);
 
     if derive::contains(&strct.derives, Trait::PartialEq) {
+        out.pragma.dollar_in_identifier = true;
+
         out.next_section();
         writeln!(
             out,
@@ -636,6 +743,8 @@ fn write_struct_operators<'a>(out: &mut OutFile<'a>, strct: &'a Struct) {
     }
 
     if derive::contains(&strct.derives, Trait::PartialOrd) {
+        out.pragma.dollar_in_identifier = true;
+
         out.next_section();
         writeln!(
             out,
@@ -689,6 +798,8 @@ fn write_struct_operators<'a>(out: &mut OutFile<'a>, strct: &'a Struct) {
 fn write_opaque_type_layout_decls<'a>(out: &mut OutFile<'a>, ety: &'a ExternType) {
     out.set_namespace(&ety.name.namespace);
     out.begin_block(Block::ExternC);
+    out.pragma.dollar_in_identifier = true;
+    out.pragma.missing_declarations = true;
 
     let link_name = mangle::operator(&ety.name, "sizeof");
     writeln!(out, "::std::size_t {}() noexcept;", link_name);
@@ -705,6 +816,7 @@ fn write_opaque_type_layout<'a>(out: &mut OutFile<'a>, ety: &'a ExternType) {
     }
 
     out.set_namespace(&ety.name.namespace);
+    out.pragma.dollar_in_identifier = true;
 
     out.next_section();
     let link_name = mangle::operator(&ety.name, "sizeof");
@@ -734,6 +846,8 @@ fn begin_function_definition(out: &mut OutFile) {
 }
 
 fn write_cxx_function_shim<'a>(out: &mut OutFile<'a>, efn: &'a ExternFn) {
+    out.pragma.dollar_in_identifier = true;
+    out.pragma.missing_declarations = true;
     out.next_section();
     out.set_namespace(&efn.name.namespace);
     out.begin_block(Block::ExternC);
@@ -742,11 +856,11 @@ fn write_cxx_function_shim<'a>(out: &mut OutFile<'a>, efn: &'a ExternFn) {
         out.builtin.ptr_len = true;
         write!(out, "::rust::repr::PtrLen ");
     } else {
-        write_extern_return_type_space(out, &efn.ret);
+        write_extern_return_type_space(out, efn, efn.lang);
     }
     let mangled = mangle::extern_fn(efn, out.types);
     write!(out, "{}(", mangled);
-    if let Some(receiver) = &efn.receiver {
+    if let FnKind::Method(receiver) = &efn.kind {
         write!(
             out,
             "{}",
@@ -758,7 +872,7 @@ fn write_cxx_function_shim<'a>(out: &mut OutFile<'a>, efn: &'a ExternFn) {
         write!(out, " &self");
     }
     for (i, arg) in efn.args.iter().enumerate() {
-        if i > 0 || efn.receiver.is_some() {
+        if i > 0 || matches!(efn.kind, FnKind::Method(_)) {
             write!(out, ", ");
         }
         if arg.ty == RustString {
@@ -771,18 +885,24 @@ fn write_cxx_function_shim<'a>(out: &mut OutFile<'a>, efn: &'a ExternFn) {
             write_extern_arg(out, arg);
         }
     }
-    let indirect_return = indirect_return(efn, out.types);
+    let indirect_return = indirect_return(efn, out.types, efn.lang);
     if indirect_return {
-        if !efn.args.is_empty() || efn.receiver.is_some() {
+        if !efn.args.is_empty() || matches!(efn.kind, FnKind::Method(_)) {
             write!(out, ", ");
         }
         write_indirect_return_type_space(out, efn.ret.as_ref().unwrap());
         write!(out, "*return$");
     }
-    writeln!(out, ") noexcept {{");
+    write!(out, ")");
+    match efn.lang {
+        Lang::Cxx => write!(out, " noexcept"),
+        Lang::CxxUnwind => {}
+        Lang::Rust => unreachable!(),
+    }
+    writeln!(out, " {{");
     write!(out, "  ");
     write_return_type(out, &efn.ret);
-    match &efn.receiver {
+    match efn.receiver() {
         None => write!(out, "(*{}$)(", efn.name.rust),
         Some(receiver) => write!(
             out,
@@ -798,18 +918,18 @@ fn write_cxx_function_shim<'a>(out: &mut OutFile<'a>, efn: &'a ExternFn) {
         write_type(out, &arg.ty);
     }
     write!(out, ")");
-    if let Some(receiver) = &efn.receiver {
+    if let Some(receiver) = efn.receiver() {
         if !receiver.mutable {
             write!(out, " const");
         }
     }
     write!(out, " = ");
-    match &efn.receiver {
+    match efn.self_type() {
         None => write!(out, "{}", efn.name.to_fully_qualified()),
-        Some(receiver) => write!(
+        Some(self_type) => write!(
             out,
             "&{}::{}",
-            out.types.resolve(&receiver.ty).name.to_fully_qualified(),
+            out.types.resolve(self_type).name.to_fully_qualified(),
             efn.name.cxx,
         ),
     }
@@ -845,7 +965,7 @@ fn write_cxx_function_shim<'a>(out: &mut OutFile<'a>, efn: &'a ExternFn) {
         }
         _ => {}
     }
-    match &efn.receiver {
+    match efn.receiver() {
         None => write!(out, "{}$(", efn.name.rust),
         Some(_) => write!(out, "(self.*{}$)(", efn.name.rust),
     }
@@ -905,6 +1025,8 @@ fn write_cxx_function_shim<'a>(out: &mut OutFile<'a>, efn: &'a ExternFn) {
 }
 
 fn write_function_pointer_trampoline(out: &mut OutFile, efn: &ExternFn, var: &Pair, f: &Signature) {
+    out.pragma.return_type_c_linkage = true;
+
     let r_trampoline = mangle::r_trampoline(efn, var, out.types);
     let indirect_call = true;
     write_rust_function_decl_impl(out, &r_trampoline, f, indirect_call);
@@ -912,7 +1034,16 @@ fn write_function_pointer_trampoline(out: &mut OutFile, efn: &ExternFn, var: &Pa
     out.next_section();
     let c_trampoline = mangle::c_trampoline(efn, var, out.types).to_string();
     let doc = Doc::new();
-    write_rust_function_shim_impl(out, &c_trampoline, f, &doc, &r_trampoline, indirect_call);
+    let main = false;
+    write_rust_function_shim_impl(
+        out,
+        &c_trampoline,
+        f,
+        &doc,
+        &r_trampoline,
+        indirect_call,
+        main,
+    );
 }
 
 fn write_rust_function_decl<'a>(out: &mut OutFile<'a>, efn: &'a ExternFn) {
@@ -931,15 +1062,16 @@ fn write_rust_function_decl_impl(
     indirect_call: bool,
 ) {
     out.next_section();
+    out.pragma.dollar_in_identifier = true;
     if sig.throws {
         out.builtin.ptr_len = true;
         write!(out, "::rust::repr::PtrLen ");
     } else {
-        write_extern_return_type_space(out, &sig.ret);
+        write_extern_return_type_space(out, sig, Lang::Rust);
     }
     write!(out, "{}(", link_name);
     let mut needs_comma = false;
-    if let Some(receiver) = &sig.receiver {
+    if let FnKind::Method(receiver) = &sig.kind {
         write!(
             out,
             "{}",
@@ -958,7 +1090,7 @@ fn write_rust_function_decl_impl(
         write_extern_arg(out, arg);
         needs_comma = true;
     }
-    if indirect_return(sig, out.types) {
+    if indirect_return(sig, out.types, Lang::Rust) {
         if needs_comma {
             write!(out, ", ");
         }
@@ -986,28 +1118,44 @@ fn write_rust_function_decl_impl(
 
 fn write_rust_function_shim<'a>(out: &mut OutFile<'a>, efn: &'a ExternFn) {
     out.set_namespace(&efn.name.namespace);
-    let local_name = match &efn.sig.receiver {
+    let local_name = match efn.self_type() {
         None => efn.name.cxx.to_string(),
-        Some(receiver) => format!(
+        Some(self_type) => format!(
             "{}::{}",
-            out.types.resolve(&receiver.ty).name.cxx,
+            out.types.resolve(self_type).name.cxx,
             efn.name.cxx,
         ),
     };
     let doc = &efn.doc;
     let invoke = mangle::extern_fn(efn, out.types);
     let indirect_call = false;
-    write_rust_function_shim_impl(out, &local_name, efn, doc, &invoke, indirect_call);
+    let main = efn.name.cxx == *"main"
+        && efn.name.namespace == Namespace::ROOT
+        && efn.sig.asyncness.is_none()
+        && matches!(efn.kind, FnKind::Free)
+        && efn.sig.args.is_empty()
+        && efn.sig.ret.is_none()
+        && !efn.sig.throws;
+    write_rust_function_shim_impl(out, &local_name, efn, doc, &invoke, indirect_call, main);
 }
 
 fn write_rust_function_shim_decl(
     out: &mut OutFile,
     local_name: &str,
     sig: &Signature,
+    in_class: bool,
     indirect_call: bool,
+    main: bool,
 ) {
     begin_function_definition(out);
-    write_return_type(out, &sig.ret);
+    if matches!(sig.kind, FnKind::Assoc(_)) && in_class {
+        write!(out, "static ");
+    }
+    if main {
+        write!(out, "int ");
+    } else {
+        write_return_type(out, &sig.ret);
+    }
     write!(out, "{}(", local_name);
     for (i, arg) in sig.args.iter().enumerate() {
         if i > 0 {
@@ -1023,7 +1171,7 @@ fn write_rust_function_shim_decl(
         write!(out, "void *extern$");
     }
     write!(out, ")");
-    if let Some(receiver) = &sig.receiver {
+    if let FnKind::Method(receiver) = &sig.kind {
         if !receiver.mutable {
             write!(out, " const");
         }
@@ -1040,16 +1188,22 @@ fn write_rust_function_shim_impl(
     doc: &Doc,
     invoke: &Symbol,
     indirect_call: bool,
+    main: bool,
 ) {
-    if out.header && sig.receiver.is_some() {
+    if match sig.kind {
+        FnKind::Free => false,
+        FnKind::Method(_) | FnKind::Assoc(_) => out.header,
+    } {
         // We've already defined this inside the struct.
         return;
     }
-    if sig.receiver.is_none() {
+    out.pragma.dollar_in_identifier = true;
+    if matches!(sig.kind, FnKind::Free) {
         // Member functions already documented at their declaration.
         write_doc(out, "", doc);
     }
-    write_rust_function_shim_decl(out, local_name, sig, indirect_call);
+    let in_class = false;
+    write_rust_function_shim_decl(out, local_name, sig, in_class, indirect_call, main);
     if out.header {
         writeln!(out, ";");
         return;
@@ -1065,7 +1219,7 @@ fn write_rust_function_shim_impl(
         }
     }
     write!(out, "  ");
-    let indirect_return = indirect_return(sig, out.types);
+    let indirect_return = indirect_return(sig, out.types, Lang::Rust);
     if indirect_return {
         out.builtin.maybe_uninit = true;
         write!(out, "::rust::MaybeUninit<");
@@ -1112,7 +1266,7 @@ fn write_rust_function_shim_impl(
     }
     write!(out, "{}(", invoke);
     let mut needs_comma = false;
-    if sig.receiver.is_some() {
+    if matches!(sig.kind, FnKind::Method(_)) {
         write!(out, "*this");
         needs_comma = true;
     }
@@ -1181,10 +1335,15 @@ fn write_return_type(out: &mut OutFile, ty: &Option<Type>) {
     }
 }
 
-fn indirect_return(sig: &Signature, types: &Types) -> bool {
-    sig.ret
-        .as_ref()
-        .is_some_and(|ret| sig.throws || types.needs_indirect_abi(ret))
+fn indirect_return(sig: &Signature, types: &Types, lang: Lang) -> bool {
+    sig.ret.as_ref().is_some_and(|ret| {
+        sig.throws
+            || types.needs_indirect_abi(ret)
+            || match lang {
+                Lang::Cxx | Lang::CxxUnwind => types.contains_elided_lifetime(ret),
+                Lang::Rust => false,
+            }
+    })
 }
 
 fn write_indirect_return_type(out: &mut OutFile, ty: &Type) {
@@ -1213,8 +1372,9 @@ fn write_indirect_return_type_space(out: &mut OutFile, ty: &Type) {
     }
 }
 
-fn write_extern_return_type_space(out: &mut OutFile, ty: &Option<Type>) {
-    match ty {
+fn write_extern_return_type_space(out: &mut OutFile, sig: &Signature, lang: Lang) {
+    match &sig.ret {
+        Some(_) if indirect_return(sig, out.types, lang) => write!(out, "void "),
         Some(Type::RustBox(ty) | Type::UniquePtr(ty)) => {
             write_type_space(out, &ty.inner);
             write!(out, "*");
@@ -1230,8 +1390,7 @@ fn write_extern_return_type_space(out: &mut OutFile, ty: &Option<Type>) {
             out.builtin.repr_fat = true;
             write!(out, "::rust::repr::Fat ");
         }
-        Some(ty) if out.types.needs_indirect_abi(ty) => write!(out, "void "),
-        _ => write_return_type(out, ty),
+        ty => write_return_type(out, ty),
     }
 }
 
@@ -1440,7 +1599,7 @@ fn write_generic_instantiations(out: &mut OutFile) {
     out.begin_block(Block::ExternC);
     for impl_key in out.types.impls.keys() {
         out.next_section();
-        match *impl_key {
+        match impl_key {
             ImplKey::RustBox(ident) => write_rust_box_extern(out, ident),
             ImplKey::RustVec(ident) => write_rust_vec_extern(out, ident),
             ImplKey::UniquePtr(ident) => write_unique_ptr(out, ident),
@@ -1454,7 +1613,7 @@ fn write_generic_instantiations(out: &mut OutFile) {
     out.begin_block(Block::Namespace("rust"));
     out.begin_block(Block::InlineNamespace("cxxbridge1"));
     for impl_key in out.types.impls.keys() {
-        match *impl_key {
+        match impl_key {
             ImplKey::RustBox(ident) => write_rust_box_impl(out, ident),
             ImplKey::RustVec(ident) => write_rust_vec_impl(out, ident),
             _ => {}
@@ -1464,11 +1623,13 @@ fn write_generic_instantiations(out: &mut OutFile) {
     out.end_block(Block::Namespace("rust"));
 }
 
-fn write_rust_box_extern(out: &mut OutFile, key: NamedImplKey) {
-    let resolve = out.types.resolve(&key);
+fn write_rust_box_extern(out: &mut OutFile, key: &NamedImplKey) {
+    let resolve = out.types.resolve(key);
     let inner = resolve.name.to_fully_qualified();
     let instance = resolve.name.to_symbol();
 
+    out.pragma.dollar_in_identifier = true;
+
     writeln!(
         out,
         "{} *cxxbridge1$box${}$alloc() noexcept;",
@@ -1486,12 +1647,13 @@ fn write_rust_box_extern(out: &mut OutFile, key: NamedImplKey) {
     );
 }
 
-fn write_rust_vec_extern(out: &mut OutFile, key: NamedImplKey) {
+fn write_rust_vec_extern(out: &mut OutFile, key: &NamedImplKey) {
     let element = key.rust;
     let inner = element.to_typename(out.types);
     let instance = element.to_mangled(out.types);
 
     out.include.cstddef = true;
+    out.pragma.dollar_in_identifier = true;
 
     writeln!(
         out,
@@ -1535,11 +1697,13 @@ fn write_rust_vec_extern(out: &mut OutFile, key: NamedImplKey) {
     );
 }
 
-fn write_rust_box_impl(out: &mut OutFile, key: NamedImplKey) {
-    let resolve = out.types.resolve(&key);
+fn write_rust_box_impl(out: &mut OutFile, key: &NamedImplKey) {
+    let resolve = out.types.resolve(key);
     let inner = resolve.name.to_fully_qualified();
     let instance = resolve.name.to_symbol();
 
+    out.pragma.dollar_in_identifier = true;
+
     writeln!(out, "template <>");
     begin_function_definition(out);
     writeln!(
@@ -1567,12 +1731,13 @@ fn write_rust_box_impl(out: &mut OutFile, key: NamedImplKey) {
     writeln!(out, "}}");
 }
 
-fn write_rust_vec_impl(out: &mut OutFile, key: NamedImplKey) {
+fn write_rust_vec_impl(out: &mut OutFile, key: &NamedImplKey) {
     let element = key.rust;
     let inner = element.to_typename(out.types);
     let instance = element.to_mangled(out.types);
 
     out.include.cstddef = true;
+    out.pragma.dollar_in_identifier = true;
 
     writeln!(out, "template <>");
     begin_function_definition(out);
@@ -1655,7 +1820,7 @@ fn write_rust_vec_impl(out: &mut OutFile, key: NamedImplKey) {
     writeln!(out, "}}");
 }
 
-fn write_unique_ptr(out: &mut OutFile, key: NamedImplKey) {
+fn write_unique_ptr(out: &mut OutFile, key: &NamedImplKey) {
     let ty = UniquePtr::Ident(key.rust);
     write_unique_ptr_common(out, ty);
 }
@@ -1664,6 +1829,9 @@ fn write_unique_ptr(out: &mut OutFile, key: NamedImplKey) {
 fn write_unique_ptr_common(out: &mut OutFile, ty: UniquePtr) {
     out.include.new = true;
     out.include.utility = true;
+    out.pragma.dollar_in_identifier = true;
+    out.pragma.missing_declarations = true;
+
     let inner = ty.to_typename(out.types);
     let instance = ty.to_mangled(out.types);
 
@@ -1676,25 +1844,12 @@ fn write_unique_ptr_common(out: &mut OutFile, ty: UniquePtr) {
         UniquePtr::CxxVector(_) => false,
     };
 
-    let conditional_delete = match ty {
-        UniquePtr::Ident(ident) => {
-            !out.types.structs.contains_key(ident) && !out.types.enums.contains_key(ident)
-        }
-        UniquePtr::CxxVector(_) => false,
-    };
-
-    if conditional_delete {
-        out.builtin.is_complete = true;
-        let definition = match ty {
-            UniquePtr::Ident(ty) => &out.types.resolve(ty).name.cxx,
-            UniquePtr::CxxVector(_) => unreachable!(),
-        };
-        writeln!(
-            out,
-            "static_assert(::rust::detail::is_complete<{}>::value, \"definition of {} is required\");",
-            inner, definition,
-        );
-    }
+    out.builtin.is_complete = true;
+    writeln!(
+        out,
+        "static_assert(::rust::detail::is_complete<::std::remove_extent<{}>::type>::value, \"definition of `{}` is required\");",
+        inner, inner,
+    );
     writeln!(
         out,
         "static_assert(sizeof(::std::unique_ptr<{}>) == sizeof(void *), \"\");",
@@ -1736,7 +1891,7 @@ fn write_unique_ptr_common(out: &mut OutFile, ty: UniquePtr) {
     begin_function_definition(out);
     writeln!(
         out,
-        "void cxxbridge1$unique_ptr${}$raw(::std::unique_ptr<{}> *ptr, {} *raw) noexcept {{",
+        "void cxxbridge1$unique_ptr${}$raw(::std::unique_ptr<{}> *ptr, ::std::unique_ptr<{}>::pointer raw) noexcept {{",
         instance, inner, inner,
     );
     writeln!(out, "  ::new (ptr) ::std::unique_ptr<{}>(raw);", inner);
@@ -1745,7 +1900,7 @@ fn write_unique_ptr_common(out: &mut OutFile, ty: UniquePtr) {
     begin_function_definition(out);
     writeln!(
         out,
-        "{} const *cxxbridge1$unique_ptr${}$get(::std::unique_ptr<{}> const &ptr) noexcept {{",
+        "::std::unique_ptr<{}>::element_type const *cxxbridge1$unique_ptr${}$get(::std::unique_ptr<{}> const &ptr) noexcept {{",
         inner, instance, inner,
     );
     writeln!(out, "  return ptr.get();");
@@ -1754,7 +1909,7 @@ fn write_unique_ptr_common(out: &mut OutFile, ty: UniquePtr) {
     begin_function_definition(out);
     writeln!(
         out,
-        "{} *cxxbridge1$unique_ptr${}$release(::std::unique_ptr<{}> &ptr) noexcept {{",
+        "::std::unique_ptr<{}>::pointer cxxbridge1$unique_ptr${}$release(::std::unique_ptr<{}> &ptr) noexcept {{",
         inner, instance, inner,
     );
     writeln!(out, "  return ptr.release();");
@@ -1766,20 +1921,16 @@ fn write_unique_ptr_common(out: &mut OutFile, ty: UniquePtr) {
         "void cxxbridge1$unique_ptr${}$drop(::std::unique_ptr<{}> *ptr) noexcept {{",
         instance, inner,
     );
-    if conditional_delete {
-        out.builtin.deleter_if = true;
-        writeln!(
-            out,
-            "  ::rust::deleter_if<::rust::detail::is_complete<{}>::value>{{}}(ptr);",
-            inner,
-        );
-    } else {
-        writeln!(out, "  ptr->~unique_ptr();");
-    }
+    out.builtin.deleter_if = true;
+    writeln!(
+        out,
+        "  ::rust::deleter_if<::rust::detail::is_complete<{}>::value>{{}}(ptr);",
+        inner,
+    );
     writeln!(out, "}}");
 }
 
-fn write_shared_ptr(out: &mut OutFile, key: NamedImplKey) {
+fn write_shared_ptr(out: &mut OutFile, key: &NamedImplKey) {
     let ident = key.rust;
     let resolve = out.types.resolve(ident);
     let inner = resolve.name.to_fully_qualified();
@@ -1787,6 +1938,8 @@ fn write_shared_ptr(out: &mut OutFile, key: NamedImplKey) {
 
     out.include.new = true;
     out.include.utility = true;
+    out.pragma.dollar_in_identifier = true;
+    out.pragma.missing_declarations = true;
 
     // Some aliases are to opaque types; some are to trivial types. We can't
     // know at code generation time, so we generate both C++ and Rust side
@@ -1832,6 +1985,21 @@ fn write_shared_ptr(out: &mut OutFile, key: NamedImplKey) {
         writeln!(out, "}}");
     }
 
+    out.builtin.shared_ptr = true;
+    begin_function_definition(out);
+    writeln!(
+        out,
+        "bool cxxbridge1$shared_ptr${}$raw(::std::shared_ptr<{}> *ptr, ::std::shared_ptr<{}>::element_type *raw) noexcept {{",
+        instance, inner, inner,
+    );
+    writeln!(
+        out,
+        "  ::new (ptr) ::rust::shared_ptr_if_destructible<{}>(raw);",
+        inner,
+    );
+    writeln!(out, "  return ::rust::is_destructible<{}>::value;", inner);
+    writeln!(out, "}}");
+
     begin_function_definition(out);
     writeln!(
         out,
@@ -1844,7 +2012,7 @@ fn write_shared_ptr(out: &mut OutFile, key: NamedImplKey) {
     begin_function_definition(out);
     writeln!(
         out,
-        "{} const *cxxbridge1$shared_ptr${}$get(::std::shared_ptr<{}> const &self) noexcept {{",
+        "::std::shared_ptr<{}>::element_type const *cxxbridge1$shared_ptr${}$get(::std::shared_ptr<{}> const &self) noexcept {{",
         inner, instance, inner,
     );
     writeln!(out, "  return self.get();");
@@ -1860,13 +2028,15 @@ fn write_shared_ptr(out: &mut OutFile, key: NamedImplKey) {
     writeln!(out, "}}");
 }
 
-fn write_weak_ptr(out: &mut OutFile, key: NamedImplKey) {
-    let resolve = out.types.resolve(&key);
+fn write_weak_ptr(out: &mut OutFile, key: &NamedImplKey) {
+    let resolve = out.types.resolve(key);
     let inner = resolve.name.to_fully_qualified();
     let instance = resolve.name.to_symbol();
 
     out.include.new = true;
     out.include.utility = true;
+    out.pragma.dollar_in_identifier = true;
+    out.pragma.missing_declarations = true;
 
     writeln!(
         out,
@@ -1929,7 +2099,7 @@ fn write_weak_ptr(out: &mut OutFile, key: NamedImplKey) {
     writeln!(out, "}}");
 }
 
-fn write_cxx_vector(out: &mut OutFile, key: NamedImplKey) {
+fn write_cxx_vector(out: &mut OutFile, key: &NamedImplKey) {
     let element = key.rust;
     let inner = element.to_typename(out.types);
     let instance = element.to_mangled(out.types);
@@ -1937,6 +2107,9 @@ fn write_cxx_vector(out: &mut OutFile, key: NamedImplKey) {
     out.include.cstddef = true;
     out.include.utility = true;
     out.builtin.destroy = true;
+    out.builtin.vector = true;
+    out.pragma.dollar_in_identifier = true;
+    out.pragma.missing_declarations = true;
 
     begin_function_definition(out);
     writeln!(
@@ -1956,6 +2129,15 @@ fn write_cxx_vector(out: &mut OutFile, key: NamedImplKey) {
     writeln!(out, "  return s.size();");
     writeln!(out, "}}");
 
+    begin_function_definition(out);
+    writeln!(
+        out,
+        "::std::size_t cxxbridge1$std$vector${}$capacity(::std::vector<{}> const &s) noexcept {{",
+        instance, inner,
+    );
+    writeln!(out, "  return s.capacity();");
+    writeln!(out, "}}");
+
     begin_function_definition(out);
     writeln!(
         out,
@@ -1965,6 +2147,19 @@ fn write_cxx_vector(out: &mut OutFile, key: NamedImplKey) {
     writeln!(out, "  return &(*s)[pos];");
     writeln!(out, "}}");
 
+    begin_function_definition(out);
+    writeln!(
+        out,
+        "bool cxxbridge1$std$vector${}$reserve(::std::vector<{}> *s, ::std::size_t new_cap) noexcept {{",
+        instance, inner,
+    );
+    writeln!(
+        out,
+        "  return ::rust::if_move_constructible<{}>::reserve(*s, new_cap);",
+        inner,
+    );
+    writeln!(out, "}}");
+
     if out.types.is_maybe_trivial(element) {
         begin_function_definition(out);
         writeln!(
