@@ -1,13 +1,12 @@
 use crate::{
     cache::{kernel_params, DivCeil, KernelParams, CACHE_INFO},
     gemv, gevv,
-    microkernel::MicroKernelFn,
+    microkernel::{HMicroKernelFn, MicroKernelFn},
     pack_operands::{pack_lhs, pack_rhs},
     simd::MixedSimd,
     Parallelism, Ptr,
 };
 use core::sync::atomic::{AtomicUsize, Ordering};
-use dyn_stack::GlobalMemBuffer;
 use dyn_stack::{DynStack, StackReq};
 #[cfg(feature = "f16")]
 use half::f16;
@@ -58,7 +57,7 @@ pub const CACHELINE_ALIGN: usize = {
 
 #[cfg(feature = "std")]
 thread_local! {
-    pub static L2_SLAB: core::cell::RefCell<GlobalMemBuffer> = core::cell::RefCell::new(GlobalMemBuffer::new(
+    pub static L2_SLAB: core::cell::RefCell<dyn_stack::MemBuffer> = core::cell::RefCell::new(dyn_stack::MemBuffer::new(
         StackReq::new_aligned::<u8>(CACHE_INFO[1].cache_bytes, CACHELINE_ALIGN)
     ));
 }
@@ -189,6 +188,8 @@ pub unsafe fn gemm_basic_generic<
     const MR: usize,
     const NR: usize,
     const MR_DIV_N: usize,
+    const H_M: usize,
+    const H_N: usize,
 >(
     simd: S,
     m: usize,
@@ -211,6 +212,7 @@ pub unsafe fn gemm_basic_generic<
     conj_rhs: bool,
     mul_add: impl Copy + Fn(T, T, T) -> T,
     dispatcher: &[[MicroKernelFn<T>; NR]; MR_DIV_N],
+    horizontal_dispatcher: &[[HMicroKernelFn<T>; H_N]; H_M],
     _requires_row_major_rhs: bool,
     parallelism: Parallelism,
 ) {
@@ -255,6 +257,59 @@ pub unsafe fn gemm_basic_generic<
         return;
     }
 
+    if (H_M > 0 && H_N > 0) && (!conj_dst && lhs_cs == 1 && rhs_rs == 1 && (m * n) <= 16 * 16) {
+        let kc = 1024;
+        let mut depth = 0;
+        let mut conj_dst = conj_dst;
+        while depth < k {
+            let kb = Ord::min(kc, k - depth);
+            let alpha_status = if alpha.is_zero() {
+                0
+            } else if alpha.is_one() {
+                1
+            } else {
+                2
+            };
+
+            let mut col = 0;
+            while col < n {
+                let nb = Ord::min(H_N, n - col);
+
+                let mut row = 0;
+                while row < m {
+                    let mb = Ord::min(H_M, m - row);
+
+                    horizontal_dispatcher[mb - 1][nb - 1](
+                        kb,
+                        dst.wrapping_offset(dst_rs * row as isize + dst_cs * col as isize),
+                        lhs.wrapping_offset(lhs_rs * row as isize + depth as isize),
+                        rhs.wrapping_offset(rhs_cs * col as isize + depth as isize),
+                        dst_cs,
+                        dst_rs,
+                        lhs_rs,
+                        rhs_cs,
+                        alpha,
+                        beta,
+                        alpha_status,
+                        conj_dst,
+                        conj_lhs,
+                        conj_rhs,
+                    );
+
+                    row += mb;
+                }
+
+                col += nb;
+            }
+
+            alpha = T::one();
+            conj_dst = false;
+            depth += kb;
+        }
+
+        return;
+    }
+
     if !conj_dst && !conj_lhs && !conj_rhs {
         if k <= 2 {
             gevv::gevv(
@@ -340,7 +395,18 @@ pub unsafe fn gemm_basic_generic<
     };
 
     #[cfg(feature = "rayon")]
-    let threading_threshold = get_threading_threshold();
+    let threading_threshold = {
+        use core::any::TypeId;
+        let is_c32 = TypeId::of::<c32>() == TypeId::of::<T>();
+        let is_c64 = TypeId::of::<c64>() == TypeId::of::<T>();
+        if is_c32 {
+            get_threading_threshold() / 4
+        } else if is_c64 {
+            get_threading_threshold() / 16
+        } else {
+            get_threading_threshold()
+        }
+    };
 
     #[cfg(target_arch = "aarch64")]
     let do_pack_rhs = _requires_row_major_rhs || m > get_rhs_packing_threshold() * MR;
@@ -368,13 +434,13 @@ pub unsafe fn gemm_basic_generic<
             },
             simd_align,
         );
-        Some(GlobalMemBuffer::new(rhs_req.and(lhs_req)))
+        Some(dyn_stack::MemBuffer::new(rhs_req.and(lhs_req)))
     } else {
         None
     };
 
     #[cfg(not(feature = "std"))]
-    let mut l2_slab = GlobalMemBuffer::new(StackReq::new_aligned::<T>(
+    let mut l2_slab = dyn_stack::MemBuffer::new(StackReq::new_aligned::<T>(
         packed_lhs_stride * (mc / MR),
         simd_align,
     ));
@@ -743,7 +809,7 @@ pub unsafe fn gemm_basic_generic<
                     L2_SLAB.with(|mem| {
                         let mut mem = mem.borrow_mut();
                         let stack = DynStack::new(&mut mem);
-                        let (mut packed_lhs_storage, _) = stack
+                        let (packed_lhs_storage, _) = stack
                             .make_aligned_uninit::<T>(packed_lhs_stride * (mc / MR), simd_align);
                         let packed_lhs = Ptr(packed_lhs_storage.as_mut_ptr() as *mut T);
                         func(tid, packed_lhs);
@@ -753,7 +819,7 @@ pub unsafe fn gemm_basic_generic<
                 #[cfg(not(feature = "std"))]
                 let mut func = |tid: usize| {
                     let stack = DynStack::new(&mut l2_slab);
-                    let (mut packed_lhs_storage, _) =
+                    let (packed_lhs_storage, _) =
                         stack.make_aligned_uninit::<T>(packed_lhs_stride * (mc / MR), simd_align);
                     let packed_lhs = Ptr(packed_lhs_storage.as_mut_ptr() as *mut T);
                     func(tid, packed_lhs);
@@ -812,7 +878,16 @@ macro_rules! __inject_mod {
                 conj_rhs: bool,
                 parallelism: $crate::Parallelism,
             ) {
-                $crate::gemm::gemm_basic_generic::<_, $ty, N, { MR_DIV_N * N }, NR, MR_DIV_N>(
+                $crate::gemm::gemm_basic_generic::<
+                    _,
+                    $ty,
+                    N,
+                    { MR_DIV_N * N },
+                    NR,
+                    MR_DIV_N,
+                    H_M,
+                    H_N,
+                >(
                     <$crate::simd::$simd as MixedSimd<$ty, $ty, $ty, $ty>>::try_new().unwrap(),
                     m,
                     n,
@@ -834,6 +909,7 @@ macro_rules! __inject_mod {
                     conj_rhs,
                     |a, b, c| a * b + c,
                     &UKR,
+                    &H_UKR,
                     $requires_packed_rhs,
                     parallelism,
                 );
@@ -874,7 +950,7 @@ macro_rules! __inject_mod_cplx {
                     conj_rhs: bool,
                     parallelism: $crate::Parallelism,
                     ) {
-                    $crate::gemm::gemm_basic_generic::<_, _, N, { CPLX_MR_DIV_N * N }, CPLX_NR, CPLX_MR_DIV_N>(
+                    $crate::gemm::gemm_basic_generic::<_, _, N, { CPLX_MR_DIV_N * N }, CPLX_NR, CPLX_MR_DIV_N, H_CPLX_M, H_CPLX_N>(
                         <$crate::simd::$simd as MixedSimd<T, T, T, T>>::try_new().unwrap(),
                         m,
                         n,
@@ -896,6 +972,7 @@ macro_rules! __inject_mod_cplx {
                         conj_rhs,
                         |a, b, c| a * b + c,
                         &CPLX_UKR,
+                        &H_CPLX_UKR,
                         false,
                         parallelism,
                         );
