@@ -1,5 +1,4 @@
 use std::collections::{HashMap, VecDeque};
-use std::convert::From;
 use std::iter::Peekable;
 use std::str::FromStr;
 
@@ -62,6 +61,7 @@ impl Subexpression {
                 inverse: None,
                 block_param: None,
                 block: false,
+                chain: false,
             }))),
         }
     }
@@ -135,6 +135,7 @@ pub struct HelperTemplate {
     pub template: Option<Template>,
     pub inverse: Option<Template>,
     pub block: bool,
+    pub chain: bool,
 }
 
 impl HelperTemplate {
@@ -147,6 +148,20 @@ impl HelperTemplate {
             block,
             template: None,
             inverse: None,
+            chain: false,
+        }
+    }
+
+    pub fn new_chain(exp: ExpressionSpec, block: bool) -> HelperTemplate {
+        HelperTemplate {
+            name: exp.name,
+            params: exp.params,
+            hash: exp.hash,
+            block_param: exp.block_param,
+            block,
+            template: None,
+            inverse: None,
+            chain: true,
         }
     }
 
@@ -160,12 +175,88 @@ impl HelperTemplate {
             template: None,
             inverse: None,
             block: false,
+            chain: false,
         }
     }
 
     pub(crate) fn is_name_only(&self) -> bool {
         !self.block && self.params.is_empty() && self.hash.is_empty()
     }
+
+    fn insert_inverse_node(&mut self, mut node: Box<HelperTemplate>) {
+        // Create a list in "inverse" member to hold the else-chain.
+        // Here we create the new template to save the else-chain node.
+        // The template render could render it successfully without any code add.
+        let mut new_chain_template = Template::new();
+        node.inverse = self.inverse.take();
+        new_chain_template.elements.push(HelperBlock(node));
+        self.inverse = Some(new_chain_template);
+    }
+
+    fn ref_chain_head_mut(&mut self) -> Option<&mut Box<HelperTemplate>> {
+        if self.chain {
+            if let Some(inverse_tmpl) = &mut self.inverse {
+                assert_eq!(inverse_tmpl.elements.len(), 1);
+                if let HelperBlock(helper) = &mut inverse_tmpl.elements[0] {
+                    return Some(helper);
+                }
+            }
+        }
+        None
+    }
+
+    fn set_chain_template(&mut self, tmpl: Option<Template>) {
+        if let Some(hepler) = self.ref_chain_head_mut() {
+            hepler.template = tmpl;
+        } else {
+            self.template = tmpl;
+        }
+    }
+
+    fn revert_chain_and_set(&mut self, inverse: Option<Template>) {
+        if self.chain {
+            let mut prev = None;
+
+            if let Some(head) = self.ref_chain_head_mut() {
+                if head.template.is_some() {
+                    // Here the prev will hold the head inverse template.
+                    // It will be set when reverse the chain.
+                    prev = inverse;
+                } else {
+                    // If the head already has template. set the inverse template.
+                    head.template = inverse;
+                }
+            }
+
+            // Reverse the else chain, to the normal list order.
+            while let Some(mut node) = self.inverse.take() {
+                assert_eq!(node.elements.len(), 1);
+                if let HelperBlock(c) = &mut node.elements[0] {
+                    self.inverse = c.inverse.take();
+                    c.inverse = prev;
+                    prev = Some(node);
+                }
+            }
+
+            self.inverse = prev;
+        } else {
+            // If the helper has no else chain.
+            // set the template to self.
+            if self.template.is_some() {
+                self.inverse = inverse;
+            } else {
+                self.template = inverse;
+            }
+        }
+    }
+
+    fn set_chained(&mut self) {
+        self.chain = true;
+    }
+
+    pub fn is_chained(&self) -> bool {
+        self.chain
+    }
 }
 
 #[derive(PartialEq, Eq, Clone, Debug)]
@@ -211,7 +302,7 @@ impl Parameter {
         if let Some(name) = self.as_name() {
             name.to_owned()
         } else {
-            format!("{:?}", self)
+            format!("{self:?}")
         }
     }
 }
@@ -277,7 +368,7 @@ impl Template {
         I: Iterator<Item = Pair<'a, Rule>>,
     {
         let mut param = it.next().unwrap();
-        if param.as_rule() == Rule::param {
+        if param.as_rule() == Rule::helper_parameter {
             param = it.next().unwrap();
         }
         let param_rule = param.as_rule();
@@ -391,7 +482,7 @@ impl Template {
         let mut omit_pro_ws = false;
         let mut block_param = None;
 
-        if it.peek().unwrap().as_rule() == Rule::pre_whitespace_omitter {
+        if it.peek().unwrap().as_rule() == Rule::leading_tilde_to_omit_whitespace {
             omit_pre_ws = true;
             it.next();
         }
@@ -416,7 +507,7 @@ impl Template {
             it.next();
 
             match rule {
-                Rule::param => {
+                Rule::helper_parameter => {
                     params.push(Template::parse_param(source, it.by_ref(), end)?);
                 }
                 Rule::hash => {
@@ -426,7 +517,7 @@ impl Template {
                 Rule::block_param => {
                     block_param = Some(Template::parse_block_param(source, it.by_ref(), end));
                 }
-                Rule::pro_whitespace_omitter => {
+                Rule::trailing_tilde_to_omit_whitespace => {
                     omit_pro_ws = true;
                 }
                 _ => {}
@@ -445,7 +536,7 @@ impl Template {
     fn remove_previous_whitespace(template_stack: &mut VecDeque<Template>) {
         let t = template_stack.front_mut().unwrap();
         if let Some(RawString(ref mut text)) = t.elements.last_mut() {
-            *text = text.trim_end().to_owned();
+            text.trim_end().to_owned().clone_into(text);
         }
     }
 
@@ -475,9 +566,9 @@ impl Template {
                 // check the last element before current
                 if let Some(RawString(ref mut text)) = t.elements.last_mut() {
                     // trim leading space for standalone statement
-                    *text = text
-                        .trim_end_matches(support::str::whitespace_matcher)
-                        .to_owned();
+                    text.trim_end_matches(support::str::whitespace_matcher)
+                        .to_owned()
+                        .clone_into(text);
                 }
             }
 
@@ -549,9 +640,11 @@ impl Template {
                 LineColLocation::Pos(line_col) => line_col,
                 LineColLocation::Span(line_col, _) => line_col,
             };
-            TemplateError::of(TemplateErrorReason::InvalidSyntax)
-                .at(source, line_no, col_no)
-                .in_template(options.name())
+            TemplateError::of(TemplateErrorReason::InvalidSyntax(
+                e.variant.message().to_string(),
+            ))
+            .at(source, line_no, col_no)
+            .in_template(options.name())
         })?;
 
         // dbg!(parser_queue.clone().flatten());
@@ -674,9 +767,13 @@ impl Template {
                         let t = template_stack.front_mut().unwrap();
                         t.mapping.push(TemplateMapping(line_no, col_no));
                     }
-                    Rule::invert_tag => {
+                    Rule::invert_tag | Rule::invert_chain_tag => {
                         // hack: invert_tag structure is similar to ExpressionSpec, so I
                         // use it here to represent the data
+
+                        if rule == Rule::invert_chain_tag {
+                            let _ = Template::parse_name(source, &mut it, span.end())?;
+                        }
                         let exp = Template::parse_expression(source, it.by_ref(), span.end())?;
 
                         if exp.omit_pre_ws {
@@ -695,8 +792,17 @@ impl Template {
 
                         let t = template_stack.pop_front().unwrap();
                         let h = helper_stack.front_mut().unwrap();
-                        h.template = Some(t);
+
+                        if rule == Rule::invert_chain_tag {
+                            h.set_chained();
+                        }
+
+                        h.set_chain_template(Some(t));
+                        if rule == Rule::invert_chain_tag {
+                            h.insert_inverse_node(Box::new(HelperTemplate::new_chain(exp, true)));
+                        }
                     }
+
                     Rule::raw_block_text => {
                         let mut t = Template::new();
                         t.push_element(
@@ -784,11 +890,8 @@ impl Template {
                                 let close_tag_name = exp.name.as_name();
                                 if h.name.as_name() == close_tag_name {
                                     let prev_t = template_stack.pop_front().unwrap();
-                                    if h.template.is_some() {
-                                        h.inverse = Some(prev_t);
-                                    } else {
-                                        h.template = Some(prev_t);
-                                    }
+                                    h.revert_chain_and_set(Some(prev_t));
+
                                     let t = template_stack.front_mut().unwrap();
                                     t.elements.push(HelperBlock(Box::new(h)));
                                 } else {
@@ -1032,9 +1135,11 @@ mod test {
 
         let terr = Template::compile(source).unwrap_err();
 
-        assert!(matches!(terr.reason(), TemplateErrorReason::InvalidSyntax));
-        assert_eq!(terr.line_no.unwrap(), 4);
-        assert_eq!(terr.column_no.unwrap(), 5);
+        assert!(matches!(
+            terr.reason(),
+            TemplateErrorReason::InvalidSyntax(_)
+        ));
+        assert_eq!(terr.pos(), Some((4, 5)));
     }
 
     #[test]
@@ -1148,10 +1253,16 @@ mod test {
         let sources = ["{{invalid", "{{{invalid", "{{invalid}", "{{!hello"];
         for s in sources.iter() {
             let result = Template::compile(s.to_owned());
-            assert!(matches!(
-                *result.unwrap_err().reason(),
-                TemplateErrorReason::InvalidSyntax
-            ));
+            let err = result.expect_err("expected a syntax error");
+            let syntax_error_msg = match err.reason() {
+                TemplateErrorReason::InvalidSyntax(s) => s,
+                _ => panic!("InvalidSyntax expected"),
+            };
+            assert!(
+                syntax_error_msg.contains("expected identifier"),
+                "{}",
+                syntax_error_msg
+            );
         }
     }
 
@@ -1332,6 +1443,6 @@ mod test {
         let s = "{{#>(X)}}{{/X}}";
         let result = Template::compile(s);
         assert!(result.is_err());
-        assert_eq!("decorator \"Subexpression(Subexpression { element: Expression(HelperTemplate { name: Path(Relative(([Named(\\\"X\\\")], \\\"X\\\"))), params: [], hash: {}, block_param: None, template: None, inverse: None, block: false }) })\" was opened, but \"X\" is closing", format!("{}", result.unwrap_err().reason()));
+        assert_eq!("decorator \"Subexpression(Subexpression { element: Expression(HelperTemplate { name: Path(Relative(([Named(\\\"X\\\")], \\\"X\\\"))), params: [], hash: {}, block_param: None, template: None, inverse: None, block: false, chain: false }) })\" was opened, but \"X\" is closing", format!("{}", result.unwrap_err().reason()));
     }
 }
