@@ -10,13 +10,16 @@ mod handles;
 mod interface;
 mod r#type;
 
+use alloc::{boxed::Box, string::String, vec, vec::Vec};
+use core::ops;
+
+use bit_set::BitSet;
+
 use crate::{
     arena::{Handle, HandleSet},
     proc::{ExpressionKindTracker, LayoutError, Layouter, TypeResolution},
     FastHashSet,
 };
-use bit_set::BitSet;
-use std::ops;
 
 //TODO: analyze the model at the same time as we validate it,
 // merge the corresponding matches over expressions and statements.
@@ -28,7 +31,7 @@ pub use expression::{check_literal_value, LiteralError};
 pub use expression::{ConstExpressionError, ExpressionError};
 pub use function::{CallError, FunctionError, LocalVariableError};
 pub use interface::{EntryPointError, GlobalVariableError, VaryingError};
-pub use r#type::{Disalignment, TypeError, TypeFlags, WidthError};
+pub use r#type::{Disalignment, PushConstantError, TypeError, TypeFlags, WidthError};
 
 use self::handles::InvalidHandleError;
 
@@ -81,56 +84,58 @@ bitflags::bitflags! {
         /// Support for [`AddressSpace::PushConstant`][1].
         ///
         /// [1]: crate::AddressSpace::PushConstant
-        const PUSH_CONSTANT = 0x1;
+        const PUSH_CONSTANT = 1 << 0;
         /// Float values with width = 8.
-        const FLOAT64 = 0x2;
+        const FLOAT64 = 1 << 1;
         /// Support for [`BuiltIn::PrimitiveIndex`][1].
         ///
         /// [1]: crate::BuiltIn::PrimitiveIndex
-        const PRIMITIVE_INDEX = 0x4;
+        const PRIMITIVE_INDEX = 1 << 2;
         /// Support for non-uniform indexing of sampled textures and storage buffer arrays.
-        const SAMPLED_TEXTURE_AND_STORAGE_BUFFER_ARRAY_NON_UNIFORM_INDEXING = 0x8;
-        /// Support for non-uniform indexing of uniform buffers and storage texture arrays.
-        const UNIFORM_BUFFER_AND_STORAGE_TEXTURE_ARRAY_NON_UNIFORM_INDEXING = 0x10;
+        const SAMPLED_TEXTURE_AND_STORAGE_BUFFER_ARRAY_NON_UNIFORM_INDEXING = 1 << 3;
+        /// Support for non-uniform indexing of storage texture arrays.
+        const STORAGE_TEXTURE_ARRAY_NON_UNIFORM_INDEXING = 1 << 4;
+        /// Support for non-uniform indexing of uniform buffer arrays.
+        const UNIFORM_BUFFER_ARRAY_NON_UNIFORM_INDEXING = 1 << 5;
         /// Support for non-uniform indexing of samplers.
-        const SAMPLER_NON_UNIFORM_INDEXING = 0x20;
+        const SAMPLER_NON_UNIFORM_INDEXING = 1 << 6;
         /// Support for [`BuiltIn::ClipDistance`].
         ///
         /// [`BuiltIn::ClipDistance`]: crate::BuiltIn::ClipDistance
-        const CLIP_DISTANCE = 0x40;
+        const CLIP_DISTANCE = 1 << 7;
         /// Support for [`BuiltIn::CullDistance`].
         ///
         /// [`BuiltIn::CullDistance`]: crate::BuiltIn::CullDistance
-        const CULL_DISTANCE = 0x80;
+        const CULL_DISTANCE = 1 << 8;
         /// Support for 16-bit normalized storage texture formats.
-        const STORAGE_TEXTURE_16BIT_NORM_FORMATS = 0x100;
+        const STORAGE_TEXTURE_16BIT_NORM_FORMATS = 1 << 9;
         /// Support for [`BuiltIn::ViewIndex`].
         ///
         /// [`BuiltIn::ViewIndex`]: crate::BuiltIn::ViewIndex
-        const MULTIVIEW = 0x200;
+        const MULTIVIEW = 1 << 10;
         /// Support for `early_depth_test`.
-        const EARLY_DEPTH_TEST = 0x400;
+        const EARLY_DEPTH_TEST = 1 << 11;
         /// Support for [`BuiltIn::SampleIndex`] and [`Sampling::Sample`].
         ///
         /// [`BuiltIn::SampleIndex`]: crate::BuiltIn::SampleIndex
         /// [`Sampling::Sample`]: crate::Sampling::Sample
-        const MULTISAMPLED_SHADING = 0x800;
+        const MULTISAMPLED_SHADING = 1 << 12;
         /// Support for ray queries and acceleration structures.
-        const RAY_QUERY = 0x1000;
+        const RAY_QUERY = 1 << 13;
         /// Support for generating two sources for blending from fragment shaders.
-        const DUAL_SOURCE_BLENDING = 0x2000;
+        const DUAL_SOURCE_BLENDING = 1 << 14;
         /// Support for arrayed cube textures.
-        const CUBE_ARRAY_TEXTURES = 0x4000;
+        const CUBE_ARRAY_TEXTURES = 1 << 15;
         /// Support for 64-bit signed and unsigned integers.
-        const SHADER_INT64 = 0x8000;
+        const SHADER_INT64 = 1 << 16;
         /// Support for subgroup operations.
         /// Implies support for subgroup operations in both fragment and compute stages,
         /// but not necessarily in the vertex stage, which requires [`Capabilities::SUBGROUP_VERTEX_STAGE`].
-        const SUBGROUP = 0x10000;
+        const SUBGROUP = 1 << 17;
         /// Support for subgroup barriers.
-        const SUBGROUP_BARRIER = 0x20000;
+        const SUBGROUP_BARRIER = 1 << 18;
         /// Support for subgroup operations in the vertex stage.
-        const SUBGROUP_VERTEX_STAGE = 0x40000;
+        const SUBGROUP_VERTEX_STAGE = 1 << 19;
         /// Support for [`AtomicFunction::Min`] and [`AtomicFunction::Max`] on
         /// 64-bit integers in the [`Storage`] address space, when the return
         /// value is not used.
@@ -140,9 +145,26 @@ bitflags::bitflags! {
         /// [`AtomicFunction::Min`]: crate::AtomicFunction::Min
         /// [`AtomicFunction::Max`]: crate::AtomicFunction::Max
         /// [`Storage`]: crate::AddressSpace::Storage
-        const SHADER_INT64_ATOMIC_MIN_MAX = 0x80000;
+        const SHADER_INT64_ATOMIC_MIN_MAX = 1 << 20;
         /// Support for all atomic operations on 64-bit integers.
-        const SHADER_INT64_ATOMIC_ALL_OPS = 0x100000;
+        const SHADER_INT64_ATOMIC_ALL_OPS = 1 << 21;
+        /// Support for [`AtomicFunction::Add`], [`AtomicFunction::Sub`],
+        /// and [`AtomicFunction::Exchange { compare: None }`] on 32-bit floating-point numbers
+        /// in the [`Storage`] address space.
+        ///
+        /// [`AtomicFunction::Add`]: crate::AtomicFunction::Add
+        /// [`AtomicFunction::Sub`]: crate::AtomicFunction::Sub
+        /// [`AtomicFunction::Exchange { compare: None }`]: crate::AtomicFunction::Exchange
+        /// [`Storage`]: crate::AddressSpace::Storage
+        const SHADER_FLOAT32_ATOMIC = 1 << 22;
+        /// Support for atomic operations on images.
+        const TEXTURE_ATOMIC = 1 << 23;
+        /// Support for atomic operations on 64-bit images.
+        const TEXTURE_INT64_ATOMIC = 1 << 24;
+        /// Support for ray queries returning vertex position
+        const RAY_HIT_VERTEX_POSITION = 1 << 25;
+        /// Support for 16-bit floating-point types.
+        const SHADER_FLOAT16 = 1 << 26;
     }
 }
 
@@ -215,7 +237,7 @@ bitflags::bitflags! {
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Default)]
 #[cfg_attr(feature = "serialize", derive(serde::Serialize))]
 #[cfg_attr(feature = "deserialize", derive(serde::Deserialize))]
 pub struct ModuleInfo {
@@ -255,13 +277,17 @@ pub struct Validator {
     types: Vec<r#type::TypeInfo>,
     layouter: Layouter,
     location_mask: BitSet,
+    blend_src_mask: BitSet,
     ep_resource_bindings: FastHashSet<crate::ResourceBinding>,
     #[allow(dead_code)]
     switch_values: FastHashSet<crate::SwitchValue>,
     valid_expression_list: Vec<Handle<crate::Expression>>,
     valid_expression_set: HandleSet<crate::Expression>,
     override_ids: FastHashSet<u16>,
-    allow_overrides: bool,
+
+    /// Treat overrides whose initializers are not fully-evaluated
+    /// constant expressions as errors.
+    overrides_resolved: bool,
 
     /// A checklist of expressions that must be visited by a specific kind of
     /// statement.
@@ -312,6 +338,13 @@ pub enum OverrideError {
     TypeNotScalar,
     #[error("Override declarations are not allowed")]
     NotAllowed,
+    #[error("Override is uninitialized")]
+    UninitializedOverride,
+    #[error("Constant expression {handle:?} is invalid")]
+    ConstExpression {
+        handle: Handle<crate::Expression>,
+        source: ConstExpressionError,
+    },
 }
 
 #[derive(Clone, Debug, thiserror::Error)]
@@ -332,6 +365,8 @@ pub enum ValidationError {
         handle: Handle<crate::Expression>,
         source: ConstExpressionError,
     },
+    #[error("Array size expression {handle:?} is not strictly positive")]
+    ArraySizeError { handle: Handle<crate::Expression> },
     #[error("Constant {handle:?} '{name}' is invalid")]
     Constant {
         handle: Handle<crate::Constant>,
@@ -383,8 +418,8 @@ impl crate::TypeInner {
             Self::Array { .. }
             | Self::Image { .. }
             | Self::Sampler { .. }
-            | Self::AccelerationStructure
-            | Self::RayQuery
+            | Self::AccelerationStructure { .. }
+            | Self::RayQuery { .. }
             | Self::BindingArray { .. } => false,
         }
     }
@@ -445,12 +480,13 @@ impl Validator {
             types: Vec::new(),
             layouter: Layouter::default(),
             location_mask: BitSet::new(),
+            blend_src_mask: BitSet::new(),
             ep_resource_bindings: FastHashSet::default(),
             switch_values: FastHashSet::default(),
             valid_expression_list: Vec::new(),
             valid_expression_set: HandleSet::new(),
             override_ids: FastHashSet::default(),
-            allow_overrides: true,
+            overrides_resolved: false,
             needs_visit: HandleSet::new(),
         }
     }
@@ -470,6 +506,7 @@ impl Validator {
         self.types.clear();
         self.layouter.clear();
         self.location_mask.clear();
+        self.blend_src_mask.clear();
         self.ep_resource_bindings.clear();
         self.switch_values.clear();
         self.valid_expression_list.clear();
@@ -510,16 +547,8 @@ impl Validator {
         gctx: crate::proc::GlobalCtx,
         mod_info: &ModuleInfo,
     ) -> Result<(), OverrideError> {
-        if !self.allow_overrides {
-            return Err(OverrideError::NotAllowed);
-        }
-
         let o = &gctx.overrides[handle];
 
-        if o.name.is_none() && o.id.is_none() {
-            return Err(OverrideError::MissingNameAndID);
-        }
-
         if let Some(id) = o.id {
             if !self.override_ids.insert(id) {
                 return Err(OverrideError::DuplicateID);
@@ -537,6 +566,7 @@ impl Validator {
                 crate::Scalar::BOOL
                 | crate::Scalar::I32
                 | crate::Scalar::U32
+                | crate::Scalar::F16
                 | crate::Scalar::F32
                 | crate::Scalar::F64,
             ) => {}
@@ -548,6 +578,8 @@ impl Validator {
             if !decl_ty.equivalent(init_ty, gctx.types) {
                 return Err(OverrideError::InvalidType);
             }
+        } else if self.overrides_resolved {
+            return Err(OverrideError::UninitializedOverride);
         }
 
         Ok(())
@@ -558,18 +590,22 @@ impl Validator {
         &mut self,
         module: &crate::Module,
     ) -> Result<ModuleInfo, WithSpan<ValidationError>> {
-        self.allow_overrides = true;
+        self.overrides_resolved = false;
         self.validate_impl(module)
     }
 
-    /// Check the given module to be valid.
+    /// Check the given module to be valid, requiring overrides to be resolved.
     ///
-    /// With the additional restriction that overrides are not present.
-    pub fn validate_no_overrides(
+    /// This is the same as [`validate`], except that any override
+    /// whose value is not a fully-evaluated constant expression is
+    /// treated as an error.
+    ///
+    /// [`validate`]: Validator::validate
+    pub fn validate_resolved_overrides(
         &mut self,
         module: &crate::Module,
     ) -> Result<ModuleInfo, WithSpan<ValidationError>> {
-        self.allow_overrides = false;
+        self.overrides_resolved = true;
         self.validate_impl(module)
     }
 
@@ -657,16 +693,16 @@ impl Validator {
                     })?
             }
 
-            for (handle, override_) in module.overrides.iter() {
+            for (handle, r#override) in module.overrides.iter() {
                 self.validate_override(handle, module.to_ctx(), &mod_info)
                     .map_err(|source| {
                         ValidationError::Override {
                             handle,
-                            name: override_.name.clone().unwrap_or_default(),
+                            name: r#override.name.clone().unwrap_or_default(),
                             source,
                         }
                         .with_span_handle(handle, &module.overrides)
-                    })?
+                    })?;
             }
         }
 
@@ -683,7 +719,7 @@ impl Validator {
         }
 
         for (handle, fun) in module.functions.iter() {
-            match self.validate_function(fun, module, &mod_info, false, &global_expr_kind) {
+            match self.validate_function(fun, module, &mod_info, false) {
                 Ok(info) => mod_info.functions.push(info),
                 Err(error) => {
                     return Err(error.and_then(|source| {
@@ -709,7 +745,7 @@ impl Validator {
                 .with_span()); // TODO: keep some EP span information?
             }
 
-            match self.validate_entry_point(ep, module, &mod_info, &global_expr_kind) {
+            match self.validate_entry_point(ep, module, &mod_info) {
                 Ok(info) => mod_info.entry_points.push(info),
                 Err(error) => {
                     return Err(error.and_then(|source| {
