@@ -7,9 +7,6 @@ use super::text_parser::TextParser;
 
 pub type hb_codepoint_t = char; // uint32_t in C++
 
-pub const HB_FEATURE_GLOBAL_START: u32 = 0;
-pub const HB_FEATURE_GLOBAL_END: u32 = u32::MAX;
-
 /// Defines the direction in which text is to be read.
 #[derive(Clone, Copy, PartialEq, Eq, Hash, Debug)]
 pub enum Direction {
@@ -144,8 +141,7 @@ impl Direction {
             // https://github.com/harfbuzz/harfbuzz/issues/1000
             script::OLD_HUNGARIAN |
             script::OLD_ITALIC |
-            script::RUNIC |
-            script::TIFINAGH => {
+            script::RUNIC => {
                 None
             }
 
@@ -457,17 +453,6 @@ pub mod script {
     pub const TANGSA: Script = Script::from_bytes(b"Tnsa");
     pub const TOTO: Script = Script::from_bytes(b"Toto");
     pub const VITHKUQI: Script = Script::from_bytes(b"Vith");
-    // Since 15.0
-    pub const KAWI: Script = Script::from_bytes(b"Kawi");
-    pub const NAG_MUNDARI: Script = Script::from_bytes(b"Nagm");
-    // Since 16.0
-    pub const GARAY: Script = Script::from_bytes(b"Gara");
-    pub const GURUNG_KHEMA: Script = Script::from_bytes(b"Gukh");
-    pub const KIRAT_RAI: Script = Script::from_bytes(b"Krai");
-    pub const OL_ONAL: Script = Script::from_bytes(b"Onao");
-    pub const SUNUWAR: Script = Script::from_bytes(b"Sunu");
-    pub const TODHRI: Script = Script::from_bytes(b"Todr");
-    pub const TULU_TIGALARI: Script = Script::from_bytes(b"Tutg");
 
     pub const SCRIPT_MATH: Script = Script::from_bytes(b"Zmth");
 
@@ -579,10 +564,10 @@ impl core::str::FromStr for Feature {
                     p.advance(1);
                     p.consume_i32().unwrap_or(-1) as u32 // negative value overflow is ok
                 } else {
-                    if start_opt.is_some() && start != u32::MAX {
+                    if start_opt.is_some() && start != core::u32::MAX {
                         start + 1
                     } else {
-                        u32::MAX
+                        core::u32::MAX
                     }
                 };
 
@@ -590,7 +575,7 @@ impl core::str::FromStr for Feature {
 
                 (start, end)
             } else {
-                (0, u32::MAX)
+                (0, core::u32::MAX)
             };
 
             // Parse postfix.
@@ -657,7 +642,7 @@ mod tests_features {
     test!(parse_13, "kern[3:5]=2", b"kern", 2, 3..=5);
     test!(parse_14, "kern[3;5]=2", b"kern", 2, 3..=5);
     test!(parse_15, "kern[:-1]", b"kern", 1, ..);
-    test!(parse_16, "kern[-1]", b"kern", 1, u32::MAX as usize..);
+    test!(parse_16, "kern[-1]", b"kern", 1, core::u32::MAX as usize..);
     test!(parse_17, "kern=on", b"kern", 1, ..);
     test!(parse_18, "kern=off", b"kern", 0, ..);
     test!(parse_19, "kern=oN", b"kern", 1, ..);
@@ -713,7 +698,6 @@ impl core::str::FromStr for Variation {
 pub trait TagExt {
     fn default_script() -> Self;
     fn default_language() -> Self;
-    #[cfg(test)]
     fn to_lowercase(&self) -> Self;
     fn to_uppercase(&self) -> Self;
 }
@@ -730,7 +714,6 @@ impl TagExt for Tag {
     }
 
     /// Converts tag to lowercase.
-    #[cfg(test)]
     #[inline]
     fn to_lowercase(&self) -> Self {
         let b = self.to_bytes();
