@@ -7,18 +7,18 @@ pub trait FlexboxContainerStyle: CoreStyle {
     /// Which direction does the main axis flow in?
     #[inline(always)]
     fn flex_direction(&self) -> FlexDirection {
-        Style::DEFAULT.flex_direction
+        Style::<Self::CustomIdent>::DEFAULT.flex_direction
     }
     /// Should elements wrap, or stay in a single line?
     #[inline(always)]
     fn flex_wrap(&self) -> FlexWrap {
-        Style::DEFAULT.flex_wrap
+        Style::<Self::CustomIdent>::DEFAULT.flex_wrap
     }
 
     /// How large should the gaps between items in a grid or flex container be?
     #[inline(always)]
     fn gap(&self) -> Size<LengthPercentage> {
-        Style::DEFAULT.gap
+        Style::<Self::CustomIdent>::DEFAULT.gap
     }
 
     // Alignment properties
@@ -26,17 +26,17 @@ pub trait FlexboxContainerStyle: CoreStyle {
     /// How should content contained within this item be aligned in the cross/block axis
     #[inline(always)]
     fn align_content(&self) -> Option<AlignContent> {
-        Style::DEFAULT.align_content
+        Style::<Self::CustomIdent>::DEFAULT.align_content
     }
     /// How this node's children aligned in the cross/block axis?
     #[inline(always)]
     fn align_items(&self) -> Option<AlignItems> {
-        Style::DEFAULT.align_items
+        Style::<Self::CustomIdent>::DEFAULT.align_items
     }
     /// How this node's children should be aligned in the inline axis
     #[inline(always)]
     fn justify_content(&self) -> Option<JustifyContent> {
-        Style::DEFAULT.justify_content
+        Style::<Self::CustomIdent>::DEFAULT.justify_content
     }
 }
 
@@ -45,24 +45,24 @@ pub trait FlexboxItemStyle: CoreStyle {
     /// Sets the initial main axis size of the item
     #[inline(always)]
     fn flex_basis(&self) -> Dimension {
-        Style::DEFAULT.flex_basis
+        Style::<Self::CustomIdent>::DEFAULT.flex_basis
     }
     /// The relative rate at which this item grows when it is expanding to fill space
     #[inline(always)]
     fn flex_grow(&self) -> f32 {
-        Style::DEFAULT.flex_grow
+        Style::<Self::CustomIdent>::DEFAULT.flex_grow
     }
     /// The relative rate at which this item shrinks when it is contracting to fit into space
     #[inline(always)]
     fn flex_shrink(&self) -> f32 {
-        Style::DEFAULT.flex_shrink
+        Style::<Self::CustomIdent>::DEFAULT.flex_shrink
     }
 
     /// How this node should be aligned in the cross/block axis
     /// Falls back to the parents [`AlignItems`] if not set
     #[inline(always)]
     fn align_self(&self) -> Option<AlignSelf> {
-        Style::DEFAULT.align_self
+        Style::<Self::CustomIdent>::DEFAULT.align_self
     }
 }
 
