@@ -7,6 +7,7 @@ use slotmap::{DefaultKey, SlotMap};
 
 use crate::geometry::Size;
 use crate::style::{AvailableSpace, Display, Style};
+use crate::sys::DefaultCheapStr;
 use crate::tree::{
     Cache, ClearState, Layout, LayoutInput, LayoutOutput, LayoutPartialTree, NodeId, PrintTree, RoundTree, RunMode,
     TraversePartialTree, TraverseTree,
@@ -259,11 +260,11 @@ impl<NodeContext> PrintTree for TaffyTree<NodeContext> {
     }
 
     #[inline(always)]
-    fn get_final_layout(&self, node_id: NodeId) -> &Layout {
+    fn get_final_layout(&self, node_id: NodeId) -> Layout {
         if self.config.use_rounding {
-            &self.nodes[node_id.into()].final_layout
+            self.nodes[node_id.into()].final_layout
         } else {
-            &self.nodes[node_id.into()].unrounded_layout
+            self.nodes[node_id.into()].unrounded_layout
         }
     }
 }
@@ -327,6 +328,8 @@ where
     where
         Self: 'a;
 
+    type CustomIdent = DefaultCheapStr;
+
     #[inline(always)]
     fn get_core_container_style(&self, node_id: NodeId) -> Self::CoreContainerStyle<'_> {
         &self.taffy.nodes[node_id.into()].style
@@ -516,8 +519,8 @@ where
         FnMut(Size<Option<f32>>, Size<AvailableSpace>, NodeId, Option<&mut NodeContext>, &Style) -> Size<f32>,
 {
     #[inline(always)]
-    fn get_unrounded_layout(&self, node: NodeId) -> &Layout {
-        &self.taffy.nodes[node.into()].unrounded_layout
+    fn get_unrounded_layout(&self, node: NodeId) -> Layout {
+        self.taffy.nodes[node.into()].unrounded_layout
     }
 
     #[inline(always)]
