@@ -2,23 +2,25 @@
 //!
 
 mod canonical;
+pub(crate) use canonical::InternRecGroup;
 
-use self::{arc::MaybeOwned, canonical::canonicalize_and_intern_rec_group};
+use self::arc::MaybeOwned;
 use super::{
     check_max, combine_type_sizes,
     operators::{ty_to_str, OperatorValidator, OperatorValidatorAllocations},
     types::{CoreTypeId, EntityType, RecGroupId, TypeAlloc, TypeList},
 };
+#[cfg(feature = "simd")]
+use crate::VisitSimdOperator;
 use crate::{
-    limits::*, validator::types::TypeIdentifier, BinaryReaderError, CompositeType, ConstExpr, Data,
-    DataKind, Element, ElementKind, ExternalKind, FuncType, Global, GlobalType, HeapType,
-    MemoryType, PackedIndex, RecGroup, RefType, Result, StorageType, SubType, Table, TableInit,
-    TableType, TagType, TypeRef, UnpackedIndex, ValType, VisitOperator, WasmFeatures,
+    limits::*, BinaryReaderError, ConstExpr, Data, DataKind, Element, ElementKind, ExternalKind,
+    FuncType, Global, GlobalType, HeapType, MemoryType, RecGroup, RefType, Result, SubType, Table,
+    TableInit, TableType, TagType, TypeRef, UnpackedIndex, ValType, VisitOperator, WasmFeatures,
     WasmModuleResources,
 };
-use indexmap::IndexMap;
-use std::mem;
-use std::{collections::HashSet, sync::Arc};
+use crate::{prelude::*, CompositeInnerType};
+use alloc::sync::Arc;
+use core::mem;
 
 // Section order for WebAssembly modules.
 //
@@ -132,7 +134,7 @@ impl ModuleState {
         offset: usize,
     ) -> Result<()> {
         self.module
-            .check_global_type(&mut global.ty, features, offset)?;
+            .check_global_type(&mut global.ty, features, types, offset)?;
         self.check_const_expr(&global.init_expr, global.ty.content_type, features, types)?;
         self.module.assert_mut().globals.push(global.ty);
         Ok(())
@@ -146,7 +148,7 @@ impl ModuleState {
         offset: usize,
     ) -> Result<()> {
         self.module
-            .check_table_type(&mut table.ty, features, offset)?;
+            .check_table_type(&mut table.ty, features, types, offset)?;
 
         match &table.init {
             TableInit::RefNull => {
@@ -155,7 +157,7 @@ impl ModuleState {
                 }
             }
             TableInit::Expr(expr) => {
-                if !features.function_references {
+                if !features.function_references() {
                     bail!(
                         offset,
                         "tables with expression initializers require \
@@ -222,10 +224,10 @@ impl ModuleState {
                     ));
                 }
 
-                self.check_const_expr(&offset_expr, ValType::I32, features, types)?;
+                self.check_const_expr(&offset_expr, table.index_type(), features, types)?;
             }
             ElementKind::Passive | ElementKind::Declared => {
-                if !features.bulk_memory {
+                if !features.bulk_memory() {
                     return Err(BinaryReaderError::new(
                         "bulk memory must be enabled",
                         offset,
@@ -317,7 +319,7 @@ impl ModuleState {
             }
 
             fn validate_extended_const(&mut self, op: &str) -> Result<()> {
-                if self.ops.features.extended_const {
+                if self.ops.features.extended_const() {
                     Ok(())
                 } else {
                     Err(BinaryReaderError::new(
@@ -331,7 +333,21 @@ impl ModuleState {
             }
 
             fn validate_gc(&mut self, op: &str) -> Result<()> {
-                if self.features.gc {
+                if self.features.gc() {
+                    Ok(())
+                } else {
+                    Err(BinaryReaderError::new(
+                        format!(
+                            "constant expression required: non-constant operator: {}",
+                            op
+                        ),
+                        self.offset,
+                    ))
+                }
+            }
+
+            fn validate_shared_everything_threads(&mut self, op: &str) -> Result<()> {
+                if self.features.shared_everything_threads() {
                     Ok(())
                 } else {
                     Err(BinaryReaderError::new(
@@ -348,7 +364,7 @@ impl ModuleState {
                 let module = &self.resources.module;
                 let global = module.global_at(index, self.offset)?;
 
-                if index >= module.num_imported_globals && !self.features.gc {
+                if index >= module.num_imported_globals && !self.features.gc() {
                     return Err(BinaryReaderError::new(
                         "constant expression required: global.get of locally defined global",
                         self.offset,
@@ -391,10 +407,17 @@ impl ModuleState {
                         .insert(index);
                 }
             }
+
+            fn not_const(&self, instr: &str) -> BinaryReaderError {
+                BinaryReaderError::new(
+                    format!("constant expression required: non-constant operator: {instr}"),
+                    self.offset,
+                )
+            }
         }
 
         macro_rules! define_visit_operator {
-            ($(@$proposal:ident $op:ident $({ $($arg:ident: $argty:ty),* })? => $visit:ident)*) => {
+            ($(@$proposal:ident $op:ident $({ $($arg:ident: $argty:ty),* })? => $visit:ident ($($ann:tt)*))*) => {
                 $(
                     #[allow(unused_variables)]
                     fn $visit(&mut self $($(,$arg: $argty)*)?) -> Self::Output {
@@ -417,7 +440,7 @@ impl ModuleState {
                 $self.validator().visit_f64_const($val)
             }};
             (@visit $self:ident visit_v128_const $val:ident) => {{
-                $self.validator().visit_v128_const($val)
+                $self.validator().simd_visitor().unwrap().visit_v128_const($val)
             }};
             (@visit $self:ident visit_ref_null $val:ident) => {{
                 $self.validator().visit_ref_null($val)
@@ -479,6 +502,10 @@ impl ModuleState {
                 $self.validate_gc("ref.i31")?;
                 $self.validator().visit_ref_i31()
             }};
+            (@visit $self:ident visit_ref_i31_shared) => {{
+                $self.validate_shared_everything_threads("ref.i31_shared")?;
+                $self.validator().visit_ref_i31_shared()
+            }};
 
             // `global.get` is a valid const expression for imported, immutable
             // globals.
@@ -494,21 +521,31 @@ impl ModuleState {
             }};
 
             (@visit $self:ident $op:ident $($args:tt)*) => {{
-                Err(BinaryReaderError::new(
-                    format!("constant expression required: non-constant operator: {}", stringify!($op)),
-                    $self.offset,
-                ))
+                Err($self.not_const(stringify!($op)))
             }}
         }
 
         impl<'a> VisitOperator<'a> for VisitConstOperator<'a> {
             type Output = Result<()>;
 
-            for_each_operator!(define_visit_operator);
+            #[cfg(feature = "simd")]
+            fn simd_visitor(
+                &mut self,
+            ) -> Option<&mut dyn crate::VisitSimdOperator<'a, Output = Self::Output>> {
+                Some(self)
+            }
+
+            crate::for_each_visit_operator!(define_visit_operator);
+        }
+
+        #[cfg(feature = "simd")]
+        impl<'a> VisitSimdOperator<'a> for VisitConstOperator<'a> {
+            crate::for_each_visit_simd_operator!(define_visit_operator);
         }
     }
 }
 
+#[derive(Debug)]
 pub(crate) struct Module {
     // This is set once the code section starts.
     // `WasmModuleResources` implementations use the snapshot to
@@ -524,7 +561,7 @@ pub(crate) struct Module {
     // Stores indexes into `types`.
     pub functions: Vec<u32>,
     pub tags: Vec<CoreTypeId>,
-    pub function_references: HashSet<u32>,
+    pub function_references: Set<u32>,
     pub imports: IndexMap<(String, String), Vec<EntityType>>,
     pub exports: IndexMap<String, EntityType>,
     pub type_size: u32,
@@ -533,22 +570,7 @@ pub(crate) struct Module {
 }
 
 impl Module {
-    /// Get the `CoreTypeId` of the type at the given packed index.
-    pub(crate) fn at_packed_index(
-        &self,
-        types: &TypeList,
-        rec_group: RecGroupId,
-        index: PackedIndex,
-        offset: usize,
-    ) -> Result<CoreTypeId> {
-        match index.unpack() {
-            UnpackedIndex::Id(id) => Ok(id),
-            UnpackedIndex::Module(idx) => self.type_id_at(idx, offset),
-            UnpackedIndex::RecGroup(idx) => types.rec_group_local_id(rec_group, idx, offset),
-        }
-    }
-
-    pub fn add_types(
+    pub(crate) fn add_types(
         &mut self,
         rec_group: RecGroup,
         features: &WasmFeatures,
@@ -556,14 +578,6 @@ impl Module {
         offset: usize,
         check_limit: bool,
     ) -> Result<()> {
-        debug_assert!(rec_group.is_explicit_rec_group() || rec_group.types().len() == 1);
-        if rec_group.is_explicit_rec_group() && !features.gc {
-            bail!(
-                offset,
-                "rec group usage requires `gc` proposal to be enabled"
-            );
-        }
-
         if check_limit {
             check_max(
                 self.types.len(),
@@ -573,107 +587,7 @@ impl Module {
                 offset,
             )?;
         }
-
-        let (is_new, rec_group_id) =
-            canonicalize_and_intern_rec_group(features, types, self, rec_group, offset)?;
-
-        let range = &types[rec_group_id];
-        let start = range.start.index();
-        let end = range.end.index();
-
-        for i in start..end {
-            let i = u32::try_from(i).unwrap();
-            let id = CoreTypeId::from_index(i);
-            debug_assert!(types.get(id).is_some());
-            self.types.push(id);
-            if is_new {
-                self.check_subtype(rec_group_id, id, features, types, offset)?;
-            }
-        }
-
-        Ok(())
-    }
-
-    fn check_subtype(
-        &mut self,
-        rec_group: RecGroupId,
-        id: CoreTypeId,
-        features: &WasmFeatures,
-        types: &mut TypeAlloc,
-        offset: usize,
-    ) -> Result<()> {
-        let ty = &types[id];
-        if !features.gc && (!ty.is_final || ty.supertype_idx.is_some()) {
-            bail!(offset, "gc proposal must be enabled to use subtypes");
-        }
-
-        self.check_composite_type(&ty.composite_type, features, offset)?;
-
-        if let Some(supertype_index) = ty.supertype_idx {
-            debug_assert!(supertype_index.is_canonical());
-            let sup_id = self.at_packed_index(types, rec_group, supertype_index, offset)?;
-            if types[sup_id].is_final {
-                bail!(offset, "sub type cannot have a final super type");
-            }
-            if !types.matches(id, sup_id) {
-                bail!(offset, "sub type must match super type");
-            }
-        }
-
-        Ok(())
-    }
-
-    fn check_composite_type(
-        &mut self,
-        ty: &CompositeType,
-        features: &WasmFeatures,
-        offset: usize,
-    ) -> Result<()> {
-        let check = |ty: &ValType| {
-            features
-                .check_value_type(*ty)
-                .map_err(|e| BinaryReaderError::new(e, offset))
-        };
-        match ty {
-            CompositeType::Func(t) => {
-                for ty in t.params().iter().chain(t.results()) {
-                    check(ty)?;
-                }
-                if t.results().len() > 1 && !features.multi_value {
-                    return Err(BinaryReaderError::new(
-                        "func type returns multiple values but the multi-value feature is not enabled",
-                        offset,
-                    ));
-                }
-            }
-            CompositeType::Array(t) => {
-                if !features.gc {
-                    return Err(BinaryReaderError::new(
-                        "array indexed types not supported without the gc feature",
-                        offset,
-                    ));
-                }
-                match &t.0.element_type {
-                    StorageType::I8 | StorageType::I16 => {}
-                    StorageType::Val(value_type) => check(value_type)?,
-                };
-            }
-            CompositeType::Struct(t) => {
-                if !features.gc {
-                    return Err(BinaryReaderError::new(
-                        "struct indexed types not supported without the gc feature",
-                        offset,
-                    ));
-                }
-                for ty in t.fields.iter() {
-                    match &ty.element_type {
-                        StorageType::I8 | StorageType::I16 => {}
-                        StorageType::Val(value_type) => check(value_type)?,
-                    }
-                }
-            }
-        }
-        Ok(())
+        self.canonicalize_and_intern_rec_group(features, types, rec_group, offset)
     }
 
     pub fn add_import(
@@ -704,7 +618,7 @@ impl Module {
                 (self.tags.len(), MAX_WASM_TAGS, "tags")
             }
             TypeRef::Global(ty) => {
-                if !features.mutable_global && ty.mutable {
+                if !features.mutable_global() && ty.mutable {
                     return Err(BinaryReaderError::new(
                         "mutable global support is not enabled",
                         offset,
@@ -737,7 +651,7 @@ impl Module {
         check_limit: bool,
         types: &TypeList,
     ) -> Result<()> {
-        if !features.mutable_global {
+        if !features.mutable_global() {
             if let EntityType::Global(global_type) = ty {
                 if global_type.mutable {
                     return Err(BinaryReaderError::new(
@@ -792,13 +706,6 @@ impl Module {
         Ok(())
     }
 
-    pub fn type_id_at(&self, idx: u32, offset: usize) -> Result<CoreTypeId> {
-        self.types
-            .get(idx as usize)
-            .copied()
-            .ok_or_else(|| format_err!(offset, "unknown type {idx}: type index out of bounds"))
-    }
-
     fn sub_type_at<'a>(&self, types: &'a TypeList, idx: u32, offset: usize) -> Result<&'a SubType> {
         let id = self.type_id_at(idx, offset)?;
         Ok(&types[id])
@@ -810,8 +717,12 @@ impl Module {
         types: &'a TypeList,
         offset: usize,
     ) -> Result<&'a FuncType> {
-        match &self.sub_type_at(types, type_index, offset)?.composite_type {
-            CompositeType::Func(f) => Ok(f),
+        match &self
+            .sub_type_at(types, type_index, offset)?
+            .composite_type
+            .inner
+        {
+            CompositeInnerType::Func(f) => Ok(f),
             _ => bail!(offset, "type index {type_index} is not a function type"),
         }
     }
@@ -829,7 +740,7 @@ impl Module {
                 EntityType::Func(self.types[*type_index as usize])
             }
             TypeRef::Table(t) => {
-                self.check_table_type(t, features, offset)?;
+                self.check_table_type(t, features, types, offset)?;
                 EntityType::Table(*t)
             }
             TypeRef::Memory(t) => {
@@ -841,7 +752,7 @@ impl Module {
                 EntityType::Tag(self.types[t.func_type_idx as usize])
             }
             TypeRef::Global(t) => {
-                self.check_global_type(t, features, offset)?;
+                self.check_global_type(t, features, types, offset)?;
                 EntityType::Global(*t)
             }
         })
@@ -851,21 +762,40 @@ impl Module {
         &self,
         ty: &mut TableType,
         features: &WasmFeatures,
+        types: &TypeList,
         offset: usize,
     ) -> Result<()> {
-        // the `funcref` value type is allowed all the way back to the MVP, so
-        // don't check it here
+        // The `funcref` value type is allowed all the way back to the MVP, so
+        // don't check it here.
         if ty.element_type != RefType::FUNCREF {
             self.check_ref_type(&mut ty.element_type, features, offset)?
         }
 
-        self.check_limits(ty.initial, ty.maximum, offset)?;
-        if ty.initial > MAX_WASM_TABLE_ENTRIES as u32 {
+        if ty.table64 && !features.memory64() {
             return Err(BinaryReaderError::new(
-                "minimum table size is out of bounds",
+                "memory64 must be enabled for 64-bit tables",
                 offset,
             ));
         }
+
+        self.check_limits(ty.initial, ty.maximum, offset)?;
+
+        if ty.shared {
+            if !features.shared_everything_threads() {
+                return Err(BinaryReaderError::new(
+                    "shared tables require the shared-everything-threads proposal",
+                    offset,
+                ));
+            }
+
+            if !types.reftype_is_shared(ty.element_type) {
+                return Err(BinaryReaderError::new(
+                    "shared tables must have a shared element type",
+                    offset,
+                ));
+            }
+        }
+
         Ok(())
     }
 
@@ -876,21 +806,47 @@ impl Module {
         offset: usize,
     ) -> Result<()> {
         self.check_limits(ty.initial, ty.maximum, offset)?;
+        let (page_size, page_size_log2) = if let Some(page_size_log2) = ty.page_size_log2 {
+            if !features.custom_page_sizes() {
+                return Err(BinaryReaderError::new(
+                    "the custom page sizes proposal must be enabled to \
+                     customize a memory's page size",
+                    offset,
+                ));
+            }
+            // Currently 2**0 and 2**16 are the only valid page sizes, but this
+            // may be relaxed to allow any power of two in the future.
+            if page_size_log2 != 0 && page_size_log2 != 16 {
+                return Err(BinaryReaderError::new("invalid custom page size", offset));
+            }
+            let page_size = 1_u64 << page_size_log2;
+            debug_assert!(page_size.is_power_of_two());
+            debug_assert!(page_size == DEFAULT_WASM_PAGE_SIZE || page_size == 1);
+            (page_size, page_size_log2)
+        } else {
+            let page_size_log2 = 16;
+            debug_assert_eq!(DEFAULT_WASM_PAGE_SIZE, 1 << page_size_log2);
+            (DEFAULT_WASM_PAGE_SIZE, page_size_log2)
+        };
         let (true_maximum, err) = if ty.memory64 {
-            if !features.memory64 {
+            if !features.memory64() {
                 return Err(BinaryReaderError::new(
                     "memory64 must be enabled for 64-bit memories",
                     offset,
                 ));
             }
             (
-                MAX_WASM_MEMORY64_PAGES,
-                "memory size must be at most 2**48 pages",
+                max_wasm_memory64_pages(page_size),
+                format!(
+                    "memory size must be at most 2**{} pages",
+                    64 - page_size_log2
+                ),
             )
         } else {
+            let max = max_wasm_memory32_pages(page_size);
             (
-                MAX_WASM_MEMORY32_PAGES,
-                "memory size must be at most 65536 pages (4GiB)",
+                max,
+                format!("memory size must be at most {max} pages (4GiB)"),
             )
         };
         if ty.initial > true_maximum {
@@ -902,7 +858,7 @@ impl Module {
             }
         }
         if ty.shared {
-            if !features.threads {
+            if !features.threads() {
                 return Err(BinaryReaderError::new(
                     "threads must be enabled for shared memories",
                     offset,
@@ -918,11 +874,13 @@ impl Module {
         Ok(())
     }
 
+    #[cfg(feature = "component-model")]
     pub(crate) fn imports_for_module_type(
         &self,
         offset: usize,
     ) -> Result<IndexMap<(String, String), EntityType>> {
-        // Ensure imports are unique, which is a requirement of the component model
+        // Ensure imports are unique, which is a requirement of the component model:
+        // https://github.com/WebAssembly/component-model/blob/d09f907/design/mvp/Explainer.md#import-and-export-definitions
         self.imports
             .iter()
             .map(|((module, name), types)| {
@@ -970,19 +928,9 @@ impl Module {
     }
 
     fn check_heap_type(&self, ty: &mut HeapType, offset: usize) -> Result<()> {
-        // Check that the heap type is valid
+        // Check that the heap type is valid.
         let type_index = match ty {
-            HeapType::Func
-            | HeapType::Extern
-            | HeapType::Any
-            | HeapType::None
-            | HeapType::NoExtern
-            | HeapType::NoFunc
-            | HeapType::Eq
-            | HeapType::Struct
-            | HeapType::Array
-            | HeapType::I31
-            | HeapType::Exn => return Ok(()),
+            HeapType::Abstract { .. } => return Ok(()),
             HeapType::Concrete(type_index) => type_index,
         };
         match type_index {
@@ -1005,14 +953,14 @@ impl Module {
         types: &TypeList,
         offset: usize,
     ) -> Result<()> {
-        if !features.exceptions {
+        if !features.exceptions() {
             return Err(BinaryReaderError::new(
                 "exceptions proposal not enabled",
                 offset,
             ));
         }
         let ty = self.func_type_at(ty.func_type_idx, types, offset)?;
-        if !ty.results().is_empty() {
+        if !ty.results().is_empty() && !features.stack_switching() {
             return Err(BinaryReaderError::new(
                 "invalid exception type: non-empty tag result type",
                 offset,
@@ -1025,9 +973,25 @@ impl Module {
         &self,
         ty: &mut GlobalType,
         features: &WasmFeatures,
+        types: &TypeList,
         offset: usize,
     ) -> Result<()> {
-        self.check_value_type(&mut ty.content_type, features, offset)
+        self.check_value_type(&mut ty.content_type, features, offset)?;
+        if ty.shared {
+            if !features.shared_everything_threads() {
+                return Err(BinaryReaderError::new(
+                    "shared globals require the shared-everything-threads proposal",
+                    offset,
+                ));
+            }
+            if !types.valtype_is_shared(ty.content_type) {
+                return Err(BinaryReaderError::new(
+                    "shared globals must have a shared value type",
+                    offset,
+                ));
+            }
+        }
+        Ok(())
     }
 
     fn check_limits<T>(&self, initial: T, maximum: Option<T>, offset: usize) -> Result<()>
@@ -1046,7 +1010,7 @@ impl Module {
     }
 
     pub fn max_tables(&self, features: &WasmFeatures) -> usize {
-        if features.reference_types {
+        if features.reference_types() {
             MAX_WASM_TABLES
         } else {
             1
@@ -1054,7 +1018,7 @@ impl Module {
     }
 
     pub fn max_memories(&self, features: &WasmFeatures) -> usize {
-        if features.multi_memory {
+        if features.multi_memory() {
             MAX_WASM_MEMORIES
         } else {
             1
@@ -1148,6 +1112,23 @@ impl Module {
     }
 }
 
+impl InternRecGroup for Module {
+    fn add_type_id(&mut self, id: CoreTypeId) {
+        self.types.push(id);
+    }
+
+    fn type_id_at(&self, idx: u32, offset: usize) -> Result<CoreTypeId> {
+        self.types
+            .get(idx as usize)
+            .copied()
+            .ok_or_else(|| format_err!(offset, "unknown type {idx}: type index out of bounds"))
+    }
+
+    fn types_len(&self) -> u32 {
+        u32::try_from(self.types.len()).unwrap()
+    }
+}
+
 impl Default for Module {
     fn default() -> Self {
         Self {
@@ -1198,14 +1179,17 @@ impl WasmModuleResources for OperatorValidatorResources<'_> {
         Some(&self.types[id])
     }
 
+    fn sub_type_at_id(&self, at: CoreTypeId) -> &SubType {
+        &self.types[at]
+    }
+
     fn type_id_of_function(&self, at: u32) -> Option<CoreTypeId> {
         let type_index = self.module.functions.get(at as usize)?;
         self.module.types.get(*type_index as usize).copied()
     }
 
-    fn type_of_function(&self, at: u32) -> Option<&FuncType> {
-        let type_index = self.module.functions.get(at as usize)?;
-        Some(self.sub_type_at(*type_index)?.composite_type.unwrap_func())
+    fn type_index_of_function(&self, at: u32) -> Option<u32> {
+        self.module.functions.get(at as usize).copied()
     }
 
     fn check_heap_type(&self, t: &mut HeapType, offset: usize) -> Result<()> {
@@ -1224,6 +1208,10 @@ impl WasmModuleResources for OperatorValidatorResources<'_> {
         self.types.valtype_is_subtype(a, b)
     }
 
+    fn is_shared(&self, ty: RefType) -> bool {
+        self.types.reftype_is_shared(ty)
+    }
+
     fn element_count(&self) -> u32 {
         self.module.element_types.len() as u32
     }
@@ -1239,6 +1227,7 @@ impl WasmModuleResources for OperatorValidatorResources<'_> {
 
 /// The implementation of [`WasmModuleResources`] used by
 /// [`Validator`](crate::Validator).
+#[derive(Debug)]
 pub struct ValidatorResources(pub(crate) Arc<Module>);
 
 impl WasmModuleResources for ValidatorResources {
@@ -1253,8 +1242,8 @@ impl WasmModuleResources for ValidatorResources {
     fn tag_at(&self, at: u32) -> Option<&FuncType> {
         let id = *self.0.tags.get(at as usize)?;
         let types = self.0.snapshot.as_ref().unwrap();
-        match &types[id].composite_type {
-            CompositeType::Func(f) => Some(f),
+        match &types[id].composite_type.inner {
+            CompositeInnerType::Func(f) => Some(f),
             _ => None,
         }
     }
@@ -1269,14 +1258,18 @@ impl WasmModuleResources for ValidatorResources {
         Some(&types[id])
     }
 
+    fn sub_type_at_id(&self, at: CoreTypeId) -> &SubType {
+        let types = self.0.snapshot.as_ref().unwrap();
+        &types[at]
+    }
+
     fn type_id_of_function(&self, at: u32) -> Option<CoreTypeId> {
         let type_index = *self.0.functions.get(at as usize)?;
         self.0.types.get(type_index as usize).copied()
     }
 
-    fn type_of_function(&self, at: u32) -> Option<&FuncType> {
-        let type_index = *self.0.functions.get(at as usize)?;
-        Some(self.sub_type_at(type_index)?.composite_type.unwrap_func())
+    fn type_index_of_function(&self, at: u32) -> Option<u32> {
+        self.0.functions.get(at as usize).copied()
     }
 
     fn check_heap_type(&self, t: &mut HeapType, offset: usize) -> Result<()> {
@@ -1295,6 +1288,10 @@ impl WasmModuleResources for ValidatorResources {
         self.0.snapshot.as_ref().unwrap().valtype_is_subtype(a, b)
     }
 
+    fn is_shared(&self, ty: RefType) -> bool {
+        self.0.snapshot.as_ref().unwrap().reftype_is_shared(ty)
+    }
+
     fn element_count(&self) -> u32 {
         self.0.element_types.len() as u32
     }
@@ -1319,8 +1316,8 @@ const _: () = {
 };
 
 mod arc {
-    use std::ops::Deref;
-    use std::sync::Arc;
+    use alloc::sync::Arc;
+    use core::ops::Deref;
 
     enum Inner<T> {
         Owned(T),
@@ -1362,7 +1359,7 @@ mod arc {
                 return;
             }
 
-            let inner = std::mem::replace(&mut self.inner, Inner::Empty);
+            let inner = core::mem::replace(&mut self.inner, Inner::Empty);
             let x = match inner {
                 Inner::Owned(x) => x,
                 _ => Self::unreachable(),
