@@ -23,11 +23,12 @@
 
 use crate::prelude::*;
 use crate::{Engine, ModuleVersionStrategy, Precompiled};
+use core::fmt;
 use core::str::FromStr;
 use object::endian::Endianness;
 #[cfg(any(feature = "cranelift", feature = "winch"))]
 use object::write::{Object, StandardSegment};
-use object::{read::elf::ElfFile64, FileFlags, Object as _, ObjectSection, SectionKind};
+use object::{read::elf::ElfFile64, FileFlags, Object as _, ObjectSection};
 use serde_derive::{Deserialize, Serialize};
 use wasmtime_environ::obj;
 use wasmtime_environ::{FlagValue, ObjectKind, Tunables};
@@ -68,7 +69,7 @@ pub fn check_compatible(engine: &Engine, mmap: &[u8], expected: ObjectKind) -> R
             os_abi: obj::ELFOSABI_WASMTIME,
             abi_version: 0,
             e_flags,
-        } if e_flags == expected_e_flags => {}
+        } if e_flags & expected_e_flags == expected_e_flags => {}
         _ => bail!("incompatible object file format"),
     }
 
@@ -122,7 +123,7 @@ pub fn append_compiler_info(engine: &Engine, obj: &mut Object<'_>, metadata: &Me
     let section = obj.add_section(
         obj.segment_name(StandardSegment::Data).to_vec(),
         obj::ELF_WASM_ENGINE.as_bytes().to_vec(),
-        SectionKind::ReadOnlyData,
+        object::SectionKind::ReadOnlyData,
     );
     let mut data = Vec::new();
     data.push(VERSION);
@@ -149,13 +150,13 @@ fn detect_precompiled<'data, R: object::ReadRef<'data>>(
         FileFlags::Elf {
             os_abi: obj::ELFOSABI_WASMTIME,
             abi_version: 0,
-            e_flags: obj::EF_WASMTIME_MODULE,
-        } => Some(Precompiled::Module),
+            e_flags,
+        } if e_flags & obj::EF_WASMTIME_MODULE != 0 => Some(Precompiled::Module),
         FileFlags::Elf {
             os_abi: obj::ELFOSABI_WASMTIME,
             abi_version: 0,
-            e_flags: obj::EF_WASMTIME_COMPONENT,
-        } => Some(Precompiled::Component),
+            e_flags,
+        } if e_flags & obj::EF_WASMTIME_COMPONENT != 0 => Some(Precompiled::Component),
         _ => None,
     }
 }
@@ -179,106 +180,18 @@ pub struct Metadata<'a> {
     #[serde(borrow)]
     isa_flags: Vec<(&'a str, FlagValue<'a>)>,
     tunables: Tunables,
-    features: WasmFeatures,
-}
-
-// This exists because `wasmparser::WasmFeatures` isn't serializable
-#[derive(Debug, Copy, Clone, Serialize, Deserialize)]
-struct WasmFeatures {
-    reference_types: bool,
-    multi_value: bool,
-    bulk_memory: bool,
-    component_model: bool,
-    simd: bool,
-    tail_call: bool,
-    threads: bool,
-    multi_memory: bool,
-    exceptions: bool,
-    memory64: bool,
-    relaxed_simd: bool,
-    extended_const: bool,
-    function_references: bool,
-    gc: bool,
-    custom_page_sizes: bool,
-    component_model_more_flags: bool,
-    component_model_multiple_returns: bool,
-    gc_types: bool,
-    wide_arithmetic: bool,
+    features: u32,
 }
 
 impl Metadata<'_> {
     #[cfg(any(feature = "cranelift", feature = "winch"))]
     pub fn new(engine: &Engine) -> Metadata<'static> {
-        let wasmparser::WasmFeaturesInflated {
-            reference_types,
-            multi_value,
-            bulk_memory,
-            component_model,
-            simd,
-            threads,
-            tail_call,
-            multi_memory,
-            exceptions,
-            memory64,
-            relaxed_simd,
-            extended_const,
-            memory_control,
-            function_references,
-            gc,
-            custom_page_sizes,
-            shared_everything_threads,
-            component_model_values,
-            component_model_nested_names,
-            component_model_more_flags,
-            component_model_multiple_returns,
-            legacy_exceptions,
-            gc_types,
-            stack_switching,
-            wide_arithmetic,
-
-            // Always on; we don't currently have knobs for these.
-            mutable_global: _,
-            saturating_float_to_int: _,
-            sign_extension: _,
-            floats: _,
-        } = engine.features().inflate();
-
-        // These features are not implemented in Wasmtime yet. We match on them
-        // above so that once we do implement support for them, we won't
-        // silently ignore them during serialization.
-        assert!(!memory_control);
-        assert!(!component_model_values);
-        assert!(!component_model_nested_names);
-        assert!(!shared_everything_threads);
-        assert!(!legacy_exceptions);
-        assert!(!stack_switching);
-
         Metadata {
             target: engine.compiler().triple().to_string(),
             shared_flags: engine.compiler().flags(),
             isa_flags: engine.compiler().isa_flags(),
             tunables: engine.tunables().clone(),
-            features: WasmFeatures {
-                reference_types,
-                multi_value,
-                bulk_memory,
-                component_model,
-                simd,
-                threads,
-                tail_call,
-                multi_memory,
-                exceptions,
-                memory64,
-                relaxed_simd,
-                extended_const,
-                function_references,
-                gc,
-                custom_page_sizes,
-                component_model_more_flags,
-                component_model_multiple_returns,
-                gc_types,
-                wide_arithmetic,
-            },
+            features: engine.features().bits(),
         }
     }
 
@@ -333,7 +246,7 @@ impl Metadata<'_> {
         Ok(())
     }
 
-    fn check_int<T: Eq + core::fmt::Display>(found: T, expected: T, feature: &str) -> Result<()> {
+    fn check_int<T: Eq + fmt::Display>(found: T, expected: T, feature: &str) -> Result<()> {
         if found == expected {
             return Ok(());
         }
@@ -346,7 +259,7 @@ impl Metadata<'_> {
         );
     }
 
-    fn check_bool(found: bool, expected: bool, feature: &str) -> Result<()> {
+    fn check_bool(found: bool, expected: bool, feature: impl fmt::Display) -> Result<()> {
         if found == expected {
             return Ok(());
         }
@@ -451,7 +364,7 @@ impl Metadata<'_> {
         cfg_str: &str,
         found: bool,
         expected: bool,
-        feature: &str,
+        feature: impl fmt::Display,
     ) -> Result<()> {
         if cfg {
             Self::check_bool(found, expected, feature)
@@ -468,124 +381,41 @@ impl Metadata<'_> {
     }
 
     fn check_features(&mut self, other: &wasmparser::WasmFeatures) -> Result<()> {
-        let WasmFeatures {
-            reference_types,
-            multi_value,
-            bulk_memory,
-            component_model,
-            simd,
-            tail_call,
-            threads,
-            multi_memory,
-            exceptions,
-            memory64,
-            relaxed_simd,
-            extended_const,
-            function_references,
-            gc,
-            custom_page_sizes,
-            component_model_more_flags,
-            component_model_multiple_returns,
-            gc_types,
-            wide_arithmetic,
-        } = self.features;
+        let module_features = wasmparser::WasmFeatures::from_bits_truncate(self.features);
+        let difference = *other ^ module_features;
+        for (name, flag) in difference.iter_names() {
+            let found = module_features.contains(flag);
+            let expected = other.contains(flag);
+            // Give a slightly more specialized error message for the `GC_TYPES`
+            // feature which isn't actually part of wasm itself but is gated by
+            // compile-time crate features.
+            if flag == wasmparser::WasmFeatures::GC_TYPES {
+                Self::check_cfg_bool(
+                    cfg!(feature = "gc"),
+                    "gc",
+                    found,
+                    expected,
+                    WasmFeature(name),
+                )?;
+            } else {
+                Self::check_bool(found, expected, WasmFeature(name))?;
+            }
+        }
 
-        use wasmparser::WasmFeatures as F;
-        Self::check_bool(
-            reference_types,
-            other.contains(F::REFERENCE_TYPES),
-            "WebAssembly reference types support",
-        )?;
-        Self::check_bool(
-            function_references,
-            other.contains(F::FUNCTION_REFERENCES),
-            "WebAssembly function-references support",
-        )?;
-        Self::check_bool(
-            gc,
-            other.contains(F::GC),
-            "WebAssembly garbage collection support",
-        )?;
-        Self::check_bool(
-            multi_value,
-            other.contains(F::MULTI_VALUE),
-            "WebAssembly multi-value support",
-        )?;
-        Self::check_bool(
-            bulk_memory,
-            other.contains(F::BULK_MEMORY),
-            "WebAssembly bulk memory support",
-        )?;
-        Self::check_bool(
-            component_model,
-            other.contains(F::COMPONENT_MODEL),
-            "WebAssembly component model support",
-        )?;
-        Self::check_bool(simd, other.contains(F::SIMD), "WebAssembly SIMD support")?;
-        Self::check_bool(
-            tail_call,
-            other.contains(F::TAIL_CALL),
-            "WebAssembly tail calls support",
-        )?;
-        Self::check_bool(
-            threads,
-            other.contains(F::THREADS),
-            "WebAssembly threads support",
-        )?;
-        Self::check_bool(
-            multi_memory,
-            other.contains(F::MULTI_MEMORY),
-            "WebAssembly multi-memory support",
-        )?;
-        Self::check_bool(
-            exceptions,
-            other.contains(F::EXCEPTIONS),
-            "WebAssembly exceptions support",
-        )?;
-        Self::check_bool(
-            memory64,
-            other.contains(F::MEMORY64),
-            "WebAssembly 64-bit memory support",
-        )?;
-        Self::check_bool(
-            extended_const,
-            other.contains(F::EXTENDED_CONST),
-            "WebAssembly extended-const support",
-        )?;
-        Self::check_bool(
-            relaxed_simd,
-            other.contains(F::RELAXED_SIMD),
-            "WebAssembly relaxed-simd support",
-        )?;
-        Self::check_bool(
-            custom_page_sizes,
-            other.contains(F::CUSTOM_PAGE_SIZES),
-            "WebAssembly custom-page-sizes support",
-        )?;
-        Self::check_bool(
-            component_model_more_flags,
-            other.contains(F::COMPONENT_MODEL_MORE_FLAGS),
-            "WebAssembly component model support for more than 32 flags",
-        )?;
-        Self::check_bool(
-            component_model_multiple_returns,
-            other.contains(F::COMPONENT_MODEL_MULTIPLE_RETURNS),
-            "WebAssembly component model support for multiple returns",
-        )?;
-        Self::check_cfg_bool(
-            cfg!(feature = "gc"),
-            "gc",
-            gc_types,
-            other.contains(F::GC_TYPES),
-            "support for WebAssembly gc types",
-        )?;
-        Self::check_bool(
-            wide_arithmetic,
-            other.contains(F::WIDE_ARITHMETIC),
-            "WebAssembly wide-arithmetic support",
-        )?;
+        return Ok(());
 
-        Ok(())
+        struct WasmFeature<'a>(&'a str);
+
+        impl fmt::Display for WasmFeature<'_> {
+            fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+                write!(f, "support for WebAssembly feature `")?;
+                for c in self.0.chars().map(|c| c.to_lowercase()) {
+                    write!(f, "{c}")?;
+                }
+                write!(f, "`")?;
+                Ok(())
+            }
+        }
     }
 
     fn check_collector(
@@ -616,7 +446,7 @@ impl Metadata<'_> {
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::{Config, Module, OptLevel};
+    use crate::{Cache, Config, Module, OptLevel};
     use std::{
         collections::hash_map::DefaultHasher,
         hash::{Hash, Hasher},
@@ -773,11 +603,15 @@ Caused by:
 
         let engine = Engine::new(&config)?;
         let mut metadata = Metadata::new(&engine);
-        metadata.features.threads = false;
+        metadata.features &= !wasmparser::WasmFeatures::THREADS.bits();
 
         match metadata.check_compatible(&engine) {
             Ok(_) => unreachable!(),
-            Err(e) => assert_eq!(e.to_string(), "Module was compiled without WebAssembly threads support but it is enabled for the host"),
+            Err(e) => assert_eq!(
+                e.to_string(),
+                "Module was compiled without support for WebAssembly feature \
+                 `threads` but it is enabled for the host"
+            ),
         }
 
         let mut config = Config::new();
@@ -785,11 +619,15 @@ Caused by:
 
         let engine = Engine::new(&config)?;
         let mut metadata = Metadata::new(&engine);
-        metadata.features.threads = true;
+        metadata.features |= wasmparser::WasmFeatures::THREADS.bits();
 
         match metadata.check_compatible(&engine) {
             Ok(_) => unreachable!(),
-            Err(e) => assert_eq!(e.to_string(), "Module was compiled with WebAssembly threads support but it is not enabled for the host"),
+            Err(e) => assert_eq!(
+                e.to_string(),
+                "Module was compiled with support for WebAssembly feature \
+                `threads` but it is not enabled for the host"
+            ),
         }
 
         Ok(())
@@ -818,7 +656,6 @@ Caused by:
             &format!(
                 "
                     [cache]
-                    enabled = true
                     directory = '{}'
                 ",
                 td.path().join("cache").display()
@@ -826,46 +663,67 @@ Caused by:
         )?;
         let mut cfg = Config::new();
         cfg.cranelift_opt_level(OptLevel::None)
-            .cache_config_load(&config_path)?;
+            .cache(Some(Cache::from_file(Some(&config_path))?));
         let engine = Engine::new(&cfg)?;
         Module::new(&engine, "(module (func))")?;
-        assert_eq!(engine.config().cache_config.cache_hits(), 0);
-        assert_eq!(engine.config().cache_config.cache_misses(), 1);
+        let cache_config = engine
+            .config()
+            .cache
+            .as_ref()
+            .expect("Missing cache config");
+        assert_eq!(cache_config.cache_hits(), 0);
+        assert_eq!(cache_config.cache_misses(), 1);
         Module::new(&engine, "(module (func))")?;
-        assert_eq!(engine.config().cache_config.cache_hits(), 1);
-        assert_eq!(engine.config().cache_config.cache_misses(), 1);
+        assert_eq!(cache_config.cache_hits(), 1);
+        assert_eq!(cache_config.cache_misses(), 1);
 
         let mut cfg = Config::new();
         cfg.cranelift_opt_level(OptLevel::Speed)
-            .cache_config_load(&config_path)?;
+            .cache(Some(Cache::from_file(Some(&config_path))?));
         let engine = Engine::new(&cfg)?;
+        let cache_config = engine
+            .config()
+            .cache
+            .as_ref()
+            .expect("Missing cache config");
         Module::new(&engine, "(module (func))")?;
-        assert_eq!(engine.config().cache_config.cache_hits(), 0);
-        assert_eq!(engine.config().cache_config.cache_misses(), 1);
+        assert_eq!(cache_config.cache_hits(), 0);
+        assert_eq!(cache_config.cache_misses(), 1);
         Module::new(&engine, "(module (func))")?;
-        assert_eq!(engine.config().cache_config.cache_hits(), 1);
-        assert_eq!(engine.config().cache_config.cache_misses(), 1);
+        assert_eq!(cache_config.cache_hits(), 1);
+        assert_eq!(cache_config.cache_misses(), 1);
 
         let mut cfg = Config::new();
         cfg.cranelift_opt_level(OptLevel::SpeedAndSize)
-            .cache_config_load(&config_path)?;
+            .cache(Some(Cache::from_file(Some(&config_path))?));
         let engine = Engine::new(&cfg)?;
+        let cache_config = engine
+            .config()
+            .cache
+            .as_ref()
+            .expect("Missing cache config");
         Module::new(&engine, "(module (func))")?;
-        assert_eq!(engine.config().cache_config.cache_hits(), 0);
-        assert_eq!(engine.config().cache_config.cache_misses(), 1);
+        assert_eq!(cache_config.cache_hits(), 0);
+        assert_eq!(cache_config.cache_misses(), 1);
         Module::new(&engine, "(module (func))")?;
-        assert_eq!(engine.config().cache_config.cache_hits(), 1);
-        assert_eq!(engine.config().cache_config.cache_misses(), 1);
+        assert_eq!(cache_config.cache_hits(), 1);
+        assert_eq!(cache_config.cache_misses(), 1);
 
         let mut cfg = Config::new();
-        cfg.debug_info(true).cache_config_load(&config_path)?;
+        cfg.debug_info(true)
+            .cache(Some(Cache::from_file(Some(&config_path))?));
         let engine = Engine::new(&cfg)?;
+        let cache_config = engine
+            .config()
+            .cache
+            .as_ref()
+            .expect("Missing cache config");
         Module::new(&engine, "(module (func))")?;
-        assert_eq!(engine.config().cache_config.cache_hits(), 0);
-        assert_eq!(engine.config().cache_config.cache_misses(), 1);
+        assert_eq!(cache_config.cache_hits(), 0);
+        assert_eq!(cache_config.cache_misses(), 1);
         Module::new(&engine, "(module (func))")?;
-        assert_eq!(engine.config().cache_config.cache_hits(), 1);
-        assert_eq!(engine.config().cache_config.cache_misses(), 1);
+        assert_eq!(cache_config.cache_hits(), 1);
+        assert_eq!(cache_config.cache_misses(), 1);
 
         Ok(())
     }
@@ -926,21 +784,25 @@ Caused by:
             &format!(
                 "
                     [cache]
-                    enabled = true
                     directory = '{}'
                 ",
                 td.path().join("cache").display()
             ),
         )?;
         let mut cfg = Config::new();
-        cfg.cache_config_load(&config_path)?;
+        cfg.cache(Some(Cache::from_file(Some(&config_path))?));
         let engine = Engine::new(&cfg)?;
+        let cache_config = engine
+            .config()
+            .cache
+            .as_ref()
+            .expect("Missing cache config");
         Component::new(&engine, "(component (core module (func)))")?;
-        assert_eq!(engine.config().cache_config.cache_hits(), 0);
-        assert_eq!(engine.config().cache_config.cache_misses(), 1);
+        assert_eq!(cache_config.cache_hits(), 0);
+        assert_eq!(cache_config.cache_misses(), 1);
         Component::new(&engine, "(component (core module (func)))")?;
-        assert_eq!(engine.config().cache_config.cache_hits(), 1);
-        assert_eq!(engine.config().cache_config.cache_misses(), 1);
+        assert_eq!(cache_config.cache_hits(), 1);
+        assert_eq!(cache_config.cache_misses(), 1);
 
         Ok(())
     }
