@@ -19,8 +19,8 @@
 //! support in this module is enough to run wasm modules but any customization
 //! beyond that [`WasiCtxBuilder`] already supports is not possible yet.
 //!
-//! [`WasiCtxBuilder`]: crate::WasiCtxBuilder
-//! [`build_p1`]: crate::WasiCtxBuilder::build_p1
+//! [`WasiCtxBuilder`]: crate::p2::WasiCtxBuilder
+//! [`build_p1`]: crate::p2::WasiCtxBuilder::build_p1
 //! [`Config::async_support`]: wasmtime::Config::async_support
 //!
 //! # Components vs Modules
@@ -36,7 +36,7 @@
 //! ```no_run
 //! use wasmtime::{Result, Engine, Linker, Module, Store};
 //! use wasmtime_wasi::preview1::{self, WasiP1Ctx};
-//! use wasmtime_wasi::WasiCtxBuilder;
+//! use wasmtime_wasi::p2::WasiCtxBuilder;
 //!
 //! // An example of executing a WASIp1 "command"
 //! fn main() -> Result<()> {
@@ -63,35 +63,37 @@
 //! }
 //! ```
 
-use crate::bindings::{
+use crate::p2::bindings::{
     cli::{
         stderr::Host as _, stdin::Host as _, stdout::Host as _, terminal_input, terminal_output,
         terminal_stderr::Host as _, terminal_stdin::Host as _, terminal_stdout::Host as _,
     },
     clocks::{monotonic_clock, wall_clock},
     filesystem::{preopens::Host as _, types as filesystem},
-    io::streams,
-};
-use crate::{
-    FsError, IsATTY, ResourceTable, StreamError, StreamResult, WasiCtx, WasiImpl, WasiView,
 };
+use crate::p2::{FsError, IsATTY, WasiCtx, WasiImpl, WasiView};
+use crate::ResourceTable;
 use anyhow::{bail, Context};
-use std::collections::{BTreeMap, HashSet};
+use std::collections::{btree_map, BTreeMap, BTreeSet, HashSet};
 use std::mem::{self, size_of, size_of_val};
-use std::ops::{Deref, DerefMut};
 use std::slice;
 use std::sync::atomic::{AtomicU64, Ordering};
 use std::sync::Arc;
 use system_interface::fs::FileIoExt;
 use wasmtime::component::Resource;
+use wasmtime_wasi_io::{
+    bindings::wasi::io::streams,
+    streams::{StreamError, StreamResult},
+    IoImpl, IoView,
+};
 use wiggle::tracing::instrument;
 use wiggle::{GuestError, GuestMemory, GuestPtr, GuestType};
 
 // Bring all WASI traits in scope that this implementation builds on.
-use crate::bindings::cli::environment::Host as _;
-use crate::bindings::filesystem::types::HostDescriptor as _;
-use crate::bindings::io::poll::Host as _;
-use crate::bindings::random::random::Host as _;
+use crate::p2::bindings::cli::environment::Host as _;
+use crate::p2::bindings::filesystem::types::HostDescriptor as _;
+use crate::p2::bindings::random::random::Host as _;
+use wasmtime_wasi_io::bindings::wasi::io::poll::Host as _;
 
 /// Structure containing state for WASIp1.
 ///
@@ -103,15 +105,15 @@ use crate::bindings::random::random::Host as _;
 /// Instances of [`WasiP1Ctx`] are typically stored within the `T` of
 /// [`Store<T>`](wasmtime::Store).
 ///
-/// [`WasiCtxBuilder::build_p1`]: crate::WasiCtxBuilder::build_p1
-/// [`WasiCtxBuilder`]: crate::WasiCtxBuilder
+/// [`WasiCtxBuilder::build_p1`]: crate::p2::WasiCtxBuilder::build_p1
+/// [`WasiCtxBuilder`]: crate::p2::WasiCtxBuilder
 ///
 /// # Examples
 ///
 /// ```no_run
 /// use wasmtime::{Result, Linker};
 /// use wasmtime_wasi::preview1::{self, WasiP1Ctx};
-/// use wasmtime_wasi::WasiCtxBuilder;
+/// use wasmtime_wasi::p2::WasiCtxBuilder;
 ///
 /// struct MyState {
 ///     // ... custom state as necessary ...
@@ -153,14 +155,19 @@ impl WasiP1Ctx {
     }
 
     fn as_wasi_impl(&mut self) -> WasiImpl<&mut Self> {
-        WasiImpl(self)
+        WasiImpl(IoImpl(self))
+    }
+    fn as_io_impl(&mut self) -> IoImpl<&mut Self> {
+        IoImpl(self)
     }
 }
 
-impl WasiView for WasiP1Ctx {
+impl IoView for WasiP1Ctx {
     fn table(&mut self) -> &mut ResourceTable {
         &mut self.table
     }
+}
+impl WasiView for WasiP1Ctx {
     fn ctx(&mut self) -> &mut WasiCtx {
         &mut self.wasi
     }
@@ -168,7 +175,7 @@ impl WasiView for WasiP1Ctx {
 
 #[derive(Debug)]
 struct File {
-    /// The handle to the preview2 descriptor of type [`crate::filesystem::Descriptor::File`].
+    /// The handle to the preview2 descriptor of type [`crate::p2::filesystem::Descriptor::File`].
     fd: Resource<filesystem::Descriptor>,
 
     /// The current-position pointer.
@@ -259,14 +266,14 @@ enum Descriptor {
         stream: Resource<streams::OutputStream>,
         isatty: IsATTY,
     },
-    /// A fd of type [`crate::filesystem::Descriptor::Dir`]
+    /// A fd of type [`crate::p2::filesystem::Descriptor::Dir`]
     Directory {
         fd: Resource<filesystem::Descriptor>,
         /// The path this directory was preopened as.
         /// `None` means this directory was opened using `open-at`.
         preopen_path: Option<String>,
     },
-    /// A fd of type [`crate::filesystem::Descriptor::File`]
+    /// A fd of type [`crate::p2::filesystem::Descriptor::File`]
     File(File),
 }
 
@@ -278,21 +285,7 @@ struct WasiPreview1Adapter {
 #[derive(Debug, Default)]
 struct Descriptors {
     used: BTreeMap<u32, Descriptor>,
-    free: Vec<u32>,
-}
-
-impl Deref for Descriptors {
-    type Target = BTreeMap<u32, Descriptor>;
-
-    fn deref(&self) -> &Self::Target {
-        &self.used
-    }
-}
-
-impl DerefMut for Descriptors {
-    fn deref_mut(&mut self) -> &mut Self::Target {
-        &mut self.used
-    }
+    free: BTreeSet<u32>,
 }
 
 impl Descriptors {
@@ -369,42 +362,34 @@ impl Descriptors {
 
     /// Returns next descriptor number, which was never assigned
     fn unused(&self) -> Result<u32> {
-        match self.last_key_value() {
+        match self.used.last_key_value() {
             Some((fd, _)) => {
                 if let Some(fd) = fd.checked_add(1) {
                     return Ok(fd);
                 }
-                if self.len() == u32::MAX as usize {
+                if self.used.len() == u32::MAX as usize {
                     return Err(types::Errno::Loop.into());
                 }
                 // TODO: Optimize
                 Ok((0..u32::MAX)
                     .rev()
-                    .find(|fd| !self.contains_key(fd))
+                    .find(|fd| !self.used.contains_key(fd))
                     .expect("failed to find an unused file descriptor"))
             }
             None => Ok(0),
         }
     }
 
-    /// Removes the [Descriptor] corresponding to `fd`
-    fn remove(&mut self, fd: types::Fd) -> Option<Descriptor> {
-        let fd = fd.into();
-        let desc = self.used.remove(&fd)?;
-        self.free.push(fd);
-        Some(desc)
-    }
-
     /// Pushes the [Descriptor] returning corresponding number.
     /// This operation will try to reuse numbers previously removed via [`Self::remove`]
     /// and rely on [`Self::unused`] if no free numbers are recorded
     fn push(&mut self, desc: Descriptor) -> Result<u32> {
-        let fd = if let Some(fd) = self.free.pop() {
+        let fd = if let Some(fd) = self.free.pop_last() {
             fd
         } else {
             self.unused()?
         };
-        assert!(self.insert(fd, desc).is_none());
+        assert!(self.used.insert(fd, desc).is_none());
         Ok(fd)
     }
 }
@@ -445,7 +430,7 @@ impl Transaction<'_> {
     /// Returns [`types::Errno::Badf`] if no [`Descriptor`] is found
     fn get_descriptor(&self, fd: types::Fd) -> Result<&Descriptor> {
         let fd = fd.into();
-        let desc = self.descriptors.get(&fd).ok_or(types::Errno::Badf)?;
+        let desc = self.descriptors.used.get(&fd).ok_or(types::Errno::Badf)?;
         Ok(desc)
     }
 
@@ -453,7 +438,7 @@ impl Transaction<'_> {
     /// if it describes a [`Descriptor::File`]
     fn get_file(&self, fd: types::Fd) -> Result<&File> {
         let fd = fd.into();
-        match self.descriptors.get(&fd) {
+        match self.descriptors.used.get(&fd) {
             Some(Descriptor::File(file)) => Ok(file),
             _ => Err(types::Errno::Badf.into()),
         }
@@ -463,7 +448,7 @@ impl Transaction<'_> {
     /// if it describes a [`Descriptor::File`]
     fn get_file_mut(&mut self, fd: types::Fd) -> Result<&mut File> {
         let fd = fd.into();
-        match self.descriptors.get_mut(&fd) {
+        match self.descriptors.used.get_mut(&fd) {
             Some(Descriptor::File(file)) => Ok(file),
             _ => Err(types::Errno::Badf.into()),
         }
@@ -477,7 +462,7 @@ impl Transaction<'_> {
     /// Returns [`types::Errno::Spipe`] if the descriptor corresponds to stdio
     fn get_seekable(&self, fd: types::Fd) -> Result<&File> {
         let fd = fd.into();
-        match self.descriptors.get(&fd) {
+        match self.descriptors.used.get(&fd) {
             Some(Descriptor::File(file)) => Ok(file),
             Some(
                 Descriptor::Stdin { .. } | Descriptor::Stdout { .. } | Descriptor::Stderr { .. },
@@ -510,7 +495,7 @@ impl Transaction<'_> {
     /// if it describes a [`Descriptor::Directory`]
     fn get_dir_fd(&self, fd: types::Fd) -> Result<Resource<filesystem::Descriptor>> {
         let fd = fd.into();
-        match self.descriptors.get(&fd) {
+        match self.descriptors.used.get(&fd) {
             Some(Descriptor::Directory { fd, .. }) => Ok(fd.borrowed()),
             _ => Err(types::Errno::Badf.into()),
         }
@@ -543,7 +528,7 @@ impl WasiP1Ctx {
 
     /// Lazily initializes [`WasiPreview1Adapter`] returned by [`WasiPreview1View::adapter_mut`]
     /// and returns [`filesystem::Descriptor`] corresponding to `fd`
-    /// if it describes a [`Descriptor::File`] of [`crate::filesystem::File`] type
+    /// if it describes a [`Descriptor::File`] of [`crate::p2::filesystem::File`] type
     fn get_file_fd(
         &mut self,
         fd: types::Fd,
@@ -556,7 +541,7 @@ impl WasiP1Ctx {
     /// Lazily initializes [`WasiPreview1Adapter`] returned by [`WasiPreview1View::adapter_mut`]
     /// and returns [`filesystem::Descriptor`] corresponding to `fd`
     /// if it describes a [`Descriptor::File`] or [`Descriptor::PreopenDirectory`]
-    /// of [`crate::filesystem::Dir`] type
+    /// of [`crate::p2::filesystem::Dir`] type
     fn get_dir_fd(
         &mut self,
         fd: types::Fd,
@@ -648,7 +633,7 @@ impl WasiP1Ctx {
                 drop(t);
                 let buf = first_non_empty_ciovec(memory, ciovs)?;
                 let n = BlockingMode::Blocking
-                    .write(memory, &mut self.as_wasi_impl(), stream, buf)
+                    .write(memory, &mut self.as_io_impl(), stream, buf)
                     .await?
                     .try_into()?;
                 Ok(n)
@@ -667,14 +652,16 @@ enum FdWrite {
 /// Adds asynchronous versions of all WASIp1 functions to the
 /// [`wasmtime::Linker`] provided.
 ///
-/// This method will add WASIp1 functions to `linker`. The `f` closure provided
-/// is used to project from the `T` state that `Linker` is associated with to a
-/// [`WasiP1Ctx`]. If `T` is `WasiP1Ctx` itself then this is the identity
-/// closure, but otherwise it must project out the field where `WasiP1Ctx` is
-/// stored within `T`.
+/// This method will add WASIp1 functions to `linker`. Access to [`WasiP1Ctx`]
+/// is provided with `f` by projecting from the store-local state of `T` to
+/// [`WasiP1Ctx`]. The closure `f` is invoked every time a WASIp1 function is
+/// called to get access to [`WASIp1`] from `T`. The returned [`WasiP1Ctx`] is
+/// used to implement I/O and controls what each function will return.
 ///
-/// The state provided by `f` is used to implement all WASIp1 functions and
-/// provides configuration to know what to return.
+/// It's recommended that [`WasiP1Ctx`] is stored as a field in `T` or that `T =
+/// WasiP1Ctx` itself. The closure `f` should be a small projection (e.g. `&mut
+/// arg.field`) or something otherwise "small" as it will be executed every time
+/// a WASI call is made.
 ///
 /// Note that this function is intended for use with
 /// [`Config::async_support(true)`]. If you're looking for a synchronous version
@@ -739,14 +726,16 @@ pub fn add_to_linker_async<T: Send>(
 /// Adds synchronous versions of all WASIp1 functions to the
 /// [`wasmtime::Linker`] provided.
 ///
-/// This method will add WASIp1 functions to `linker`. The `f` closure provided
-/// is used to project from the `T` state that `Linker` is associated with to a
-/// [`WasiP1Ctx`]. If `T` is `WasiP1Ctx` itself then this is the identity
-/// closure, but otherwise it must project out the field where `WasiP1Ctx` is
-/// stored within `T`.
+/// This method will add WASIp1 functions to `linker`. Access to [`WasiP1Ctx`]
+/// is provided with `f` by projecting from the store-local state of `T` to
+/// [`WasiP1Ctx`]. The closure `f` is invoked every time a WASIp1 function is
+/// called to get access to [`WASIp1`] from `T`. The returned [`WasiP1Ctx`] is
+/// used to implement I/O and controls what each function will return.
 ///
-/// The state provided by `f` is used to implement all WASIp1 functions and
-/// provides configuration to know what to return.
+/// It's recommended that [`WasiP1Ctx`] is stored as a field in `T` or that `T =
+/// WasiP1Ctx` itself. The closure `f` should be a small projection (e.g. `&mut
+/// arg.field`) or something otherwise "small" as it will be executed every time
+/// a WASI call is made.
 ///
 /// Note that this function is intended for use with
 /// [`Config::async_support(false)`]. If you're looking for a synchronous version
@@ -805,7 +794,7 @@ pub fn add_to_linker_sync<T: Send>(
     linker: &mut wasmtime::Linker<T>,
     f: impl Fn(&mut T) -> &mut WasiP1Ctx + Copy + Send + Sync + 'static,
 ) -> anyhow::Result<()> {
-    crate::preview1::sync::add_wasi_snapshot_preview1_to_linker(linker, f)
+    sync::add_wasi_snapshot_preview1_to_linker(linker, f)
 }
 
 // Generate the wasi_snapshot_preview1::WasiSnapshotPreview1 trait,
@@ -813,7 +802,7 @@ pub fn add_to_linker_sync<T: Send>(
 // None of the generated modules, traits, or types should be used externally
 // to this module.
 wiggle::from_witx!({
-    witx: ["$CARGO_MANIFEST_DIR/witx/preview1/wasi_snapshot_preview1.witx"],
+    witx: ["witx/preview1/wasi_snapshot_preview1.witx"],
     async: {
         wasi_snapshot_preview1::{
             fd_advise, fd_close, fd_datasync, fd_fdstat_get, fd_filestat_get, fd_filestat_set_size,
@@ -831,7 +820,7 @@ pub(crate) mod sync {
     use std::future::Future;
 
     wiggle::wasmtime_integration!({
-        witx: ["$CARGO_MANIFEST_DIR/witx/preview1/wasi_snapshot_preview1.witx"],
+        witx: ["witx/preview1/wasi_snapshot_preview1.witx"],
         target: super,
         block_on[in_tokio]: {
             wasi_snapshot_preview1::{
@@ -1316,19 +1305,21 @@ impl wasi_snapshot_preview1::WasiSnapshotPreview1 for WasiP1Ctx {
         _memory: &mut GuestMemory<'_>,
         fd: types::Fd,
     ) -> Result<(), types::Error> {
-        let desc = self
-            .transact()?
-            .descriptors
-            .remove(fd)
-            .ok_or(types::Errno::Badf)?;
+        let desc = {
+            let fd = fd.into();
+            let mut st = self.transact()?;
+            let desc = st.descriptors.used.remove(&fd).ok_or(types::Errno::Badf)?;
+            st.descriptors.free.insert(fd);
+            desc
+        };
         match desc {
             Descriptor::Stdin { stream, .. } => {
-                streams::HostInputStream::drop(&mut self.as_wasi_impl(), stream)
+                streams::HostInputStream::drop(&mut self.as_io_impl(), stream)
                     .await
                     .context("failed to call `drop` on `input-stream`")
             }
             Descriptor::Stdout { stream, .. } | Descriptor::Stderr { stream, .. } => {
-                streams::HostOutputStream::drop(&mut self.as_wasi_impl(), stream)
+                streams::HostOutputStream::drop(&mut self.as_io_impl(), stream)
                     .await
                     .context("failed to call `drop` on `output-stream`")
             }
@@ -1679,7 +1670,7 @@ impl wasi_snapshot_preview1::WasiSnapshotPreview1 for WasiP1Ctx {
                 drop(t);
                 let buf = first_non_empty_iovec(memory, iovs)?;
                 let read = BlockingMode::Blocking
-                    .read(&mut self.as_wasi_impl(), stream, buf.len().try_into()?)
+                    .read(&mut self.as_io_impl(), stream, buf.len().try_into()?)
                     .await?;
                 if read.len() > buf.len().try_into()? {
                     return Err(types::Errno::Range.into());
@@ -1717,12 +1708,12 @@ impl wasi_snapshot_preview1::WasiSnapshotPreview1 for WasiP1Ctx {
                 let stream = self.as_wasi_impl().read_via_stream(fd, offset)?;
                 let read = blocking_mode
                     .read(
-                        &mut self.as_wasi_impl(),
+                        &mut self.as_io_impl(),
                         stream.borrowed(),
                         buf.len().try_into()?,
                     )
                     .await;
-                streams::HostInputStream::drop(&mut self.as_wasi_impl(), stream)
+                streams::HostInputStream::drop(&mut self.as_io_impl(), stream)
                     .await
                     .map_err(|e| types::Error::trap(e))?;
                 (buf, read?)
@@ -1819,8 +1810,17 @@ impl wasi_snapshot_preview1::WasiSnapshotPreview1 for WasiP1Ctx {
         to: types::Fd,
     ) -> Result<(), types::Error> {
         let mut st = self.transact()?;
-        let desc = st.descriptors.remove(from).ok_or(types::Errno::Badf)?;
-        st.descriptors.insert(to.into(), desc);
+        let from = from.into();
+        let btree_map::Entry::Occupied(desc) = st.descriptors.used.entry(from) else {
+            return Err(types::Errno::Badf.into());
+        };
+        let to = to.into();
+        if from != to {
+            let desc = desc.remove();
+            st.descriptors.free.insert(from);
+            st.descriptors.free.remove(&to);
+            st.descriptors.used.insert(to, desc);
+        }
         Ok(())
     }
 
@@ -2146,11 +2146,11 @@ impl wasi_snapshot_preview1::WasiSnapshotPreview1 for WasiP1Ctx {
             .await?;
         let mut t = self.transact()?;
         let desc = match t.view.table().get(&fd)? {
-            crate::filesystem::Descriptor::Dir(_) => Descriptor::Directory {
+            crate::p2::filesystem::Descriptor::Dir(_) => Descriptor::Directory {
                 fd,
                 preopen_path: None,
             },
-            crate::filesystem::Descriptor::File(_) => Descriptor::File(File {
+            crate::p2::filesystem::Descriptor::File(_) => Descriptor::File(File {
                 fd,
                 position: Default::default(),
                 append: fdflags.contains(types::Fdflags::APPEND),
@@ -2367,7 +2367,7 @@ impl wasi_snapshot_preview1::WasiSnapshotPreview1 for WasiP1Ctx {
                             _ => return Err(types::Errno::Badf.into()),
                         }
                     };
-                    streams::HostInputStream::subscribe(&mut self.as_wasi_impl(), stream)
+                    streams::HostInputStream::subscribe(&mut self.as_io_impl(), stream)
                         .context("failed to call `subscribe` on `input-stream`")
                         .map_err(types::Error::trap)?
                 }
@@ -2401,7 +2401,7 @@ impl wasi_snapshot_preview1::WasiSnapshotPreview1 for WasiP1Ctx {
                             _ => return Err(types::Errno::Badf.into()),
                         }
                     };
-                    streams::HostOutputStream::subscribe(&mut self.as_wasi_impl(), stream)
+                    streams::HostOutputStream::subscribe(&mut self.as_io_impl(), stream)
                         .context("failed to call `subscribe` on `output-stream`")
                         .map_err(types::Error::trap)?
                 }
@@ -2409,7 +2409,7 @@ impl wasi_snapshot_preview1::WasiSnapshotPreview1 for WasiP1Ctx {
             pollables.push(p);
         }
         let ready: HashSet<_> = self
-            .as_wasi_impl()
+            .as_io_impl()
             .poll(pollables)
             .await
             .context("failed to call `poll-oneoff`")
@@ -2559,7 +2559,6 @@ impl wasi_snapshot_preview1::WasiSnapshotPreview1 for WasiP1Ctx {
         Ok(())
     }
 
-    #[allow(unused_variables)]
     #[instrument(skip(self, _memory))]
     fn sock_accept(
         &mut self,
@@ -2572,7 +2571,6 @@ impl wasi_snapshot_preview1::WasiSnapshotPreview1 for WasiP1Ctx {
         Err(types::Errno::Notsock.into())
     }
 
-    #[allow(unused_variables)]
     #[instrument(skip(self, _memory))]
     fn sock_recv(
         &mut self,
@@ -2586,7 +2584,6 @@ impl wasi_snapshot_preview1::WasiSnapshotPreview1 for WasiP1Ctx {
         Err(types::Errno::Notsock.into())
     }
 
-    #[allow(unused_variables)]
     #[instrument(skip(self, _memory))]
     fn sock_send(
         &mut self,
@@ -2600,7 +2597,6 @@ impl wasi_snapshot_preview1::WasiSnapshotPreview1 for WasiP1Ctx {
         Err(types::Errno::Notsock.into())
     }
 
-    #[allow(unused_variables)]
     #[instrument(skip(self, _memory))]
     fn sock_shutdown(
         &mut self,
