@@ -172,103 +172,88 @@ pub const DXGI_MODE_SCANLINE_ORDER_UPPER_FIELD_FIRST: DXGI_MODE_SCANLINE_ORDER =
 pub const DXGI_STANDARD_MULTISAMPLE_QUALITY_PATTERN: u32 = 4294967295u32;
 pub const _FACDXGI: u32 = 2170u32;
 #[repr(transparent)]
-#[derive(::core::cmp::PartialEq, ::core::cmp::Eq, ::core::marker::Copy, ::core::clone::Clone, ::core::default::Default)]
+#[derive(PartialEq, Eq, Copy, Clone, Default)]
 pub struct DXGI_ALPHA_MODE(pub i32);
-impl ::windows_core::TypeKind for DXGI_ALPHA_MODE {
-    type TypeKind = ::windows_core::CopyType;
+impl windows_core::TypeKind for DXGI_ALPHA_MODE {
+    type TypeKind = windows_core::CopyType;
 }
-impl ::core::fmt::Debug for DXGI_ALPHA_MODE {
-    fn fmt(&self, f: &mut ::core::fmt::Formatter<'_>) -> ::core::fmt::Result {
+impl core::fmt::Debug for DXGI_ALPHA_MODE {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         f.debug_tuple("DXGI_ALPHA_MODE").field(&self.0).finish()
     }
 }
 #[repr(transparent)]
-#[derive(::core::cmp::PartialEq, ::core::cmp::Eq, ::core::marker::Copy, ::core::clone::Clone, ::core::default::Default)]
+#[derive(PartialEq, Eq, Copy, Clone, Default)]
 pub struct DXGI_COLOR_SPACE_TYPE(pub i32);
-impl ::windows_core::TypeKind for DXGI_COLOR_SPACE_TYPE {
-    type TypeKind = ::windows_core::CopyType;
+impl windows_core::TypeKind for DXGI_COLOR_SPACE_TYPE {
+    type TypeKind = windows_core::CopyType;
 }
-impl ::core::fmt::Debug for DXGI_COLOR_SPACE_TYPE {
-    fn fmt(&self, f: &mut ::core::fmt::Formatter<'_>) -> ::core::fmt::Result {
+impl core::fmt::Debug for DXGI_COLOR_SPACE_TYPE {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         f.debug_tuple("DXGI_COLOR_SPACE_TYPE").field(&self.0).finish()
     }
 }
 #[repr(transparent)]
-#[derive(::core::cmp::PartialEq, ::core::cmp::Eq, ::core::marker::Copy, ::core::clone::Clone, ::core::default::Default)]
+#[derive(PartialEq, Eq, Copy, Clone, Default)]
 pub struct DXGI_FORMAT(pub i32);
-impl ::windows_core::TypeKind for DXGI_FORMAT {
-    type TypeKind = ::windows_core::CopyType;
+impl windows_core::TypeKind for DXGI_FORMAT {
+    type TypeKind = windows_core::CopyType;
 }
-impl ::core::fmt::Debug for DXGI_FORMAT {
-    fn fmt(&self, f: &mut ::core::fmt::Formatter<'_>) -> ::core::fmt::Result {
+impl core::fmt::Debug for DXGI_FORMAT {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         f.debug_tuple("DXGI_FORMAT").field(&self.0).finish()
     }
 }
 #[repr(transparent)]
-#[derive(::core::cmp::PartialEq, ::core::cmp::Eq, ::core::marker::Copy, ::core::clone::Clone, ::core::default::Default)]
+#[derive(PartialEq, Eq, Copy, Clone, Default)]
 pub struct DXGI_MODE_ROTATION(pub i32);
-impl ::windows_core::TypeKind for DXGI_MODE_ROTATION {
-    type TypeKind = ::windows_core::CopyType;
+impl windows_core::TypeKind for DXGI_MODE_ROTATION {
+    type TypeKind = windows_core::CopyType;
 }
-impl ::core::fmt::Debug for DXGI_MODE_ROTATION {
-    fn fmt(&self, f: &mut ::core::fmt::Formatter<'_>) -> ::core::fmt::Result {
+impl core::fmt::Debug for DXGI_MODE_ROTATION {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         f.debug_tuple("DXGI_MODE_ROTATION").field(&self.0).finish()
     }
 }
 #[repr(transparent)]
-#[derive(::core::cmp::PartialEq, ::core::cmp::Eq, ::core::marker::Copy, ::core::clone::Clone, ::core::default::Default)]
+#[derive(PartialEq, Eq, Copy, Clone, Default)]
 pub struct DXGI_MODE_SCALING(pub i32);
-impl ::windows_core::TypeKind for DXGI_MODE_SCALING {
-    type TypeKind = ::windows_core::CopyType;
+impl windows_core::TypeKind for DXGI_MODE_SCALING {
+    type TypeKind = windows_core::CopyType;
 }
-impl ::core::fmt::Debug for DXGI_MODE_SCALING {
-    fn fmt(&self, f: &mut ::core::fmt::Formatter<'_>) -> ::core::fmt::Result {
+impl core::fmt::Debug for DXGI_MODE_SCALING {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         f.debug_tuple("DXGI_MODE_SCALING").field(&self.0).finish()
     }
 }
 #[repr(transparent)]
-#[derive(::core::cmp::PartialEq, ::core::cmp::Eq, ::core::marker::Copy, ::core::clone::Clone, ::core::default::Default)]
+#[derive(PartialEq, Eq, Copy, Clone, Default)]
 pub struct DXGI_MODE_SCANLINE_ORDER(pub i32);
-impl ::windows_core::TypeKind for DXGI_MODE_SCANLINE_ORDER {
-    type TypeKind = ::windows_core::CopyType;
+impl windows_core::TypeKind for DXGI_MODE_SCANLINE_ORDER {
+    type TypeKind = windows_core::CopyType;
 }
-impl ::core::fmt::Debug for DXGI_MODE_SCANLINE_ORDER {
-    fn fmt(&self, f: &mut ::core::fmt::Formatter<'_>) -> ::core::fmt::Result {
+impl core::fmt::Debug for DXGI_MODE_SCANLINE_ORDER {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         f.debug_tuple("DXGI_MODE_SCANLINE_ORDER").field(&self.0).finish()
     }
 }
 #[repr(C)]
+#[derive(Clone, Copy, Debug, PartialEq)]
 pub struct DXGI_GAMMA_CONTROL {
     pub Scale: DXGI_RGB,
     pub Offset: DXGI_RGB,
     pub GammaCurve: [DXGI_RGB; 1025],
 }
-impl ::core::marker::Copy for DXGI_GAMMA_CONTROL {}
-impl ::core::clone::Clone for DXGI_GAMMA_CONTROL {
-    fn clone(&self) -> Self {
-        *self
-    }
+impl windows_core::TypeKind for DXGI_GAMMA_CONTROL {
+    type TypeKind = windows_core::CopyType;
 }
-impl ::core::fmt::Debug for DXGI_GAMMA_CONTROL {
-    fn fmt(&self, f: &mut ::core::fmt::Formatter<'_>) -> ::core::fmt::Result {
-        f.debug_struct("DXGI_GAMMA_CONTROL").field("Scale", &self.Scale).field("Offset", &self.Offset).field("GammaCurve", &self.GammaCurve).finish()
-    }
-}
-impl ::windows_core::TypeKind for DXGI_GAMMA_CONTROL {
-    type TypeKind = ::windows_core::CopyType;
-}
-impl ::core::cmp::PartialEq for DXGI_GAMMA_CONTROL {
-    fn eq(&self, other: &Self) -> bool {
-        self.Scale == other.Scale && self.Offset == other.Offset && self.GammaCurve == other.GammaCurve
-    }
-}
-impl ::core::cmp::Eq for DXGI_GAMMA_CONTROL {}
-impl ::core::default::Default for DXGI_GAMMA_CONTROL {
+impl Default for DXGI_GAMMA_CONTROL {
     fn default() -> Self {
-        unsafe { ::core::mem::zeroed() }
+        unsafe { core::mem::zeroed() }
     }
 }
 #[repr(C)]
+#[derive(Clone, Copy, Debug, PartialEq)]
 pub struct DXGI_GAMMA_CONTROL_CAPABILITIES {
     pub ScaleAndOffsetSupported: super::super::super::Foundation::BOOL,
     pub MaxConvertedValue: f32,
@@ -276,121 +261,57 @@ pub struct DXGI_GAMMA_CONTROL_CAPABILITIES {
     pub NumGammaControlPoints: u32,
     pub ControlPointPositions: [f32; 1025],
 }
-impl ::core::marker::Copy for DXGI_GAMMA_CONTROL_CAPABILITIES {}
-impl ::core::clone::Clone for DXGI_GAMMA_CONTROL_CAPABILITIES {
-    fn clone(&self) -> Self {
-        *self
-    }
+impl windows_core::TypeKind for DXGI_GAMMA_CONTROL_CAPABILITIES {
+    type TypeKind = windows_core::CopyType;
 }
-impl ::core::fmt::Debug for DXGI_GAMMA_CONTROL_CAPABILITIES {
-    fn fmt(&self, f: &mut ::core::fmt::Formatter<'_>) -> ::core::fmt::Result {
-        f.debug_struct("DXGI_GAMMA_CONTROL_CAPABILITIES").field("ScaleAndOffsetSupported", &self.ScaleAndOffsetSupported).field("MaxConvertedValue", &self.MaxConvertedValue).field("MinConvertedValue", &self.MinConvertedValue).field("NumGammaControlPoints", &self.NumGammaControlPoints).field("ControlPointPositions", &self.ControlPointPositions).finish()
-    }
-}
-impl ::windows_core::TypeKind for DXGI_GAMMA_CONTROL_CAPABILITIES {
-    type TypeKind = ::windows_core::CopyType;
-}
-impl ::core::cmp::PartialEq for DXGI_GAMMA_CONTROL_CAPABILITIES {
-    fn eq(&self, other: &Self) -> bool {
-        self.ScaleAndOffsetSupported == other.ScaleAndOffsetSupported && self.MaxConvertedValue == other.MaxConvertedValue && self.MinConvertedValue == other.MinConvertedValue && self.NumGammaControlPoints == other.NumGammaControlPoints && self.ControlPointPositions == other.ControlPointPositions
-    }
-}
-impl ::core::cmp::Eq for DXGI_GAMMA_CONTROL_CAPABILITIES {}
-impl ::core::default::Default for DXGI_GAMMA_CONTROL_CAPABILITIES {
+impl Default for DXGI_GAMMA_CONTROL_CAPABILITIES {
     fn default() -> Self {
-        unsafe { ::core::mem::zeroed() }
+        unsafe { core::mem::zeroed() }
     }
 }
 #[repr(C)]
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub struct DXGI_JPEG_AC_HUFFMAN_TABLE {
     pub CodeCounts: [u8; 16],
     pub CodeValues: [u8; 162],
 }
-impl ::core::marker::Copy for DXGI_JPEG_AC_HUFFMAN_TABLE {}
-impl ::core::clone::Clone for DXGI_JPEG_AC_HUFFMAN_TABLE {
-    fn clone(&self) -> Self {
-        *self
-    }
+impl windows_core::TypeKind for DXGI_JPEG_AC_HUFFMAN_TABLE {
+    type TypeKind = windows_core::CopyType;
 }
-impl ::core::fmt::Debug for DXGI_JPEG_AC_HUFFMAN_TABLE {
-    fn fmt(&self, f: &mut ::core::fmt::Formatter<'_>) -> ::core::fmt::Result {
-        f.debug_struct("DXGI_JPEG_AC_HUFFMAN_TABLE").field("CodeCounts", &self.CodeCounts).field("CodeValues", &self.CodeValues).finish()
-    }
-}
-impl ::windows_core::TypeKind for DXGI_JPEG_AC_HUFFMAN_TABLE {
-    type TypeKind = ::windows_core::CopyType;
-}
-impl ::core::cmp::PartialEq for DXGI_JPEG_AC_HUFFMAN_TABLE {
-    fn eq(&self, other: &Self) -> bool {
-        self.CodeCounts == other.CodeCounts && self.CodeValues == other.CodeValues
-    }
-}
-impl ::core::cmp::Eq for DXGI_JPEG_AC_HUFFMAN_TABLE {}
-impl ::core::default::Default for DXGI_JPEG_AC_HUFFMAN_TABLE {
+impl Default for DXGI_JPEG_AC_HUFFMAN_TABLE {
     fn default() -> Self {
-        unsafe { ::core::mem::zeroed() }
+        unsafe { core::mem::zeroed() }
     }
 }
 #[repr(C)]
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub struct DXGI_JPEG_DC_HUFFMAN_TABLE {
     pub CodeCounts: [u8; 12],
     pub CodeValues: [u8; 12],
 }
-impl ::core::marker::Copy for DXGI_JPEG_DC_HUFFMAN_TABLE {}
-impl ::core::clone::Clone for DXGI_JPEG_DC_HUFFMAN_TABLE {
-    fn clone(&self) -> Self {
-        *self
-    }
+impl windows_core::TypeKind for DXGI_JPEG_DC_HUFFMAN_TABLE {
+    type TypeKind = windows_core::CopyType;
 }
-impl ::core::fmt::Debug for DXGI_JPEG_DC_HUFFMAN_TABLE {
-    fn fmt(&self, f: &mut ::core::fmt::Formatter<'_>) -> ::core::fmt::Result {
-        f.debug_struct("DXGI_JPEG_DC_HUFFMAN_TABLE").field("CodeCounts", &self.CodeCounts).field("CodeValues", &self.CodeValues).finish()
-    }
-}
-impl ::windows_core::TypeKind for DXGI_JPEG_DC_HUFFMAN_TABLE {
-    type TypeKind = ::windows_core::CopyType;
-}
-impl ::core::cmp::PartialEq for DXGI_JPEG_DC_HUFFMAN_TABLE {
-    fn eq(&self, other: &Self) -> bool {
-        self.CodeCounts == other.CodeCounts && self.CodeValues == other.CodeValues
-    }
-}
-impl ::core::cmp::Eq for DXGI_JPEG_DC_HUFFMAN_TABLE {}
-impl ::core::default::Default for DXGI_JPEG_DC_HUFFMAN_TABLE {
+impl Default for DXGI_JPEG_DC_HUFFMAN_TABLE {
     fn default() -> Self {
-        unsafe { ::core::mem::zeroed() }
+        unsafe { core::mem::zeroed() }
     }
 }
 #[repr(C)]
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub struct DXGI_JPEG_QUANTIZATION_TABLE {
     pub Elements: [u8; 64],
 }
-impl ::core::marker::Copy for DXGI_JPEG_QUANTIZATION_TABLE {}
-impl ::core::clone::Clone for DXGI_JPEG_QUANTIZATION_TABLE {
-    fn clone(&self) -> Self {
-        *self
-    }
+impl windows_core::TypeKind for DXGI_JPEG_QUANTIZATION_TABLE {
+    type TypeKind = windows_core::CopyType;
 }
-impl ::core::fmt::Debug for DXGI_JPEG_QUANTIZATION_TABLE {
-    fn fmt(&self, f: &mut ::core::fmt::Formatter<'_>) -> ::core::fmt::Result {
-        f.debug_struct("DXGI_JPEG_QUANTIZATION_TABLE").field("Elements", &self.Elements).finish()
-    }
-}
-impl ::windows_core::TypeKind for DXGI_JPEG_QUANTIZATION_TABLE {
-    type TypeKind = ::windows_core::CopyType;
-}
-impl ::core::cmp::PartialEq for DXGI_JPEG_QUANTIZATION_TABLE {
-    fn eq(&self, other: &Self) -> bool {
-        self.Elements == other.Elements
-    }
-}
-impl ::core::cmp::Eq for DXGI_JPEG_QUANTIZATION_TABLE {}
-impl ::core::default::Default for DXGI_JPEG_QUANTIZATION_TABLE {
+impl Default for DXGI_JPEG_QUANTIZATION_TABLE {
     fn default() -> Self {
-        unsafe { ::core::mem::zeroed() }
+        unsafe { core::mem::zeroed() }
     }
 }
 #[repr(C)]
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub struct DXGI_MODE_DESC {
     pub Width: u32,
     pub Height: u32,
@@ -399,119 +320,54 @@ pub struct DXGI_MODE_DESC {
     pub ScanlineOrdering: DXGI_MODE_SCANLINE_ORDER,
     pub Scaling: DXGI_MODE_SCALING,
 }
-impl ::core::marker::Copy for DXGI_MODE_DESC {}
-impl ::core::clone::Clone for DXGI_MODE_DESC {
-    fn clone(&self) -> Self {
-        *self
-    }
+impl windows_core::TypeKind for DXGI_MODE_DESC {
+    type TypeKind = windows_core::CopyType;
 }
-impl ::core::fmt::Debug for DXGI_MODE_DESC {
-    fn fmt(&self, f: &mut ::core::fmt::Formatter<'_>) -> ::core::fmt::Result {
-        f.debug_struct("DXGI_MODE_DESC").field("Width", &self.Width).field("Height", &self.Height).field("RefreshRate", &self.RefreshRate).field("Format", &self.Format).field("ScanlineOrdering", &self.ScanlineOrdering).field("Scaling", &self.Scaling).finish()
-    }
-}
-impl ::windows_core::TypeKind for DXGI_MODE_DESC {
-    type TypeKind = ::windows_core::CopyType;
-}
-impl ::core::cmp::PartialEq for DXGI_MODE_DESC {
-    fn eq(&self, other: &Self) -> bool {
-        self.Width == other.Width && self.Height == other.Height && self.RefreshRate == other.RefreshRate && self.Format == other.Format && self.ScanlineOrdering == other.ScanlineOrdering && self.Scaling == other.Scaling
-    }
-}
-impl ::core::cmp::Eq for DXGI_MODE_DESC {}
-impl ::core::default::Default for DXGI_MODE_DESC {
+impl Default for DXGI_MODE_DESC {
     fn default() -> Self {
-        unsafe { ::core::mem::zeroed() }
+        unsafe { core::mem::zeroed() }
     }
 }
 #[repr(C)]
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub struct DXGI_RATIONAL {
     pub Numerator: u32,
     pub Denominator: u32,
 }
-impl ::core::marker::Copy for DXGI_RATIONAL {}
-impl ::core::clone::Clone for DXGI_RATIONAL {
-    fn clone(&self) -> Self {
-        *self
-    }
+impl windows_core::TypeKind for DXGI_RATIONAL {
+    type TypeKind = windows_core::CopyType;
 }
-impl ::core::fmt::Debug for DXGI_RATIONAL {
-    fn fmt(&self, f: &mut ::core::fmt::Formatter<'_>) -> ::core::fmt::Result {
-        f.debug_struct("DXGI_RATIONAL").field("Numerator", &self.Numerator).field("Denominator", &self.Denominator).finish()
-    }
-}
-impl ::windows_core::TypeKind for DXGI_RATIONAL {
-    type TypeKind = ::windows_core::CopyType;
-}
-impl ::core::cmp::PartialEq for DXGI_RATIONAL {
-    fn eq(&self, other: &Self) -> bool {
-        self.Numerator == other.Numerator && self.Denominator == other.Denominator
-    }
-}
-impl ::core::cmp::Eq for DXGI_RATIONAL {}
-impl ::core::default::Default for DXGI_RATIONAL {
+impl Default for DXGI_RATIONAL {
     fn default() -> Self {
-        unsafe { ::core::mem::zeroed() }
+        unsafe { core::mem::zeroed() }
     }
 }
 #[repr(C)]
+#[derive(Clone, Copy, Debug, PartialEq)]
 pub struct DXGI_RGB {
     pub Red: f32,
     pub Green: f32,
     pub Blue: f32,
 }
-impl ::core::marker::Copy for DXGI_RGB {}
-impl ::core::clone::Clone for DXGI_RGB {
-    fn clone(&self) -> Self {
-        *self
-    }
+impl windows_core::TypeKind for DXGI_RGB {
+    type TypeKind = windows_core::CopyType;
 }
-impl ::core::fmt::Debug for DXGI_RGB {
-    fn fmt(&self, f: &mut ::core::fmt::Formatter<'_>) -> ::core::fmt::Result {
-        f.debug_struct("DXGI_RGB").field("Red", &self.Red).field("Green", &self.Green).field("Blue", &self.Blue).finish()
-    }
-}
-impl ::windows_core::TypeKind for DXGI_RGB {
-    type TypeKind = ::windows_core::CopyType;
-}
-impl ::core::cmp::PartialEq for DXGI_RGB {
-    fn eq(&self, other: &Self) -> bool {
-        self.Red == other.Red && self.Green == other.Green && self.Blue == other.Blue
-    }
-}
-impl ::core::cmp::Eq for DXGI_RGB {}
-impl ::core::default::Default for DXGI_RGB {
+impl Default for DXGI_RGB {
     fn default() -> Self {
-        unsafe { ::core::mem::zeroed() }
+        unsafe { core::mem::zeroed() }
     }
 }
 #[repr(C)]
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub struct DXGI_SAMPLE_DESC {
     pub Count: u32,
     pub Quality: u32,
 }
-impl ::core::marker::Copy for DXGI_SAMPLE_DESC {}
-impl ::core::clone::Clone for DXGI_SAMPLE_DESC {
-    fn clone(&self) -> Self {
-        *self
-    }
+impl windows_core::TypeKind for DXGI_SAMPLE_DESC {
+    type TypeKind = windows_core::CopyType;
 }
-impl ::core::fmt::Debug for DXGI_SAMPLE_DESC {
-    fn fmt(&self, f: &mut ::core::fmt::Formatter<'_>) -> ::core::fmt::Result {
-        f.debug_struct("DXGI_SAMPLE_DESC").field("Count", &self.Count).field("Quality", &self.Quality).finish()
-    }
-}
-impl ::windows_core::TypeKind for DXGI_SAMPLE_DESC {
-    type TypeKind = ::windows_core::CopyType;
-}
-impl ::core::cmp::PartialEq for DXGI_SAMPLE_DESC {
-    fn eq(&self, other: &Self) -> bool {
-        self.Count == other.Count && self.Quality == other.Quality
-    }
-}
-impl ::core::cmp::Eq for DXGI_SAMPLE_DESC {}
-impl ::core::default::Default for DXGI_SAMPLE_DESC {
+impl Default for DXGI_SAMPLE_DESC {
     fn default() -> Self {
-        unsafe { ::core::mem::zeroed() }
+        unsafe { core::mem::zeroed() }
     }
 }
